<?php
c_include('api/apiAdaptador');
/**
* Eventos Api Adapter class for the Mideas Framework.
*
* @package		Mideas.Framework
* @subpackage	Api
* @since		1.0
*/
class ShopApiAdaptador extends ApiAdaptador
{
    protected $extension = 'shop';
    protected $carpeta = 'ext_shop';
    protected $elementos_permitidos = array("producto", "cliente", "factura", "pedido");
    protected $campos_ignorados_nuevo = array("id", "fecha", "numero_valoraciones", "media_valoracion", "visitas");
    protected $campos_multilang = array("titulo", "contenido", "lugar");
    public function __construct($elemento, $accion)
    {
        parent::__construct($elemento, $accion);
        $this->mensajes = array(
            "nuevo_ok" => __("Se ha creado el producto. Haz click <a href='%s'>aquí</a> para ver el nuevo producto."),
            "nuevo_ko" => __("Se ha producido un error al intentar crear el producto"),
            "editar_ok" => __("Se ha modificado el producto. Haz click <a href='%s'>aquí</a> para visitar la página del producto."),
            "borrar_ok" => __("El producto se ha eliminado.")
        );
    }
	public function getRecalcularStock()
    {
        if ($this->get('elemento') != 'producto') {
            return $this->error(__('Acceso incorrecto'));
        }
        if (empty($_POST['accion']) || $_POST['accion'] != 'recalcular_stock') {
            return $this->error(__('Acción incorrecta'));
        }
		if (empty($_POST['token']) || Helpers::getFormToken() != $_POST['token']){
			return $this->error(__('Token incorrecto'));
		}
        e_include('ext_shop/class/producto', 'ext_shop/models/producto');
        $productos = new Producto();
		$productos = $productos->Listar();
		$error = false;
		$modificados = 0;
		foreach ($productos as $pr){
			$pm = new ProductoModel($pr->id, false);
				
			if (sizeof($pm->variaciones_atributos) > 0 && is_array($pm->variaciones_atributos)){
				$val_tmp = 0;
				foreach ($pm->variaciones_atributos as $va){
					$val_tmp += $va->stock;
				}
				$pr->setstock($val_tmp);
				if (!$pr->Modificar()){
					$error = true;
				} else {
					$modificados ++;
				}
			}
		}
		
        if (!$error) {
            $data = array(
                'num_productos_modificados' => $modificados
            );
            return $this->arrResp(__('Stock de todos los productos recalculado con éxito'), $data);
        } else {
            return $this->error(__('Se ha producido un error al intentar recalcular el stock de todos los productos'));
        }
    }
    public function getCargarDatos()
    {
        if ($this->get('elemento') != 'cliente') {
            return $this->error(__('Acceso incorrecto'));
        }
        if (empty($_POST['id'])) {
            return $this->error(__('No se ha definido el id del cliente'));
        }
        e_include('ext_shop/class/cliente');
        $cliente = new ClienteShop();
        $cliente->setid($_POST['id']);
        if ($cliente = $cliente->BuscarId()) {
            $data = array(
                'id' => $cliente->id,
                'facturacion' => (array) $cliente->facturacion,
                'envio' => (array) $cliente->envio
            );
            return $this->arrResp(__('Datos del cliente recuperados con éxito'), $data);
        } else {
            return $this->error(__('No se ha definido el cliente'));
        }
    }
    public function getBuscar($campo_nombre = 'titulo')
    {
        return parent::getBuscar('nombre');
    }
    public function getGenerarFacturasPeriodicas()
    {
        if ($this->get('elemento') != 'factura') {
            return $this->error(__('Acceso incorrecto'));
        }
        if ($_POST['token'] != Helpers::getFormToken()) {
            return $this->error(__('Token no válido'));
        }
        if (!Usuario::isLoged()) {
            return $this->error(__('Acceso no permitido'));
        }
        $usr_logged = Usuario::getLoged();
        if ($usr_logged->es_admin != '1') {
            return $this->error(__('Acceso no permitido. Administrador requerido.'));
        }
        e_include('ext_shop/class/facturaShop');
        $where_periodicas = "params LIKE '%\"periodica\":\"1\"%'";
        $facturas         = new FacturaShop();
        $facturas         = $facturas->Listar($where_periodicas, "fecha", "ASC");
        if (!is_array($facturas) || sizeof($facturas) == 0) {
            return $this->error(__('No se han encontrado facturas periódicas pendientes de generar'));
        }
        $hay_pendientes       = false;
        $num_pendientes_total = 0;
        foreach ($facturas as $f) {
            $pendientes = $f->getPeriodicasPendientes();
            if ($pendientes) {
                $hay_pendientes = true;
                $num_pendientes_total += $pendientes;
            }
        }
        if (!$hay_pendientes) {
            return $this->error(__('No hay facturas pendientes de generar'));
        }
        $data = array(
            'num_facturas' => $num_pendientes_total
        );
        return $this->arrResp(sprintf(__('Se han generado %s facturas periódicas'), $num_pendientes_total), $data);
    }
    public function getGenerarInforme()
    {
        if ($this->get('elemento') != 'factura') {
            return $this->error(__('Acceso incorrecto'));
        }
        if ($_POST['token'] != Helpers::getFormToken()) {
            return $this->error(__('Token no válido'));
        }
        if (!Usuario::isLoged()) {
            return $this->error(__('Acceso no permitido'));
        }
        $usr_logged = Usuario::getLoged();
        if ($usr_logged->es_admin != '1') {
            return $this->error(__('Acceso no permitido. Administrador requerido.'));
        }
        e_include('ext_shop/class/facturaShop');
        $where_informe = "";
        if (!empty($_POST['fecha_inicio'])) {
            $where_informe = "fecha >= '" . strftime("%Y-%m-%d 00:00:00", strtotime($_POST['fecha_inicio'])) . "'";
        }
        if (!empty($_POST['fecha_fin'])) {
            if (empty($where_informe)) {
                $where_informe = "fecha <= '" . strftime("%Y-%m-%d 23:59:59", strtotime($_POST['fecha_fin'])) . "'";
            } else {
                $where_informe .= " AND fecha <= '" . strftime("%Y-%m-%d 23:59:59", strtotime($_POST['fecha_fin'])) . "'";
            }
        }
        if (!empty($_POST['cliente'])) {
            if (empty($where_informe)) {
                $where_informe = "datos_facturacion LIKE '%\"cliente_id\":\"" . Helpers::mres($_POST['cliente']) . "\"%'";
            } else {
                $where_informe .= " AND datos_facturacion LIKE '%\"cliente_id\":\"" . Helpers::mres($_POST['cliente']) . "\"%'";
            }
        }
        if (!empty($_POST['estado'])) {
            if (empty($where_informe)) {
                $where_informe = "estado='" . Helpers::mres($_POST['estado']) . "'";
            } else {
                $where_informe .= " AND estado='" . Helpers::mres($_POST['estado']) . "'";
            }
        }
        $facturas                   = new FacturaShop();
        $facturas                   = $facturas->Listar($where_informe, "fecha", "ASC");
        $totales['base_imponible']  = 0;
        $totales['total_descuento'] = 0;
        $totales['total_impuestos'] = 0;
        $totales['total_recargo']   = 0;
        $totales['total_retencion'] = 0;
        $totales['total']           = 0;
        if (is_array($facturas) && sizeof($facturas) > 0) {
            $data = array();
            foreach ($facturas as $f) {
                $f->loadLineas();
                $lineas_f = array();
                foreach ($f->lineas as $lin) {
                    $lineas_f[] = Helpers::toArray($lin);
                }
                $f_parsed           = Helpers::toArray($f);
                $f_parsed['lineas'] = $lineas_f;
                $data['facturas'][] = $f_parsed;
                $totales['base_imponible'] += $f->base_imponible;
                $totales['total_descuento'] += $f->total_descuento;
                $totales['total_impuestos'] += $f->total_impuesto;
                $totales['total_recargo'] += $f->total_recargo;
                $totales['total_retencion'] += $f->total_retencion;
                $totales['total'] += $f->total;
            }
            $data['totales'] = $totales;
            Helpers::getSesion()->set('ext_shop.informe_facturas', $data);
            return $this->arrResp(__('Informe de facturas generado con éxito'), $data);
        } else {
            $data = array();
            return $this->arrResp(__('No se han encontrado facturas'), $data);
        }
    }
    public function getMostrarInforme()
    {
        if ($this->get('elemento') != 'factura') {
            return $this->error(__('Acceso incorrecto'));
        }
        $informe_facturas = Helpers::getSesion()->get('ext_shop.informe_facturas');
        e_include('ext_shop/class/helper');
        if (!empty($informe_facturas['facturas'])) {
            Helpers::getSesion()->clear('ext_shop.informe_facturas');
            echo '<h4>' . __('Totales') . '</h4>
			<div class="totales-informe-facturas">
			<div class="row">
			<div class="col-sm-6">
			<ul class="list-group">
			<li class="list-group-item"><span class="badge">' . number_format($informe_facturas['totales']['base_imponible'], 2, ',', '.') . ' ' . ShopHelper::get_shop_moneda_simbolo() . '</span><b>' . __('Base imponible:') . '</b></li>
			<li class="list-group-item"><span class="badge">' . number_format($informe_facturas['totales']['total_impuestos'], 2, ',', '.') . ' ' . ShopHelper::get_shop_moneda_simbolo() . '</span><b>' . __('Total impuestos:') . '</b></li>
			<li class="list-group-item"><span class="badge">' . number_format($informe_facturas['totales']['total'], 2, ',', '.') . ' ' . ShopHelper::get_shop_moneda_simbolo() . '</span><b>' . __('Total:') . '</b></li>
			</ul>
			</div>
			<div class="col-sm-6">
			<ul class="list-group">
			<li class="list-group-item"><span class="badge">' . number_format($informe_facturas['totales']['total_descuento'], 2, ',', '.') . ' ' . ShopHelper::get_shop_moneda_simbolo() . '</span><b>' . __('Total descuentos:') . '</b></li>
			<li class="list-group-item"><span class="badge">' . number_format($informe_facturas['totales']['total_recargo'], 2, ',', '.') . ' ' . ShopHelper::get_shop_moneda_simbolo() . '</span><b>' . __('Total recargo:') . '</b></li>
			<li class="list-group-item"><span class="badge">' . number_format($informe_facturas['totales']['total_retencion'], 2, ',', '.') . ' ' . ShopHelper::get_shop_moneda_simbolo() . '</span><b>' . __('Total retención:') . '</b></li>
			</ul>
			</div>
			</div>
			</div>';
			
            echo '<h4>' . __('Facturas') . '</h4>
			<table class="table">
			<thead>
			<th>' . __('Cód.') . '</th>
			<th>' . __('Fecha') . '</th>
			<th>' . __('Cliente') . '</th>
			<th style="text-align: right;">' . __('Base imponible') . '</th>
			<th style="text-align: right;">' . __('Impuestos') . '</th>
			<th style="text-align: right;">' . __('Retención') . '</th>
			<th style="text-align: right;">' . __('Total') . '</th>
			</thead>
			<tbody>';
            foreach ($informe_facturas['facturas'] as $f) {
                $cliente_f = Helpers::toArray(json_decode($f['datos_facturacion']));
                echo '<tr><td>' . $f['codigo'] . '</td><td>' . strftime(SistemaIdioma::formatoFecha('corta'), strtotime($f['fecha'])) . '</td><td>' . $cliente_f['nombre'] . ' ' . $cliente_f['apellidos'] . ' (' . __('NIF:') . ' ' . $cliente_f['nif'] . ')</td><td align="right">' . number_format($f['base_imponible'], 2, ',', '.') . ' ' . ShopHelper::get_shop_moneda_simbolo() . '</td><td align="right">' . number_format($f['total_impuesto'], 2, ',', '.') . ' ' . ShopHelper::get_shop_moneda_simbolo() . '</td><td align="right">' . number_format($f['total_retencion'], 2, ',', '.') . ' ' . ShopHelper::get_shop_moneda_simbolo() . '</td><td align="right">' . number_format($f['total'], 2, ',', '.') . ' ' . ShopHelper::get_shop_moneda_simbolo() . '</td></tr>';
            }
            echo '</tbody></table>';
            //return $this->arrResp(__('Informe de facturas generado con éxito'), $informe_facturas);
        } else {
            echo '<pre>' . __('No se han encontrado facturas.') . '</pre>';
            //return $this->error(__('No se ha generado ningún informe de facturas'));
        }
        exit;
    }
	function getUpdateVarFactura(){
		if ($this->get('elemento') != 'factura') {
            return $this->error(__('Acceso incorrecto'));
        }
		if ($_POST['token'] != Helpers::getFormToken()) {
            return $this->error(__('Token no válido'));
        }
        if (!Usuario::isLoged()) {
            return $this->error(__('Acceso no permitido'));
        }
        $usr_logged = Usuario::getLoged();
        if ($usr_logged->es_admin != '1') {
            return $this->error(__('Acceso no permitido. Administrador requerido.'));
        }
		if (empty($_POST['id'])){
			return $this->error(__('No se ha definido el id de la factura.'));
		}
		if (empty($_POST['campo']) || empty($_POST['valor'])){
			return $this->error(__('Falta algún campo requerido.'));
		}
        e_include('ext_shop/class/facturaShop');
		$f = new FacturaShop();
		$f->setid($_POST['id']);
		if ($f = $f->BuscarId()){
			if ($f->UpdateField($_POST['campo'], $_POST['valor'])){
				$data = array("campo"=>$_POST['campo'], "valor"=>$_POST['valor']);
           		return $this->arrResp(__('Factura actualizada con éxito'), $data);
			} else {
				return $this->error(__('No se ha podido actualizar la factura.'));
			}
		} else {
			return $this->error(__('No se ha encontrado la factura.'));
		}
	}
	function getUpdateVarPedido(){
		if ($this->get('elemento') != 'pedido') {
            return $this->error(__('Acceso incorrecto'));
        }
		if ($_POST['token'] != Helpers::getFormToken()) {
            return $this->error(__('Token no válido'));
        }
        if (!Usuario::isLoged()) {
            return $this->error(__('Acceso no permitido'));
        }
        $usr_logged = Usuario::getLoged();
        if ($usr_logged->es_admin != '1') {
            return $this->error(__('Acceso no permitido. Administrador requerido.'));
        }
		if (empty($_POST['id'])){
			return $this->error(__('No se ha definido el id del pedido.'));
		}
		if (empty($_POST['campo']) || empty($_POST['valor'])){
			return $this->error(__('Falta algún campo requerido.'));
		}
        e_include('ext_shop/class/pedidoShop');
		$p = new PedidoShop();
		$p->setid($_POST['id']);
		if ($p = $p->BuscarId()){
			if ($_POST['campo'] != 'estado'){
				if ($p->UpdateField($_POST['campo'], $_POST['valor'])){
					$data = array("campo"=>$_POST['campo'], "valor"=>$_POST['valor']);
					return $this->arrResp(__('Pedido actualizado con éxito'), $data);
				} else {
					return $this->error(__('No se ha podido actualizar el pedido.'));
				}
			} else {
				
				$estado_old = $p->estado;
				$estado_new = $_POST["valor"];
				
				$estado_new_obj = new EstadoPedidoShop($estado_new);
				
				if ($estado_old != $estado_new && $estado_old == $GLOBALS['opciones']['ext_shop']['venta']['estado_completado']){
					if ($p->add_nota_pedido(SistemaIdioma::Imprimir($estado_new_obj->nombre) . ': ' . strftime("%d-%m-%Y %H:%M"), 0, $estado_new)){
						$p = new PedidoShop();
						$p->setid($_POST["id"]);
						$p = $p->BuscarId();
					}
					$p->setestado($estado_new);
					$p->Modificar();
					
					$p->restaurar_stock_pedido();
					
					$data = array("campo"=>$_POST['campo'], "valor"=>$_POST['valor']);
					return $this->arrResp(__('Pedido actualizado con éxito'), $data);				
				} elseif ($estado_old != $estado_new){
					if ($p->add_nota_pedido(SistemaIdioma::Imprimir($estado_new_obj->nombre) . ': ' . strftime("%d-%m-%Y %H:%M"), 0, $estado_new)){
						$p = new PedidoShop();
						$p->setid($_POST["id"]);
						$p = $p->BuscarId();
					}
					
					$notificar = $GLOBALS['opciones']['ext_shop']['notificaciones']['notificar'];
					$notificables = array(	'nuevo'=>$GLOBALS['opciones']['ext_shop']['venta']['estado_creado'],
											'en_proceso'=>$GLOBALS['opciones']['ext_shop']['venta']['estado_procesando'],
											'enviado'=>$GLOBALS['opciones']['ext_shop']['venta']['estado_enviado'],
											'completado'=>$GLOBALS['opciones']['ext_shop']['venta']['estado_completado'],
											'fallido'=>$GLOBALS['opciones']['ext_shop']['venta']['estado_fallido'],
											'cancelado'=>$GLOBALS['opciones']['ext_shop']['venta']['estado_cancelado']);
											
					foreach ($notificables as $kn=>$n){
						if ($estado_new == $n && in_array($kn, $notificar)){
							$p->enviar_email_pedido($kn);
						}
					}
					
					if ($estado_new == $GLOBALS['opciones']['ext_shop']['venta']['estado_completado']){
						$p->pago_completo($estado_new);
						$data = array("campo"=>$_POST['campo'], "valor"=>$_POST['valor']);
						return $this->arrResp(__('Pedido actualizado con éxito'), $data);
					}
					$p->setestado($estado_new);
					$p->Modificar();
				} else {
					$p->setestado($estado_new);
					$p->Modificar();
				}
				
				$data = array("campo"=>$_POST['campo'], "valor"=>$_POST['valor']);
				return $this->arrResp(__('Pedido actualizado con éxito'), $data);
			}
		} else {
			return $this->error(__('No se ha encontrado el pedido.'));
		}
	}
	function getComprobarClientePago(){
		if ($this->get('elemento') != 'pedido') {
            return $this->error(__('Acceso incorrecto'));
        }
		
		if (empty($_GET['email']) || empty($_GET['pedido'])){
			return $this->error(__('Datos incorrectos'));
		}
		
		e_include('ext_shop/class/pedidoShop');
		
		$tmp_pedido = new PedidoShop();
		$tmp_pedido->setid($_GET['pedido']);
		
		if ($tmp_pedido = $tmp_pedido->BuscarId()){
			if ($tmp_pedido->email != $_GET['email']){
				return $this->error(__('El correo electrónico no coincide con el del pedido'));
			} else {
				if ($tmp_pedido->estado == $GLOBALS['opciones']['ext_shop']['venta']['estado_procesando'] || $tmp_pedido->estado == $GLOBALS['opciones']['ext_shop']['venta']['estado_enviado'] || $tmp_pedido->estado == $GLOBALS['opciones']['ext_shop']['venta']['estado_completado']){
					$data = array('pedido'=>$tmp_pedido->id, 'email'=>$tmp_pedido->email, 'estado'=>'pagado');
					return $this->arrResp(__('El pedido es correcto y se ha pagado'), $data);
				} else {
					return $this->error(__('El pedido no se ha pagado'));
				}
			}
		} else {
			return $this->error(__('Pedido incorrecto'));
		}
		
	}
}