<?php
/**
 * Shop cart
 * 
 * The Shop cart class stores cart data and active cupones as well as handling customer sessions and some cart related urls.
 * The cart class also has a precio calculation function which calls upon other classes to calculate totals.
 *
 * @class Carrito
 * @package Shop
 * @category Class
 * @author Mideas
 */
class Carrito {
	var $contenido_carrito;
	var $reglas_aplicadas;
	var $reglas_ya_aplicadas = array();
	var $contenido_carrito_total;
	var $contenido_carrito_peso;
	var $contenido_carrito_cuenta;
	var $contenido_carrito_impuesto;
	var $total;
	var $subtotal;
	var $subtotal_ex_impuesto;
	var $impuesto_total;
	var $impuestos;
	var $envio_impuestos;
	var $descuento_carrito;
	var $descuento_total;
	var $envio_total;
	var $envio_impuesto_total;
	var $envio_nombre;
	var $envio_tipo;
	var $impuesto;
	var $envio_gratis = false;
	var $metodo_pago_elegido;
	var $recargo_metodo_pago = 0;
	
	/**
	 * Constructor
	 *
	 * Gets options from DB, loads the impuesto class, and queues the cart init
	 */
	function __construct() {
		$this->includes();
		
		$config_shop = App::getConfigValue('ext_shop');
		
		$this->impuesto = new ImpuestoTarifa();
		$this->precios_incluyen_impuesto = ( $config_shop['impuestos']['impuestos_incluidos'] == '1' ) ? true : false;
		$this->mostrar_totales_ex_impuesto = false;
		$this->mostrar_carrito_ex_impuestos = false;
		
		$this->init();
	}
	
	private function includes(){
		c_include('loc/region','loc/provincia','loc/localidad','loc/pais');
		e_include('ext_shop/class/helper','ext_shop/class/producto','ext_shop/class/productoVariacion','ext_shop/class/categoriaProducto','ext_comentarios/class/comentario','ext_shop/class/envio','ext_shop/class/metodoEnvio','ext_shop/class/envio/tarifaPlana','ext_shop/class/envioShop','ext_shop/class/carrito','ext_shop/class/impuestoShop','ext_shop/class/impuestoTarifa','ext_shop/class/checkout','ext_shop/class/cliente','ext_forms/class/formulario','ext_shop/class/pago','ext_shop/class/metodoPago','ext_shop/class/pedidoShop','ext_shop/class/pedidoLineaShop','ext_shop/class/reglaPrecio','ext_shop/class/facturaShop','ext_shop/class/facturaLineaShop','ext_shop/class/fabricante');
		e_include('ext_shop/models/producto','ext_shop/models/categoria','ext_shop/models/home','ext_shop/models/cesta','ext_shop/models/cliente','ext_shop/models/pedido','ext_shop/models/reglas','ext_shop/models/fabricante','ext_shop/models/categoria_fabricantes');
	}
    
    /**
	 * Loads the cart data from the session during WordPress init
	 */
    function init() {
		$this->get_carrito_from_session();
		
		$this->chequea_items_carrito();
		$this->chequea_reglas_aplicadas();
    }
	
	public function parseaVariacion($producto, $variacion_id){
		if (empty($producto) || empty($variacion_id))
			return false;
			
		if (!$producto instanceof Producto)
			return false;
			
		if (empty($variacion_id))
			return $producto;
			
		e_include('ext_shop/class/productoVariacion');
		$pv = new ProductoVariacion();
		$pv->setid($variacion_id);
		
		if ($pv = $pv->BuscarId()){
		} else {
			return $producto;
		}
		
		if (!empty($pv->imagen_destacada))
			$producto->imagen_destacada = $pv->imagen_destacada;
			
		if (trim($pv->stock)!='')
			$producto->stock = $pv->stock;
			
		if (!empty($pv->sku))
			$producto->sku = $pv->sku;
			
		if (!empty($pv->peso))
			$producto->peso = $pv->peso;
			
		if (!empty($pv->precio))
			$producto->precio = $pv->precio;
			
		if (!empty($pv->precio_rebajado))
			$producto->precio_rebajado = $pv->precio_rebajado;
			
		if (!empty($pv->desde_precio_rebajado))
			$producto->desde_precio_rebajado = $pv->desde_precio_rebajado;
			
		if (!empty($pv->hasta_precio_rebajado))
			$producto->hasta_precio_rebajado = $pv->hasta_precio_rebajado;
			
		return $producto;
	}
 	/*-----------------------------------------------------------------------------------*/
	/* Cart Session Handling */
	/*-----------------------------------------------------------------------------------*/ 
	
		/**
		 * Get the cart data from the PHP session
		 */
		function get_carrito_from_session() {
			$config_shop = App::getConfigValue('ext_shop');
			
			$_reglas = Helpers::getSesion()->get('shop.reglas');
			$_cesta = Helpers::getSesion()->get('shop.cesta');
			
			$_pago = Helpers::getSesion()->get('shop._metodo_pago_elegido');
			if (!empty($_pago)){
				$_pasarelas = new Pago();
				if (is_object($_pasarelas->pasarelas_pago[$_pago])){
					$_pago = $_pasarelas->pasarelas_pago[$_pago];
				} else {
					unset($_pago);
				}
				$this->metodo_pago_elegido = $_pago;
			}
			
			// Load the cupones
			$this->reglas_aplicadas = ( empty( $_reglas ) ) ? array() : Helpers::getSesion()->get('shop.reglas');
			
			// Load the cart
			if ( isset( $_cesta ) && is_array( Helpers::getSesion()->get('shop.cesta') ) ) {
				$cart = Helpers::getSesion()->get('shop.cesta');
				
				foreach ( $cart as $key => $values ) {
					$_product = new Producto( $values['product_id'] );
					$_product->setid($values['product_id']);
					$_product=$_product->BuscarId();
					
					if (!empty($values['variation_id']))
						$_product = $this->parseaVariacion($_product, $values['variation_id']);
					
					if ( $_product->id != "" && $values['cantidad'] > 0 ) {
						
						// Put session data into array. Run through filter so other plugins can load their own session data
						$this->contenido_carrito[$key] = array(
							'product_id'	=> $values['product_id'],
							'variation_id'	=> $values['variation_id'],
							'variation' 	=> $values['variation'],
							'cantidad' 		=> $values['cantidad'],
							'data'			=> $_product
						);
	
					}
				}
				
				if ( ! is_array( $this->contenido_carrito ) ) 
					$this->contenido_carrito = array();
				
			} else {
				$this->contenido_carrito = array();
			}
			
			// Load totals
			$this->contenido_carrito_total 	= Helpers::getSesion()->get('shop.contenido_carrito_total');
			$this->contenido_carrito_peso = Helpers::getSesion()->get('shop.contenido_carrito_peso');
			$this->contenido_carrito_cuenta 	= Helpers::getSesion()->get('shop.contenido_carrito_cuenta');
			$this->contenido_carrito_impuesto 	= Helpers::getSesion()->get('shop.contenido_carrito_impuesto');
			$this->total 				= Helpers::getSesion()->get('shop.total');
			$this->subtotal 			= Helpers::getSesion()->get('shop.subtotal');
			$this->subtotal_ex_impuesto 		= Helpers::getSesion()->get('shop.subtotal_ex_impuesto');
			$this->impuesto_total 			= Helpers::getSesion()->get('shop.impuesto_total');
			$this->impuestos 				= Helpers::getSesion()->get('shop.impuestos');
			$this->envio_impuestos		= Helpers::getSesion()->get('shop.envio_impuestos');
			$this->descuento_carrito 		= Helpers::getSesion()->get('shop.descuento_carrito');
			$this->descuento_total 		= Helpers::getSesion()->get('shop.descuento_total');
			$this->envio_total 		= Helpers::getSesion()->get('shop.envio_total');
			$this->envio_impuesto_total 	= Helpers::getSesion()->get('shop.envio_impuesto_total');
			$this->envio_nombre		= Helpers::getSesion()->get('shop.envio_nombre');
			$this->envio_tipo		= Helpers::getSesion()->get('shop.envio_impuesto_tipo');
			$this->recargo_metodo_pago = Helpers::getSesion()->get('shop.recargo_metodo_pago');
			
			// Queue re-calc if subtotal is not set
			if ( ! $this->subtotal && sizeof( $this->contenido_carrito ) > 0 ) 
				$this->set_session();
		}
		
		/**
		 * Sets the php session data for the cart and cupones and re-calculates totals
		 */
		function set_session() {
			// Re-calc totals
			$this->calcular_totales();
			
			// Set cart and coupon session data
			$cart_session = array();
			
			if ( $this->contenido_carrito ) {
				foreach ( $this->contenido_carrito as $key => $values ) {
					
					$cart_session[$key] = $values;
					
					// Unset product object
					unset( $cart_session[$key]['data'] );
				}
			}
			
			Helpers::getSesion()->set('shop.cesta', $cart_session);
			Helpers::getSesion()->set('shop.reglas', $this->reglas_aplicadas);
			
			// Store totals to avoid re-calc on page load
			Helpers::getSesion()->set('shop.contenido_carrito_total', $this->contenido_carrito_total);
			Helpers::getSesion()->set('shop.contenido_carrito_peso', $this->contenido_carrito_peso);
			Helpers::getSesion()->set('shop.contenido_carrito_cuenta', $this->contenido_carrito_cuenta);
			Helpers::getSesion()->set('shop.contenido_carrito_impuesto', $this->contenido_carrito_impuesto);
			Helpers::getSesion()->set('shop.total', $this->total);
			Helpers::getSesion()->set('shop.subtotal', $this->subtotal);
			Helpers::getSesion()->set('shop.subtotal_ex_impuesto', $this->subtotal_ex_impuesto);
			Helpers::getSesion()->set('shop.impuesto_total', $this->impuesto_total);
			Helpers::getSesion()->set('shop.envio_impuestos', $this->envio_impuestos);
			Helpers::getSesion()->set('shop.impuestos', $this->impuestos);
			Helpers::getSesion()->set('shop.descuento_carrito', $this->descuento_carrito);
			Helpers::getSesion()->set('shop.descuento_total', $this->descuento_total);
			Helpers::getSesion()->set('shop.envio_total', $this->envio_total);
			Helpers::getSesion()->set('shop.envio_impuesto_total', $this->envio_impuesto_total);
			Helpers::getSesion()->set('shop.envio_nombre', $this->envio_nombre);
			Helpers::getSesion()->set('shop.envio_tipo', $this->envio_tipo);
			Helpers::getSesion()->set('shop.recargo_metodo_pago', $this->recargo_metodo_pago);
		}
		
		/**
		 * Empties the cart and optionally the persistent cart too
		 *
		 * @access public
		 * @param bool $clear_persistent_cart (default: true)
		 */
		function vaciar_carrito( $clear_persistent_cart = true ) {
		
			$this->contenido_carrito = array();
			//$this->reglas_aplicadas = array();
			$this->reset();
			
			Helpers::getSesion()->clear('shop.reglas');
			Helpers::getSesion()->clear('shop.cesta');
			
		}
 	/*-----------------------------------------------------------------------------------*/
	/* Persistent cart handling */
	/*-----------------------------------------------------------------------------------*/ 
		
		/**
		 * Save the persistent cart when updated
		 */
		function persistent_cart_update() {
		}
		
		/**
		 * Delete the persistent cart
		 */
		function persistent_cart_destroy() {
		}
		
 	/*-----------------------------------------------------------------------------------*/
	/* Cart Data Functions */
	/*-----------------------------------------------------------------------------------*/ 
		/**
		 * Check cart items for errors
		 */
		function chequea_items_carrito() {
			// Check item stock
			$result = $this->chequea_stock_item_carrito();
		}
		
		/**
		 * Get cart items cantidades - merged so we can do accutarifa stock checks
		 * 
		 * @access public
		 * @return array
		 */
		function get_cantidades_item_carrito() {
			$cantidades = array();
			
			foreach ( $this->get_carrito() as $cart_item_key => $values ) {
				$cantidades[ $values['product_id'].'.'.$values['variation_id'] ] = isset( $cantidades[ $values['product_id'].'.'.$values['variation_id'] ] ) ? $cantidades[ $values['product_id'].'.'.$values['variation_id'] ] + $values['cantidad'] : $values['cantidad'];
			}			
			return $cantidades;
		}
		
		/**
		 * Check for user cupones (now we have billing email)
		 * 
		 * @access public
		 * @param array $posted
		 */
		function chequea_reglas_aplicadas() {
			//var_dumpre($this);
		}
		/**
		 * Looks through the cart to check each item is in stock 
		 * 
		 * @access public
		 * @return bool
		 */
		function chequea_stock_item_carrito() {
			$error = array();
			
			$product_qty_in_cart = $this->get_cantidades_item_carrito();
			
			// First stock check loop
			foreach ( $this->get_carrito() as $cart_item_key => $values ) {
				
				$_product = $values['data'];
				
				/**
				 * Check stock based on inventory
				 */
				if ( $_product->stock ) {
					
					/**
					 * Check the stock for this item individually
					 */
					if ( $_product->stock==0 || $_product->stock < $values['cantidad'] ) {
						App::setMsg( sprintf(__('No puedes añadir esa cantidad de "%s" al carrito. Tenemos %s en stock'), SistemaIdioma::Imprimir($_product->nombre), $_product->getstock()), 'error' );
						return $error;
					}
					
					// For later on...
					$key     = '_product_id';
					$value   = $values['product_id'];
					$in_cart = $values['cantidad'];
					
					/**
					 * Next check entire cart cantidades
					 */
					if ( $values['variation_id'] /*&& $_product->variation_has_stock*/ && isset( $product_qty_in_cart[ $values['product_id'].'.'.$values['variation_id'] ] ) ) {
						$key     = '_variation_id';
						$value   = $values['variation_id'];
						$in_cart = $product_qty_in_cart[ $values['product_id'].'.'.$values['variation_id'] ];
						if ( ! $_product->stock <  $product_qty_in_cart[ $values['product_id'].'.'.$values['variation_id'] ] ) {
							App::setMsg( sprintf(__( 'Sorry, we do not have enough "%s" in stock to fulfil your order (%s in stock). Please edit your cart and try again. We apologise for any inconvenience caused.'), SistemaIdioma::Imprimir($_product->nombre), $_product->stock ) );
							return $error;
						}
					} elseif ( isset( $product_qty_in_cart[$values['product_id'].'.'.$values['variation_id']] ) ) {
						
						if ( $_product->stock < $product_qty_in_cart[$values['product_id']] ) {
							App::setMsg( sprintf(__('No puedes añadir esa cantidad de "%s" al carrito. Tenemos %s en stock'), SistemaIdioma::Imprimir($_product->nombre), $_product->getstock()) );
							return $error;
						}
						
					}
				
				/**
				 * Check stock based on stock-status
				 */
				} else {
					if ( $_product->stock==0 ) {
						App::setMsg( sprintf(__('No puedes añadir esa cantidad de "%s" al carrito. No tenemos stock'), SistemaIdioma::Imprimir($_product->nombre), $_product->getstock()), 'error' );
						return $error;
					}
				}
			}
			return true;
		}
		
		/**
		 * Gets and formats a list of cart item data + variations for display on the frontend
		 * 
		 * @access public
		 * @param array $cart_item
		 * @param bool $flat (default: false)
		 * @return string
		 */
		function get_item_data( $cart_item, $flat = false ) {
			global $woocommerce;
			
			$has_data = false;
			
			if ( ! $flat ) $return = '<dl class="variation">';
			
			// Variation data
			if ( $cart_item['data'] instanceof Producto && is_array( $cart_item['variation'] ) ) {
			
				$variation_list = array();
				
				foreach ( $cart_item['variation'] as $name => $value ) {
					
					if ( ! $value ) continue;
					
					// If this is a term slug, get the term's nice name
		            if ( impuestoonomy_exists( esc_attr( str_replace( 'attribute_', '', $name ) ) ) ) {
		            	$term = get_term_by( 'slug', $value, esc_attr( str_replace( 'attribute_', '', $name ) ) );
		            	if ( ! is_wp_error( $term ) && $term->name )
		            		$value = $term->name;
		            } else {
		            	$value = ucfirst( $value );
					}
					
					if ($flat)
						$variation_list[] = $woocommerce->attribute_label( str_replace( 'attribute_', '', $name ) ) . ': ' . $value;
					else
						$variation_list[] = '<dt>'.$woocommerce->attribute_label( str_replace( 'attribute_', '', $name ) ) . ':</dt><dd>' . $value . '</dd>';
					
				}
				
				if ($flat)
					$return .= implode( ', ', $variation_list );
				else
					$return .= implode( '', $variation_list );
				
				$has_data = true;
			
			}
			
			// Other data - returned as array with name/value values
			$other_data = apply_filters( 'woocommerce_get_item_data', array(), $cart_item );
			
			if ( $other_data && is_array( $other_data ) && sizeof( $other_data ) > 0 ) {
				
				$data_list = array();
				
				foreach ($other_data as $data ) {
					
					$display_value = !empty($data['display']) ? $data['display'] : $data['value'];
					
					if ($flat)
						$data_list[] = $data['name'].': '.$display_value;
					else
						$data_list[] = '<dt>'.$data['name'].':</dt><dd>'.$display_value.'</dd>';
					
				}
				
				if ($flat)
					$return .= implode(', ', $data_list);
				else
					$return .= implode('', $data_list);
				
				$has_data = true;
				
			}
			
			if ( ! $flat ) 
				$return .= '</dl>';
			
			if ( $has_data ) 
				return $return;	
		}
	
		/**
		 * Gets cross sells based on the items in the cart
		 *
		 * @return array cross_sells (item ids)
		 */
		function get_cross_sells() {
			$cross_sells = array();
			$in_cart = array();
			if ( sizeof( $this->contenido_carrito) > 0 ) {
				foreach ( $this->contenido_carrito as $cart_item_key => $values ) {
					if ( $values['cantidad'] > 0 ) {
						$cross_sells = array_merge( $values['data']->get_cross_sells(), $cross_sells );
						$in_cart[] = $values['product_id'];
					}
				}
			}
			$cross_sells = array_diff( $cross_sells, $in_cart );
			return $cross_sells;
		}
		
		/**
		 * Gets the url to the cart page
		 * 
		 * @return string url to page
		 */
		function get_carrito_url() {
			$cart_page_id = woocommerce_get_page_id('cart');
			if ( $cart_page_id ) return apply_filters( 'woocommerce_get_carrito_url', get_permalink( $cart_page_id ) );
		}
	
		/**
		 * Gets the url to the checkout page
		 * 
		 * @return string url to page
		 */
		function get_checkout_url() {
			$checkout_page_id = woocommerce_get_page_id('checkout');
			if ( $checkout_page_id ) {
				if ( is_ssl() ) 
					return str_replace( 'http:', 'https:', get_permalink($checkout_page_id) );
				else
					return apply_filters( 'woocommerce_get_checkout_url', get_permalink($checkout_page_id) );
			}
		}
		
		/**
		 * Gets the url to remove an item from the cart
		 * 
		 * @return string url to page
		 */
		function get_remove_url( $cart_item_key ) {
			$cart_page_id = woocommerce_get_page_id('cart');
			if ($cart_page_id) 
				return apply_filters( 'woocommerce_get_remove_url', $woocommerce->nonce_url( 'cart', add_query_arg( 'remove_item', $cart_item_key, get_permalink($cart_page_id) ) ) );
		}
		
		/**
		 * Returns the contenido of the cart
		 * 
		 * @return array contenido of the cart
		 */
		function get_carrito() {
			return array_filter( (array) $this->contenido_carrito );
		}
		
		/**
		 * Returns the cart and shipping impuestos, merged
		 * 
		 * @return array merged impuestos
		 */
		function get_impuestos() {
			$merged_impuestos = array();
			// Merge
			foreach ( array_keys( $this->impuestos + $this->envio_impuestos ) as $key ) {
				$merged_impuestos[$key] = ( isset( $this->envio_impuestos[$key] ) ? $this->envio_impuestos[$key] : 0 ) + ( isset( $this->impuestos[$key] ) ? $this->impuestos[$key] : 0 );
			}
			
			return $merged_impuestos;
		}	
		
 	/*-----------------------------------------------------------------------------------*/
	/* Add to cart handling */
	/*-----------------------------------------------------------------------------------*/ 
	
		/**
	     * Check if product is in the cart and return cart item key
	     * 
	     * Cart item key will be unique based on the item and its properties, such as variations
	     *
	     * @param mixed id of product to find in the cart
	     * @return string cart item key
	     */
	    function encontrar_producto_en_carrito( $cart_id = false ) {
	        if ( $cart_id !== false ) 
	        	foreach ( $this->contenido_carrito as $cart_item_key => $cart_item ) 
	        		if ( $cart_item_key == $cart_id ) 
	        			return $cart_item_key;
	    }
		
		/**
	     * Genetarifa a unique ID for the cart item being added
	     *
	     * @param int $producto - id of the product the key is being genetarifad for
	     * @param int $variation_id of the product the key is being genetarifad for
	     * @param array $variation data for the cart item
	     * @param array $cart_item_data other cart item data passed which affects this items uniqueness in the cart
	     * @return string cart item key
	     */
	    function genera_id_carrito( $producto, $variation_id = '', $variation = '', $cart_item_data = array() ) {
	        
	        $id_parts = array( $producto );
	        
	        if ( $variation_id ) $id_parts[] = $variation_id;
	 
	        if ( is_array( $variation ) ) {
	            $variation_key = '';
	            foreach ( $variation as $key => $value ) {
	                $variation_key .= trim( $key ) . trim( $value );
	            }
	            $id_parts[] = $variation_key;
	        }
	        
	        if ( is_array( $cart_item_data ) && ! empty( $cart_item_data ) ) {
	            $cart_item_data_key = '';
	            foreach ( $cart_item_data as $key => $value ) {
	            	if ( is_array( $value ) ) $value = http_build_query( $value );
	                $cart_item_data_key .= trim($key) . trim($value);
	            }
	            $id_parts[] = $cart_item_data_key;
	        }
	
	        return md5( implode( '_', $id_parts ) );
	    }	
		
		/**
		 * Add a product to the cart
		 *
		 * @param string $producto contains the id of the product to add to the cart
		 * @param string $cantidad contains the cantidad of the item to add
		 * @param int $variation_id
		 * @param array $variation attribute values
		 * @param array $cart_item_data extra cart item data we want to pass into the item
		 * @return bool
		 */
		function add_to_cart( $producto, $cantidad = 1, $variation_id = '', $variation = '', $cart_item_data = array() ) {
			if ( $cantidad < 1 ) return false;
			
			// Load cart item data - may be added by other plugins
			$cart_item_data = (array) $producto;
			
			// Genetarifa a ID based on product ID, variation ID, variation data, and other cart item data
			$cart_id = $this->genera_id_carrito( $producto, $variation_id, $variation, $cart_item_data );
			
			// See if this product and its options is already in the cart
			$cart_item_key = $this->encontrar_producto_en_carrito( $cart_id );
			$product_data = new Producto( $producto );
			$product_data->setid($producto);
			$product_data = $product_data->BuscarId();
			
			if ($product_data->solocatalogo == 1)
				return __('Este producto no se puede comprar');
			
			$variaciones = $product_data->getProductoVariaciones();
			
			if (is_array($variaciones) && sizeof($variaciones) > 0 && empty($variation_id)){
				return 'no_variacion';
			}
			
			if (!empty($variation_id))
				$product_data = $this->parseaVariacion($product_data, $variation_id);
				
			// Price set check
			if( $product_data->getprecio() === '' ) {
				return __('Este producto no se puede comprar - aun no se ha establecido el precio.'); 
			}
	
			// Stock check - only check if we're managing stock and backorders are not allowed
			if ( $product_data->stock < $cantidad ) {
				return sprintf(__('No puedes añadir esa cantidad al carrito. Tenemos %s en stock'), $product_data->stock); 
			} elseif ( $product_data->stock == 0 ) {
				return __('El producto está fuera de stock.');
			}
			// Downloadable/virtual qty check
			/*if ( get_option('woocommerce_limit_downloadable_product_qty')=='yes' && $product_data->is_downloadable() && $product_data->is_virtual() ) {
				$qty = ( $cart_item_key ) ? $this->contenido_carrito[$cart_item_key]['cantidad'] + $cantidad : $cantidad;
				if ( $qty > 1 ) {
					$woocommerce->add_error( sprintf('<a href="%s" class="button">%s</a> %s', get_permalink(woocommerce_get_page_id('cart')), __('View Cart &rarr;', 'woocommerce'), __('You already have this item in your cart.', 'woocommerce') ) );
					return false;
				}
			}*/
			
			// Stock check - this time accounting for whats already in-cart
			$product_qty_in_cart = $this->get_cantidades_item_carrito();
			
			if ( isset( $product_qty_in_cart[ $producto.'.'.$variation_id ] ) && $product_data->stock < $product_qty_in_cart[ $producto.'.'.$variation_id ] + $cantidad ) {
				return sprintf(__('<a href=%s class=button>%s</a> No puedes añadir esa cantidad al carrito &mdash;tenemos %s en stock y ya tenías %s en tu carrito.'), Helpers::resolveLink(false, 'ext_shop', 'cesta'), __('Ver Cesta &rarr;'), $product_data->stock, $product_qty_in_cart[ $producto.'.'.$variation_id ] );
			}
			
			// If cart_item_key is set, the item is already in the cart
			if ( $cart_item_key ) {
				
				$new_cantidad = $cantidad + $this->contenido_carrito[$cart_item_key]['cantidad'];
	
				$this->set_cantidad( $cart_item_key, $new_cantidad );
	
			} else {
			
				$cart_item_key = $cart_id;
				
				// Add item after merging with $cart_item_data - hook to allow plugins to modify cart item
				$this->contenido_carrito[$cart_item_key] = array_merge( $cart_item_data, array(
					'product_id'	=> $producto,
					'variation_id'	=> $variation_id,
					'variation' 	=> $variation,
					'cantidad' 		=> $cantidad,
					'data'			=> $product_data
				) );
			}
			
			$this->set_session();
			
			return true;
		}
		/**
		 * Set the cantidad for an item in the cart
		 *
		 * @param   string	cart_item_key	contains the id of the cart item
		 * @param   string	cantidad	contains the cantidad of the item
		 */
		function set_cantidad( $cart_item_key, $cantidad = 1 ) {
			if ( $cantidad == 0 || $cantidad < 0 ) {
				unset( $this->contenido_carrito[$cart_item_key] );
			} else {
				$this->contenido_carrito[$cart_item_key]['cantidad'] = $cantidad;
			}
			
			$this->set_session();
		}
    /*-----------------------------------------------------------------------------------*/
	/* Cart Calculation Functions */
	/*-----------------------------------------------------------------------------------*/ 
		/** 
		 * Reset totals
		 */
		private function reset() {
			$this->total = 0;
			$this->contenido_carrito_total = 0;
			$this->contenido_carrito_peso = 0;
			$this->contenido_carrito_cuenta = 0;
			$this->contenido_carrito_impuesto = 0;
			$this->impuesto_total = 0;
			$this->envio_impuesto_total = 0;
			$this->envio_impuestos = array();
			$this->subtotal = 0;
			$this->subtotal_ex_impuesto = 0;
			$this->descuento_total = 0;
			$this->descuento_carrito = 0;
			$this->envio_total = 0;
			$this->impuestos = array();
			//$this->reglas_aplicadas = array();
			$this->reglas_ya_aplicadas = array();
			$this->recargo_metodo_pago = 0;
			
		}
		
		/**
		 * Function to apply discounts to a product and get the discounted precio (before impuesto is applied)
		 * 
		 * @access public
		 * @param mixed $values
		 * @param mixed $precio
		 * @param bool $add_totals (default: false)
		 * @return float precio
		 */
		function get_precio_con_descuento( $values, $precio, $add_totals = false ) {
			
			if ( ! $precio ) return $precio;
			
			if ( ! empty( $this->reglas_aplicadas ) ) {
				foreach ( $this->reglas_aplicadas as $kregla=>$regla ) {
					
					if ($regla['accion']['tipo'] == 'cesta.producto' && !in_array($regla['id'], $this->reglas_ya_aplicadas)){
						
						$ids_productos_disponibles = explode(',', $regla['accion']['cantidad_descuento']);
						foreach ($ids_productos_disponibles as $kid=>$idd){
							$ids_productos_disponibles[$kid] = trim($idd);
						}
						
						$cantidad_descuento = 0;
						if (in_array($values['product_id'],$ids_productos_disponibles)){
							$cantidad_descuento = (float) $precio / (int) $values['cantidad'];
							$this->reglas_ya_aplicadas[] = $regla['id'];
						}
						
						$precio = $precio - $cantidad_descuento;
						
					} elseif (($regla['accion']['tipo'] == 'productos.porcentaje' || $regla['accion']['tipo'] == 'productos.importe') && !in_array($regla['id'], $this->reglas_ya_aplicadas)){
						
						if ($regla['accion']['envio_gratis'] == '1' && $this->envio_total > 0 && $this->envio_gratis == false){
							$this->descuento_total = $this->descuento_total + $this->envio_total + $this->envio_impuesto_total;
							//$this->envio_total = 0;
							$this->envio_gratis = true;
						}
						
						if (strpos($regla['accion']['cantidad_descuento'], '/')){
							$val_descuento = explode('/', $regla['accion']['cantidad_descuento']);
							$id_prod = $val_descuento[1];
							$val_descuento = $val_descuento[0];
						} else {
							$val_descuento = $regla['accion']['cantidad_descuento'];
							$id_prod = 0;
						}
						switch ($regla['accion']['tipo']){
							case 'productos.importe':
								if ($id_prod == 0 || ($id_prod == $values['product_id'])){
									$precio = $precio - $val_descuento;
								}
								break;
							case 'productos.porcentaje':
								if ($id_prod == 0 || ($id_prod == $values['product_id'])){
									$percent_discount = ( $precio ) * ((float) $val_descuento / 100 );
									$precio = $precio - $percent_discount;
								}
								break;
						}
					}
				}
			}
			
			return $precio;
		}
		
		/**
		 * Function to apply product discounts after impuesto
		 * 
		 * @access public
		 * @param mixed $values
		 * @param mixed $precio
		 */
		function aplicar_descuentos_producto_despues_impuesto( $values, $precio ) {
			/*if ( $this->reglas_aplicadas ) {
				
				foreach ( $this->reglas_aplicadas as $regla ) {
					
					if ($regla['accion']['tipo'] == 'productos.importe' || $regla['accion']['tipo'] == 'productos.porcentaje'){
					
						if ($regla['accion']['envio_gratis'] == '1' && $this->envio_total > 0 && $this->envio_gratis == false){
							$this->descuento_total = $this->descuento_total + $this->envio_total + $this->envio_impuesto_total;
							//$this->envio_total = 0;
							$this->envio_gratis = true;
						}
						
						if (strpos($regla['accion']['cantidad_descuento'], '/')){
							$val_descuento = explode('/', $regla['accion']['cantidad_descuento']);
							$id_prod = $val_descuento[1];
							$val_descuento = $val_descuento[0];
						} else {
							$val_descuento = $regla['accion']['cantidad_descuento'];
							$id_prod = 0;
						}
						
						switch ($regla['accion']['tipo']){
							case 'productos.importe':
								if ($id_prod == 0 || ($id_prod == $values['product_id'])){
									$this->descuento_total = $this->descuento_total + $val_descuento * (int) $values['cantidad'];
								}
								break;
							case 'productos.porcentaje':
								if ($id_prod == 0 || ($id_prod == $values['product_id'])){
									$percent_discount = ( round( $precio , 2 ) ) * ((float) $val_descuento / 100 );
									$this->descuento_total = $this->descuento_total + $percent_discount;
								}
								break;
						}
					
					}
				}
				
				$this->envio_gratis = false;
			}*/
		}
		
		/**
		 * Function to apply cart discounts after impuesto
		 * 
		 * @access public
		 */
		function aplicar_descuentos_carrito_despues_impuesto() {
			if ( $this->reglas_aplicadas ) {
				
				foreach ( $this->reglas_aplicadas as $regla ) {
					
					if ($regla['accion']['tipo'] == 'cesta.importe' || $regla['accion']['tipo'] == 'cesta.porcentaje' || empty($regla['accion']['tipo'])){
					
						if ($regla['accion']['envio_gratis'] == '1' && $this->envio_total > 0 && $this->envio_gratis == false){
							$this->descuento_total = $this->descuento_total + $this->envio_total + $this->envio_impuesto_total;
							//$this->envio_total = 0;
							$this->envio_gratis = true;
						}
						
						switch ($regla['accion']['tipo']){
							case 'cesta.importe':
								$this->descuento_total = $this->descuento_total + $regla['accion']['cantidad_descuento'];
								break;
							case 'cesta.porcentaje':
								$percent_discount = ( $this->contenido_carrito_total + $this->impuesto_total ) * ((float) $regla['accion']['cantidad_descuento'] / 100 );
								$this->descuento_total = $this->descuento_total + $percent_discount;
								break;
						}
					
					}
				}
				
				$this->envio_gratis = false;
			}
		}
		
		/**
		 * calculate totals for the items in the cart 
		 * 
		 * @access public
		 */
		function calcular_totales() {
			$config_shop = App::getConfigValue('ext_shop');
			$this->reset();
			
			global $paises_defecto_shop;
			
			if (is_array($paises_defecto_shop)){
				$paises_defecto = $paises_defecto_shop;
			} else {
				$paises_defecto_shop = Helpers::getPaisesWithArgs($config_shop['envios']['paises'], $config_shop['envios']['disponibilidad']);
				$paises_defecto = $paises_defecto_shop;
			}
			
			$usr_sess = Helpers::getSesion()->get('shop.usuario');
			
			if ($usr_sess['envio_igual_facturacion'] == '0'){
				$pais_usr = $usr_sess['envio_pais'];
				$prov_usr = $usr_sess['envio_provincia'];
			} else {
				$pais_usr = $usr_sess['pais'];
				$prov_usr = $usr_sess['provincia'];
			}
			
			if (empty($prov_usr))
				$prov_usr = 0;
		
			if (empty($pais_usr)){
				$localizacion_usr = $GLOBALS['opciones']['ext_shop']['shop']['localizacion_defecto'];
			} else {
				$localizacion_usr = $pais_usr.'.'.$prov_usr;
			}
			
			$recargo_pago = $this->metodo_pago_elegido->recargo;
			$recargo_pago = str_replace (',','.',$recargo_pago);
			$recargo_pago = $recargo_pago / 100;
			
			// Get count of all items + weights + subtotal (we may need this for discounts)
			if ( sizeof( $this->contenido_carrito ) > 0 ) {
				foreach ( $this->contenido_carrito as $cart_item_key => $values ) {
					
					$_product = $values['data'];
					
					$this->contenido_carrito_peso = $this->contenido_carrito_peso + ($_product->getpeso() * $values['cantidad']);
					$this->contenido_carrito_cuenta 	= $this->contenido_carrito_cuenta + $values['cantidad'];
					
					$_precio = $_product->get_precio_real();
					
					// Line precios
					$linea_impuesto = $_precio["impuesto"] * $values['cantidad'];
					$linea_total 		= $_precio["precio_base"] * $values['cantidad'];
					$linea_subtotal = ($_precio["precio_base"] * $values["cantidad"]) + $linea_impuesto;

					// Add any product discounts (after impuesto)
					$this->aplicar_descuentos_producto_despues_impuesto( $values, $linea_total + ($_precio["impuesto"] * $values['cantidad']) );

					// Cart contenido total is based on discounted precios and is used for the final total calculation
					$this->contenido_carrito_total 	= $this->contenido_carrito_total + $linea_total;

					$this->recargo_metodo_pago = $this->recargo_metodo_pago + ($importe_recargo_pago * $values['cantidad']);

					// Store costs + impuestos for lines
					$this->contenido_carrito[$cart_item_key]['linea_total'] 			= $linea_total;
					$this->contenido_carrito[$cart_item_key]['linea_impuesto'] 			= $linea_impuesto;
					$this->contenido_carrito[$cart_item_key]['linea_subtotal'] 			= $linea_subtotal;
					$this->contenido_carrito[$cart_item_key]['linea_subtotal_impuesto'] = $linea_impuesto;
					
					// Sub total is based on base precios (without discounts)
					$this->subtotal 			= $this->subtotal + ( ($_precio["precio_base"] + $_precio["impuesto"]) * $values['cantidad']);

					$this->subtotal_ex_impuesto 		= $this->subtotal_ex_impuesto + ($_precio["precio_base"] * $values['cantidad']);
					
					$this->impuesto_total = $this->impuesto_total + ($_precio["impuesto"] * $values['cantidad']);
					
					
					
				}
			}
			
			
			
			// Set impuesto total to sum of all impuesto rows
			//$this->impuesto_total = $this->impuesto->get_impuesto_total( $this->impuestos );
				
			// VAT exemption done at this point - so all totals are correct before exemption
			if ( $_GET['vista'] == 'cesta' && $config_shop['impuestos']['mostrar_impuestos_carrito'] == "0" ) {
				$this->envio_impuesto_total = $this->impuesto_total = 0;
				$this->impuestos = $this->envio_impuestos = array();
			}
			
			// Cart Shipping
			$this->calcular_envio();
			
			// Cart Discounts (after impuesto)
			$this->aplicar_descuentos_carrito_despues_impuesto();
			
			// Only go beyond this point if on the cart/checkout
			if ( ! $_GET['vista'] == 'checkout' ) return;
			// VAT exemption for shipping
			/*if ( $woocommerce->customer->is_vat_exempt() ) {
				$this->envio_impuesto_total = 0;
				$this->envio_impuestos = array();
			}*/
			
			// Round cart/shipping impuesto rows
			$this->impuestos = array_map( array( &$this->impuesto, 'round' ), $this->impuestos );
			$this->envio_impuestos = array_map( array( &$this->impuesto, 'round' ), $this->envio_impuestos );
			
			// Allow plugins to hook and alter totals before final total is calculated
			//$this->calcular_totales();
			
			/** 
			 * Grand Total
			 *
			 * Based on discounted product precios, discounted impuesto, shipping cost + impuesto, and any discounts to be added after impuesto (e.g. store credit)
			 */
			$this->total = $this->subtotal_ex_impuesto + $this->impuesto_total + $this->envio_impuesto_total + $this->envio_total - $this->descuento_total;
			
			if ($this->total < 0) $this->total = 0;
			
		}
		/** 
		 * looks at the totals to see if payment is actually required 
		 *
		 * @return bool
		 */
		function necesita_pago() {
			if ( $this->total > 0 ) return true; else return false;
		}
	
    /*-----------------------------------------------------------------------------------*/
	/* Shipping related functions */
	/*-----------------------------------------------------------------------------------*/ 
		/**
		 * calcular_envio function.
		 * 
		 * Uses the shipping class to calculate shipping then gets the totals when its finished.
		 *
		 * @access public
		 * @return void
		 */
		function calcular_envio() {
			$envios = new Envio();
			$envios->init();
			
			if ( $this->necesita_envio() && $this->mostrar_envio() ) {
				$envios->calcular_envio( $this->get_paquetes_envio() );
			} else {
				$envios->reset_envio(); 
			}
			
			// Get totals for the chosen shipping method
			$this->envio_total 		= $envios->envio_total;	// Shipping Total
			$this->envio_nombre 		= $envios->envio_nombre;	// Shipping Label
			$this->envio_tipo 		= $envios->envio_tipo;	// Shipping Label
			$this->envio_impuestos		= $envios->envio_impuestos;	// Shipping Taxes
			$this->envio_impuesto_total 	= $this->impuesto->get_impuesto_total( $this->envio_impuestos );	// Shipping impuesto amount
			
			return $envios;
			
		}
		
		/**
		 * get_paquetes_envio function.
		 *
		 * Get packages to calculate shipping for - this lets us calculate costs 
		 * for carts that are shipped to multiple locations.
		 *
		 * Shipping methods are responsble for looping through these packages.
		 *
		 * By default we pass the cart itself as a package - plugins can change this
		 * through the filter and break it up.
		 * 
		 * @since 1.5.4
		 * @access public
		 * @return array of cart items
		 */
		function get_paquetes_envio() {
			// Packages array for storing 'carts'
			$packages = array();
			
			$usr_sess = Helpers::getSesion()->get('shop.usuario');
			
			$packages[0]['contenido'] 				= $this->get_carrito();	// Items in the package
			$packages[0]['contenido_coste'] 			= 0;					// Cost of items in the package
			$packages[0]['destino']['pais'] 	= $usr_sess['pais'];
			$packages[0]['destino']['estado'] 	= $usr_sess['provincia'];
			$packages[0]['destino']['codigo_postal'] = $usr_sess['codigo_postal'];
			
			foreach ( $this->get_carrito() as $item ) {
				if ( $item['data']->necesita_envio() ) {
					$packages[0]['contenido_coste'] += $item['data']->getprecio(true) * $item['cantidad'];
				}
			}
			
			return $packages;
		}
		
		function continuar_pedido(){
			$config_shop = $GLOBALS['opciones']['ext_shop'];
			
			if ($config_shop['envios']['activar_envios'] == '1'){
				$envio_tmp = new Envio();
				$envio_tmp->init();
				
				if (empty($envio_tmp->metodos_envio) || is_null($this->envio_nombre)){
					return false;
				} else {
					$_metodo_env = Helpers::getSesion()->get('shop._metodo_envio_elegido');
					if (empty($_metodo_env))
						return false;
				}
			} else {
				return true;
			}
			
			return true;
		}
		
		/** 
		 * looks through the cart to see if shipping is actually required 
		 *
		 * @return bool whether or not the cart needs shipping
		 */
		function necesita_envio() {
			//if ( get_option('woocommerce_calc_shipping')=='no' ) return false;
			if ( ! is_array( $this->contenido_carrito ) ) return false;
		
			$necesita_envio = false;
			
			foreach ( $this->contenido_carrito as $cart_item_key => $values ) {
				$_product = $values['data'];
				if ( $_product->necesita_envio() ) {
					$necesita_envio = true;
					continue;
				} else {
					$necesita_envio = false;
				}
			}
			
			return $necesita_envio;
		}
		
		/** 
		 * mostrar_envio
		 *
		 * Sees if the customer has entered enough data to calc the shipping yet
		 *
		 * @return bool
		 */
		function mostrar_envio() {
			$config_shop = $GLOBALS['opciones']['ext_shop'];
			if ( ! is_array( $this->contenido_carrito ) ) return false;
			
			if ($config_shop['envios']['activar_envios'] == '1'){
				$envio_tmp = new Envio();
				$envio_tmp->init();
				
				if (empty($envio_tmp->metodos_envio)){
					return false;
				}
			} else {
				return false;
			}
			$mostrar_envio = true;
			
			return $mostrar_envio;
		}
		
		/** 
		 * Sees if we need a shipping address 
		 *
		 * @return bool
		 */
		function ship_to_billing_address_only() {
			return false;
		}
		
		/**
		 * gets the shipping total (after calculation)
		 *
		 * @return mixed precio or string for the shipping total
		 */
		function get_carrito_envio_total() {
			if ( isset( $this->envio_nombre ) ) {
				if ( $this->envio_total > 0 ) {
				
					// Display ex impuesto if the option is set, or precios exclude impuesto
					if ( $this->mostrar_totales_ex_impuesto || !$this->precios_incluyen_impuesto ) {
						
						$return = $this->envio_total;
						if ( $this->envio_impuesto_total > 0 && $this->precios_incluyen_impuesto ) {
							//$return .= ' <small>' . $woocommerce->countries->ex_impuesto_or_vat() . '</small>';
						}
						return $return;
						
					} else {
						
						$return = $this->envio_total + $this->envio_impuesto_total;
						if ( $this->envio_impuesto_total > 0 && ! $this->precios_incluyen_impuesto ) {
							//$return .= ' <small>' . $woocommerce->countries->inc_impuesto_or_vat() . '</small>';
						}
						return $return;
					
					}
	
				} else {
					return __('Gratis');
				}
			}
		}
		
		/**
		 * gets title of the chosen shipping method
		 *
		 * @return string shipping method title
		 */
		function get_titulo_envio_carrito() {
			if ( isset( $this->envio_nombre ) ) {
				return __('via') . ' ' . $this->envio_nombre;
			}
			return false;
		}
		
		/**
		 * gets title of the chosen shipping method
		 *
		 * @return string shipping method title
		 */
		function get_peso_formateado_carrito() {
			if ( !empty( $this->envio_tipo ) && $this->envio_tipo == 'tarifaPorPeso' ) {
				return number_format($this->contenido_carrito_peso, 2, ',', '.').' '.__('kg');
			}
			return false;
		}
    /*-----------------------------------------------------------------------------------*/
	/* Coupons/Discount related functions */
	/*-----------------------------------------------------------------------------------*/ 
		/** 
		 * returns whether or not a discount has been applied 
		 *
		 * @return bool
		 */
		function tiene_descuento( $code ) {
			if ( in_array($code, $this->reglas_aplicadas) ) return true;
			return false;
		}
		/**
		 * Applies a coupon code
		 *
		 * @param string $codigo_cupon - The code to apply
		 * @return bool	True if the coupon is applied, false if it does not exist or cannot be applied
		 */
		function add_descuento( $codigo_cupon ) {
			return false;
		}
	
		/**
		 * gets the array of applied coupon codes
		 *
		 * @return array of applied cupones
		 */
		function get_reglas_aplicadas() {
			return (array) $this->reglas_aplicadas;
		}
		
		/**
		 * Remove cupones from the cart
		 *
		 * @params int type - 0 for all, 1 for before impuesto, 2 for after impuesto
		 */
		function borrar_cupones( $type = 0 ) {
		
		}
    /*-----------------------------------------------------------------------------------*/
	/* Get Formatted Totals */
	/*-----------------------------------------------------------------------------------*/ 
	
		/** 
		 * Get the total of all order discounts (after impuesto discounts)
		 *
		 * @return float
		 */
		function get_descuento_total_pedido() {
			return $this->descuento_total;
		}
		
		/** 
		 * Get the total of all cart discounts (before impuesto discounts)
		 *
		 * @return float
		 */
		function get_carrito_descuento_total() {
			return $this->descuento_carrito;
		}
		
		/**
		 * gets the total (after calculation)
		 *
		 * @return string formatted precio
		 */
		function get_total() {
			return $this->total;
		}
		
		/**
		 * gets the total excluding impuestos
		 *
		 * @return string formatted precio
		 */
		function get_total_ex_impuesto() {
			$total = $this->total - $this->impuesto_total - $this->envio_impuesto_total;
			if ( $total < 0 ) $total = 0;
			return $total;
		}
		
		/**
		 * gets the cart contenido total (after calculation)
		 *
		 * @return string formatted precio
		 */
		function get_carrito_total() {
			if ( ! $this->precios_incluyen_impuesto ) {
				return $this->contenido_carrito_total;
			} else {
				return $this->contenido_carrito_total + $this->impuesto_total;
			}
		}
		
		/**
		 * gets the sub total (after calculation)
		 *
		 * @params bool whether to include compound impuestos
		 * @return string formatted precio
		 */
		function get_carrito_subtotal( $compound = false ) {
			
			// If the cart has compound impuesto, we want to show the subtotal as
			// cart + shipping + non-compound impuestos (after discount)
			if ( $compound ) {
				
				return $this->contenido_carrito_total + $this->envio_total + $this->get_impuestos_total( false );
			
			// Otherwise we show cart items totals only (before discount)
			} else {
			
				// Display ex impuesto if the option is set, or precios exclude impuesto
				if ($this->mostrar_totales_ex_impuesto || !$this->precios_incluyen_impuesto ) {
					
					$return = $this->subtotal_ex_impuesto;
					
					if ( $this->impuesto_total>0 && $this->precios_incluyen_impuesto ) {
						//$return .= ' <small>'.$woocommerce->countries->ex_impuesto_or_vat().'</small>';
					}
					return $return;
					
				} else {
					
					$return = $this->subtotal;
					
					if ( $this->impuesto_total>0 && !$this->precios_incluyen_impuesto ) {
						//$return .= ' <small>'.$woocommerce->countries->inc_impuesto_or_vat().'</small>';
					}
					return $return;
				
				}
			}
		}
		
		/** 
		 * Get the product row subtotal
		 *
		 * Gets the impuesto etc to avoid rounding issues.
		 *
		 * When on the checkout (review order), this will get the subtotal based on the customer's impuesto tarifa rather than the base tarifa
		 *
		 * @params object product
		 * @params int cantidad
		 * @return string formatted precio
		 */
		function get_subtotal_producto( $_product, $cantidad ) {
			
			if ($_product->precio_rebajado > 0 && strtotime($_product->desde_precio_rebajado) < time() && strtotime($_product->hasta_precio_rebajado) > time()) {
				$precio 		= $_product->precio_rebajado;
			} else {
				$precio 		= $_product->precio;
			}
			$taxable 		= true;
			$base_impuesto_tarifas = $this->impuesto->get_tarifa_base_tienda( $_product->impuesto );
			$impuesto_tarifas 		= $this->impuesto->get_tarifas( $_product->impuesto ); // This will get the base tarifa unless we're on the checkout page
			
			// Taxable
			if ( $taxable ) {
	
				if ( $this->mostrar_carrito_ex_impuestos && $this->precios_incluyen_impuesto ) {
							
					$base_impuestos 		= $this->impuesto->calc_impuesto( $precio * $cantidad, $base_impuesto_tarifas, true );
					$base_impuesto_amount	= array_sum( $base_impuestos );
					
					$row_precio 			= ( $precio * $cantidad ) - $base_impuesto_amount;
					
					$return = $row_precio;
					//$return .= ' <small class="impuesto_label">' . $woocommerce->countries->ex_impuesto_or_vat() . '</small>';
				
				} elseif ( ! $this->mostrar_carrito_ex_impuestos && $impuesto_tarifas !== $base_impuesto_tarifas && $this->precios_incluyen_impuesto ) {
					
					$base_impuestos			= $this->impuesto->calc_impuesto( $precio * $cantidad, $base_impuesto_tarifas, true );
					$modded_impuestos		= $this->impuesto->calc_impuesto( ( $precio * $cantidad ) - array_sum( $base_impuestos ), $impuesto_tarifas, false );
					$row_precio 			= (( $precio * $cantidad ) - array_sum( $base_impuestos )) + array_sum( $modded_impuestos );
					
					$return = $row_precio;
					if ( ! $this->precios_incluyen_impuesto ) {
						//$return .= ' <small class="impuesto_label">' . $woocommerce->countries->inc_impuesto_or_vat() . '</small>';
					}
				
				} else {
					
					$row_precio = $precio * $cantidad;
					$return = $row_precio;
				
				}
			
			// Non taxable
			} else {
				
				$row_precio = $precio * $cantidad;
				$return = $row_precio;
			
			}
			
			return $return; 
			
		}
		
		/**
		 * gets the cart impuesto (after calculation)
		 *
		 * @return string formatted precio
		 */
		function get_carrito_impuesto() {
			$return = false;
			$cart_total_impuesto = $this->impuesto_total + $this->envio_impuesto_total;
			if ( $cart_total_impuesto > 0 ) $return = $cart_total_impuesto;
			return $return;
		}
		
		/**
		 * Get impuesto row amounts with or without compound impuestos includes
		 *
		 * @return float precio
		 */
		function get_impuestos_total( $compound = true ) {
			$total = 0;
			foreach ( $this->impuestos as $key => $impuesto ) {
				$total += $impuesto;
			}
			foreach ( $this->envio_impuestos as $key => $impuesto ) {
				$total += $impuesto;
			}
			return $total;
		}
		
		/**
		 * gets the total (product) discount amount - these are applied before impuesto
		 *
		 * @return mixed formatted precio or false if there are none
		 */
		function get_descuentos_antes_impuesto() {
			if ( $this->descuento_carrito ) {
				return $this->descuento_carrito; 
			}
			return false;
		}
		
		/**
		 * gets the order discount amount - these are applied after impuesto
		 *
		 * @return mixed formatted precio or false if there are none
		 */
		function get_descuentos_despues_impuesto() {
			if ( $this->descuento_total ) {
				return $this->descuento_total; 
			}
			return false;
		}
		
		/**
		 * gets the total discount amount - both kinds
		 *
		 * @return mixed formatted precio or false if there are none
		 */
		function get_descuento_total() {
			if ( $this->descuento_total || $this->descuento_carrito ) {
				return $this->descuento_total + $this->descuento_carrito; 
			}
			return false;
		}
		
		function get_totales_array(){
			$tarifa_obj = $this->impuesto->get_tarifas('estandar');
			$nombre_tarifa = $tarifa_obj[0]['nombre'];
			
			$array = array(
							"total"						=>	$this->total,
							"subtotal"					=>	$this->subtotal,
							"subtotal_ex_impuesto"		=>	$this->subtotal_ex_impuesto,
							"impuesto_total"			=>	$this->impuesto_total,
							"impuesto_nombre"			=>	$nombre_tarifa,
							"descuento_carrito"			=>	$this->descuento_carrito,
							"descuento_total"			=>	$this->descuento_total,
							"envio_total"				=>	$this->envio_total,
							"envio_impuesto_total"		=>	$this->envio_impuesto_total,
							"envio_nombre"				=>	$this->envio_nombre,
							"precios_incluyen_impuesto"	=>	$this->precios_incluyen_impuesto,
							"mostrar_totales_ex_impuesto"	=>	$this->mostrar_totales_ex_impuesto,
							"mostrar_carrito_ex_impuestos"	=>	$this->mostrar_carrito_ex_impuestos
							);
			return $array;
		}
}