<?php
/**
 * Checkout
 *
 * The Shop checkout class handles the checkout process, collecting user data and processing the payment.
 *
 * @class 		ShopCheckout
 * @version		1.0.0
 * @package		Shop/Classes
 * @category	Class
 * @author 		Mideas
 */
class ShopCheckout {
	/** @var array Array of posted form data. */
	public $posted;
	/** @var array Array of campos to display on the checkout. */
	public $checkout_campos;
	/** @var bool Whether or not the user must crear an cuenta to checkout. */
	public $debe_crear_cuenta;
	/** @var bool Whether or not signups are allowed. */
	public $permitir_signup;
	/** @var bool True when the user is creando an cuenta. */
	public $creando_cuenta;
	/** @var object The envio method being used. */
	private $metodo_envio;
	/** @var array The payment gateway being used. */
	private $metodo_pago;
	/** @var int ID of cliente. */
	private $cliente_id;
	
	private $error;
	/**
	 * Constructor for the checkout class. Hooks in methods and defines checkout campos.
	 *
	 * @access public
	 * @return void
	 */
	public function __construct () {
		global $shop;
		
		$config_shop = App::getConfigValue('ext_shop');
		//add_action( 'shop_checkout_process', array( $this,'checkout_process' ) );
		//add_action( 'shop_checkout_facturacion', array( $this,'checkout_form_facturacion' ) );
		//add_action( 'shop_checkout_envio', array( $this,'checkout_form_envio' ) );
		$this->permitir_signup         = false;
		$this->permitir_invitado_checkout = $config_shop['clientes']['permitir_invitados'] == '1' ? true : false;;
		$this->debe_crear_cuenta   = $this->permitir_invitado_checkout || Usuario::isLoged() ? false : true;
		
		// Define all Checkout campos
		$this->checkout_campos['facturacion'] 	= $shop->formularios['facturacion']->getCampos();
		$this->checkout_campos['envio'] 	= $shop->formularios['envio']->getCampos();
		$this->checkout_campos['usuario']['email'] = array(
			'type' 			=> 'text',
			'label' 		=> __( 'Account username' ),
			'placeholder' 	=> __( 'Username' )
		);
			
		if ( $config_shop['clientes']['permitir_invitados'] == '1' && !Usuario::isLoged() ) {
			$this->checkout_campos['usuario']['password'] = array(
				'type' 				=> 'password',
				'label' 			=> __( 'Account password' ),
				'placeholder' 		=> __( 'Password' ),
				'class' 			=> array( 'form-row-first' )
			);
	
			$this->checkout_campos['usuario']['password-2'] = array(
				'type' 				=> 'password',
				'label' 			=> __( 'Account password' ),
				'placeholder' 		=> __( 'Password' ),
				'class' 			=> array( 'form-row-last' ),
				'label_class' 		=> array( 'hidden' )
			);
		}
		$this->checkout_campos['pedido']	= array(
			'comentarios_pedido' => array(
				'type' => 'textarea',
				'class' => array('notes'),
				'label' => __( 'Pedido Notes' ),
				'placeholder' => __('Notes about your pedido, e.g. special notes for delivery.' )
				)
			);
			
		//$this->checkout_campos = apply_filters( 'shop_checkout_campos', $this->checkout_campos );
		//do_action( 'shop_checkout_init', $this );
	}
	/**
	 * Checkout process
	 *
	 * @access public
	 * @return void
	 */
	public function checkout_process() {
		// When we process the checkout, lets ensure cart items are rechecked to prevent checkout
		do_action('shop_check_cart_items');
	}
	/**
	 * Output the facturacion information form
	 *
	 * @access public
	 * @return void
	 */
	public function checkout_form_facturacion() {
		shop_get_template( 'checkout/form-facturacion.php', array( 'checkout' => $this ) );
	}
	/**
	 * Output the envio information form
	 *
	 * @access public
	 * @return void
	 */
	public function checkout_form_envio() {
		shop_get_template( 'checkout/form-envio.php', array( 'checkout' => $this ) );
	}
	/**
	 * crear_pedido function.
	 *
	 * @access public
	 * @return void
	 */
	public function crear_pedido() {
		global $shop;
		
		$config_shop = App::getConfigValue('ext_shop');
		// Give plugins the opportunity to crear an pedido themselves
		//$pedido_id = apply_filters( 'shop_crear_pedido', null, $this );
		if ( is_numeric( $pedido_id ) )
			return $pedido_id;
			
		// Create Pedido (send cart variable so we can record items and reduce inventory). Only crear if this is a new pedido, not if the payment was rejected.
		$pedido_data = array(
			'titulo' 	=> sprintf( __( 'Pedido &ndash; %s' ), strftime( __( '%b %d, %Y @ %I:%M %p' ) ) ),
			'estado' 	=> $GLOBALS['opciones']['ext_shop']['venta']['estado_creado'],
			'comentarios' 	=> isset( $this->posted['pedido']['comentarios_pedido'] ) ? $this->posted['pedido']['comentarios_pedido'] : ''
		);
		// Insert or update the post data
		$crear_new_pedido = true;
		if ( Helpers::getSesion()->get('shop.pedido_esperando_pago') > 0 ) {
			$pedido_id = Helpers::getSesion()->get('shop.pedido_esperando_pago');
			
			$pedido_tmp = new PedidoShop();
			$pedido_tmp->setid($pedido_id);
			$pedido_tmp = $pedido_tmp->BuscarId();
			/* Check pedido is unpaid by getting its status */
			$pedido_status = isset( $pedido_tmp->estado ) ? $pedido->estado : $GLOBALS['opciones']['ext_shop']['venta']['estado_creado'];
			// Resume the unpaid pedido if its pending
			if ( $pedido_status == $GLOBALS['opciones']['ext_shop']['venta']['estado_creado'] || $pedido_status == $GLOBALS['opciones']['ext_shop']['venta']['estado_fallido'] ) {
				// Update the existing pedido as we are resuming it
				$crear_new_pedido = false;
				$pedido_data['ID'] = $pedido_id;
			}
		}
		if ( $crear_new_pedido ) {
			$pedido_tmp = new PedidoShop();
			
			$pedido_tmp->settitulo($pedido_data['titulo']);
			$pedido_tmp->setestado($pedido_data['estado']);
			$pedido_tmp->setcomentarios($pedido_data['comentarios']);
			$pedido_tmp->setemail($this->posted['usuario']['email']);
			
			$datos_facturacion = array();
			if ( $this->checkout_campos['facturacion'] ) {
				$datos_facturacion = $this->posted['facturacion'];
			}
			$datos_facturacion = json_encode($datos_facturacion);
			
			$datos_envio = array();
			if ( $this->checkout_campos['envio'] && ( $shop->carrito->necesita_envio() || $this->posted['envioafacturacion'] == 0 ) ) {
				$datos_envio = $this->posted['envio'];
			}
			$datos_envio = json_encode($datos_envio);
			
			$datos_metodo_pago = array();
			$datos_metodo_pago["id"] = $this->metodo_pago->id;
			$datos_metodo_pago["titulo"] = $this->metodo_pago->titulo[SistemaIdioma::getIdiomaSesion()];
			$datos_metodo_pago["estado"] = "pending";
			$datos_metodo_pago = json_encode($datos_metodo_pago);
			
			$datos_metodo_envio = array();
			$datos_metodo_envio["id"] = $this->metodo_envio->id;
			$datos_metodo_envio["nombre"] = SistemaIdioma::Imprimir($this->metodo_envio->nombre);
			$datos_metodo_envio["coste"] = $this->metodo_envio->coste;
			$datos_metodo_envio["peso"] = str_replace(',', '.', number_format($shop->carrito->contenido_carrito_peso, 2));
			$datos_metodo_envio = json_encode($datos_metodo_envio);
			
			$pedido_tmp->setenvio_a_facturacion($this->posted['envioafacturacion']);
			$pedido_tmp->setdatos_facturacion($datos_facturacion);
			$pedido_tmp->setdatos_envio($datos_envio);
			$pedido_tmp->setmetodo_pago($datos_metodo_pago);
			$pedido_tmp->setmetodo_envio($datos_metodo_envio);
			
			$totales_array = $shop->carrito->get_totales_array();
			$totales_array = json_encode($totales_array);
			
			$pedido_tmp->settotal($totales_array);
			
			$pedido_tmp->setusuario_id($this->cliente_id);
			$reglas_tmp = array();
			if (is_array($shop->carrito->reglas_aplicadas) && sizeof($shop->carrito->reglas_aplicadas) > 0){
				foreach ($shop->carrito->reglas_aplicadas as $kreg=>$reg){
					$tmp_reg_obj = new ReglaPrecio();
					$tmp_reg_obj->setid($kreg);
					if ($tmp_reg_obj = $tmp_reg_obj->BuscarId()){
						if ($tmp_reg_obj->cumpleCondicion()){
							
							$reglas_tmp[] = serialize($tmp_reg_obj);
							
							if ($tmp_reg_obj->esCupon()){
								$tmp_reg_obj->quitarCuponSesion($tmp_reg_obj->id);
							}
							
							$tmp_reg_obj->setusado($tmp_reg_obj->usado+1);
							
							$tmp_reg_obj->Modificar();
							
						}
					}
				}
			}
			
			$pedido_tmp->setreglas_aplicadas(json_encode($reglas_tmp));
			
			if ($pedido_id = $pedido_tmp->Guardar()){
			} else {
				$this->setMsg(__( 'Error: Unable to crear pedido. Please try again.' ));
				return false;
			}
		}
		
		$shop->carrito->calcular_totales();
		
		// Store the line items to the new/resumed pedido
		foreach ( $shop->carrito->get_carrito() as $cart_item_key => $values ) {
			
			$_product = $values['data'];
			
			$item = new PedidoLineaShop();
			
			$item->setpedido_id($pedido_id);
			$item->setproduct_id($values['product_id']);
			$item->setvariation_id($values['variation_id']);
			$item->setvariation(json_encode($values['variation']));
			$item->setcantidad($values['cantidad']);
			$item->setdata(serialize($values['data']));
			$item->setlinea_total(number_format($values['linea_total'], 2, '.', ''));
			$item->setlinea_impuesto(number_format($values['linea_impuesto'], 2, '.', ''));
			$item->setlinea_subtotal(number_format($values['linea_subtotal'], 2, '.', ''));
			$item->setlinea_subtotal_impuesto(number_format($values['linea_subtotal_impuesto'], 2, '.', ''));
			
			if ($item->Guardar()){
			} else {
				$this->setMsg(__('No se ha podido guardar la línea del pedido.'), 'error');
				return false;
			}
		}
		
		$p_tmp = new PedidoShop();
		$p_tmp->setid($pedido_id);
		$p_tmp = $p_tmp->BuscarId();
		$p_tmp->add_nota_pedido(sprintf(__('Pedido # %s creado con éxito el %s'), $p_tmp->id, strftime('%d-%m-%Y a las %H:%M', strtotime($p_tmp->fecha))));
		
		if (in_array('nuevo', $config_shop['notificaciones']['notificar'])) {
			$p_tmp->enviar_email_pedido('nuevo');
			//$p_tmp->PlaceShippingOrder();
		}
		return $pedido_id;
	}
	/**
	 * Process the checkout after the confirm pedido button is pressed
	 *
	 * @access public
	 * @return void
	 */
	public function process_checkout() {
		global $shop, $current_user;
		if ( ! defined( 'SHOP_CHECKOUT' ) )
			define( 'SHOP_CHECKOUT', true );
		// Prevent timeout
		@set_time_limit(0);
		if ( sizeof( $shop->carrito->get_carrito() ) == 0 )
			$this->setMsg( sprintf( __( 'La sesión ha expirado. <a href="%s">Volver al inicio de la tienda &rarr;</a>' ), Helpers::resolveLink(false, 'ext_shop', 'categoria_productos', __('Tienda'), $GLOBALS['opciones']['ext_shop']['shop']['categoria_defecto'][0]) ) );
		// Checkout campos (not defined in checkout_campos)
		$this->posted['envioafacturacion'] 		= isset( $_POST['envio_igual_facturacion'] ) ? 1 : 0;
		$this->posted['terms'] 				= isset( $_POST['terms'] ) ? 1 : 0;
		$this->posted['crearcuenta'] 		= isset( $_POST['crearcuenta'] ) ? 1 : 0;
		$this->posted['metodo_pago'] 	= isset( $_POST['pasarela_pago'] ) ? $_POST['pasarela_pago'] : '';
		$this->posted['metodo_envio']	= isset( $_POST['metodo_envio'] ) ? $_POST['metodo_envio'] : '';
		$this->posted['invitado']	= $_POST['usuario']['invitado'] == '1'  ? 1 : 0;
		// Update cliente envio and payment method to posted method
		Helpers::getSesion()->set('shop._metodo_envio_elegido', $this->posted['metodo_envio']);
		Helpers::getSesion()->set('shop._metodo_pago_elegido', $this->posted['metodo_pago']);		
		// Note if we skip envio
		$skipped_envio = false;
		// Get posted checkout_campos and do validation
		foreach ( $this->checkout_campos as $camposet_key => $camposet ) {
			// Skip envio if its not needed
			if ( $camposet_key == 'envio' && ( /*$shop->carrito->ship_to_facturacion_address_only() || */$this->posted['envioafacturacion'] || ( ! $shop->carrito->necesita_envio()/* && get_option('shop_require_envio_address') == 'no' */) ) ) {
				$skipped_envio = true;
				continue;
			}
			
			foreach ( $camposet as $key => $campo ) {
				
				if (is_array($campo)){
					$campo_obj = new stdClass;
					$campo_obj->nombre = $key;
					$campo_obj->label = $campo['label'];
					$campo_obj->tipo = $campo['type'];
					
					$campo = $campo_obj;
				}
				if ( ! isset( $campo->tipo ) ) $campo->tipo = 'text';
				// Get Value
				switch ( $campo->tipo ) {
					case "checkbox" :
						$this->posted[$camposet_key][ $campo->nombre ] = isset( $_POST[$camposet_key][$campo->nombre] ) ? 1 : 0;
					break;
					default :
						$this->posted[$camposet_key][ $campo->nombre ] = isset( $_POST[$camposet_key][$campo->nombre] ) ? $_POST[$camposet_key][ $campo->nombre ] : '';
					break;
				}
				// Validation: Required campos
				if ( isset( $campo->obligatorio ) && $campo->obligatorio && empty( $this->posted[$camposet_key][$campo->nombre] ) ) $this->setMsg('<strong>' . SistemaIdioma::Imprimir($campo->label) . '</strong> ' . __('es un campo obligatorio.'), 'error');
				//if ( ! empty( $this->posted[$camposet_key][$campo->nombre] ) ) {
					// Special handling for validation and formatting
					switch ( $campo->nombre ) {
						case "codigo_postal" :
						case "envio_codigo_postal" :
							$this->posted[$camposet_key][$campo->nombre] = strtoupper( str_replace( ' ', '', $this->posted[$camposet_key][$campo->nombre] ) );
							if (empty($this->posted[$camposet_key][$campo->nombre])){
								$this->setMsg( __( 'Debe escribir un código postal' ) );
								return false;
							}
						break;
						case "provincia" :
						case "envio_provincia" :
							if (empty($this->posted[$camposet_key][$campo->nombre])){
								$this->setMsg( __( 'Debe elegir una provincia' ) );
								return false;
							}
						break;
						case "pais" :
						case "envio_pais" :
							if (empty($this->posted[$camposet_key][$campo->nombre])){
								$this->setMsg( __( 'Debe elegir un país' ) );
								return false;
							}
						break;
						case "localidad" :
						case "envio_localidad" :
							if (empty($this->posted[$camposet_key][$campo->nombre])){
								$this->setMsg( __( 'Debe elegir una localidad' ) );
								return false;
							}
						break;
						case "telefono" :
						case "envio_telefono" :
							if (empty($this->posted[$camposet_key][$campo->nombre])){
								$this->setMsg( __( 'Debe escribir un teléfono' ) );
								return false;
							}
						break;
						case "facturacion_email" :
							$this->posted[ $key ] = strtolower( $this->posted[$key] );
							if ( ! $validation->is_email( $this->posted[$key] ) )
								$this->setMsg( '<strong>' . $campo['label'] . '</strong> ' . __( 'no es una dirección de email válida.' ) );
						break;
					}
				//}
			}
		}
		// Update cliente location to posted location so we can correctly check available envio methods
		$sess_usr = array();
		
		if ( isset( $this->posted['facturacion']['pais'] ) )
			$sess_usr['pais'] = $this->posted['facturacion']['pais'];
		if ( isset( $this->posted['facturacion']['provincia'] ) )
			$sess_usr['provincia'] = $this->posted['facturacion']['provincia'];
		if ( isset( $this->posted['facturacion']['codigo_postal'] ) )
			$sess_usr['codigo_postal'] = $this->posted['facturacion']['codigo_postal'];
		// Shipping Information
		if ( ! $skipped_envio ) {
			// Update cliente location to posted location so we can correctly check available envio methods
			if ( isset( $this->posted['envio']['envio_pais'] ) )
				$sess_usr['envio_pais'] = $this->posted['envio']['envio_pais'];
			if ( isset( $this->posted['envio']['envio_provincia'] ) )
				$sess_usr['envio_provincia'] = $this->posted['envio']['envio_provincia'];
			if ( isset( $this->posted['envio']['codigo_postal'] ) )
				$sess_usr['envio_codigo_postal'] = $this->posted['envio']['envio_codigo_postal'];
		} else {
			// Update cliente location to posted location so we can correctly check available envio methods
			if ( isset( $this->posted['facturacion']['pais'] ) )
				$sess_usr['pais'] = $this->posted['facturacion']['pais'];
			if ( isset( $this->posted['facturacion']['provincia'] ) )
				$sess_usr['provincia'] = $this->posted['facturacion']['provincia'];
			if ( isset( $this->posted['facturacion']['codigo_postal'] ) )
				$sess_usr['codigo_postal'] = $this->posted['facturacion']['codigo_postal'];
		}
		
		Helpers::getSesion()->set('shop.usuario', $sess_usr);
		// Update cart totals now we have cliente address
		$shop->carrito->calcular_totales();
		// Handle cuentas
		$this->es_invitado = false;
		if ( Usuario::isLoged() )
			$this->creando_cuenta = false;
		elseif ( ! empty( $this->posted['crearcuenta'] ) )
			$this->creando_cuenta = true;
		elseif ($this->debe_crear_cuenta)
			$this->creando_cuenta = true;
		elseif ($this->posted['invitado'] == 1) {
			$this->creando_cuenta = false;
			$this->es_invitado = true;
		} else
			$this->creando_cuenta = false;
		if ( $this->creando_cuenta ) {
			
			if ( $shop->config['clientes']['solicitar_username'] == '1' ) {
				if ( empty( $this->posted['usuario']['username'] ) )
					$this->setMsg( __( 'Por favor, escribe un nombre de usuario.' ) );
				// Check the username
				if ( Usuario::checkUsername( $this->posted['usuario']['username'] ) == false )
					$this->setMsg( __( 'Ya existe una cuenta registrada con ese nombre de usuario. Por favor, elige otro nombre de usuario' ) );
			} else {
				$this->posted['usuario']['username'] = $this->posted['usuario']['email'];
			}
			// Validate passwords
			if ( empty($this->posted['usuario']['password']) )
				$this->setMsg( __( 'Por favor, escribe una contraseña.' ) );
			if ( $this->posted['usuario']['password-2'] !== $this->posted['usuario']['password'] )
				$this->setMsg( __( 'Las contraseñas no coinciden.' ) );
				
			$checking_email = Usuario::checkEmail($this->posted['usuario']['email']);
			
			// Check the e-mail address
			if ( $checking_email == false ){
				$this->setMsg( __( 'Ya existe una cuenta registrada con esa dirección de email. Por favor, identifícate en la web con tus datos de acceso o cambia la dirección de email.' ) );
			}
		} elseif ($this->es_invitado == true){
			$this->posted['usuario']['username'] = $this->posted['usuario']['email'];
			
			$checking_email = Usuario::checkEmail($this->posted['usuario']['email']);
			
			// Check the e-mail address
			if ( $checking_email == false ){
				$this->setMsg( __( 'Ya existe una cuenta registrada con esa dirección de email. Por favor, identifícate en la web con tus datos de acceso o cambia la dirección de email.' ) );
			}
		}
		// Terms
		if ( ! isset( $_POST['shop_checkout_update_totals'] ) && empty( $this->posted['terms'] ) && $shop->config['clientes']['aceptar_condiciones'] == "1" )
			$this->setMsg( __( 'Debes aceptar nuestros Términos y Condiciones.' ) );
		if ( $shop->carrito->necesita_envio() ) {
			
			// Shipping Method
			$envios = $shop->carrito->calcular_envio();
			$available_methods = $envios->get_metodos_envio_disponibles();
			if (! isset( $available_methods[ $this->posted['metodo_envio'] ] )){
				
				$shop->carrito->calcular_totales();
				$shop->carrito->set_session();
				
				if (isset($available_methods[ Helpers::getSesion()->get('shop._metodo_envio_elegido') ])){
					$this->metodo_envio = $available_methods[ Helpers::getSesion()->get('shop._metodo_envio_elegido') ];
				} else {
					$this->metodo_envio = '';
					$this->setMsg( __( 'No existe ningún método de envío para tu dirección de envío. Por favor, elige otra dirección o ponte en contacto con nosotros a través de la página Contacto.' ) );
				}
			} else {
				$this->metodo_envio = $available_methods[ $this->posted['metodo_envio'] ];
			}
		}
		if ( $shop->carrito->necesita_pago() ) {
			
			// Payment Method
			$available_gateways = $shop->pago->get_available_pasarelas_pago();
			
			if ( ! isset( $available_gateways[ $this->posted['metodo_pago'] ] ) ) {
				$this->metodo_pago = '';
				$this->setMsg( __( 'Método de pago no válido.' ) );
			} else {
				$this->metodo_pago = $available_gateways[ $this->posted['metodo_pago'] ];
				//$this->metodo_pago->validate_campos();
			}
		}
		
		// Abort if errors are present
		if ( $this->hasError() ){
			return false;
		}
		if ( ! isset( $_POST['shop_checkout_update_totals'] ) /*&& $shop->error_count() == 0*/ ) {
			$this->cliente_id = Usuario::getLogedId();
			
			try {
				// Create cliente cuenta and log them in
				if ( $this->creando_cuenta && ! $this->cliente_id ) {
					if ( !$this->hasError() ) {
		                $user_pass = $this->posted['usuario']['password'];
		                $new_cliente_data = array(
		                	'user_login' => $this->posted['usuario']['username'],
		                	'user_pass'  => $user_pass,
		                	'user_email' => $this->posted['usuario']['email'],
		                	'nivel_acceso'       => 1,
							'estado'	=> 0
		                );
						
						$token = md5(uniqid(rand(), true));
						
						$cliente_obj = new Usuario();
						$cliente_obj->setusername($new_cliente_data["user_login"]);
						$cliente_obj->setemail($new_cliente_data["user_email"]);
						$cliente_obj->setcontrasena($user_pass);
						$cliente_obj->setnivel_acceso($new_cliente_data["nivel_acceso"]);
						$cliente_obj->setestado($new_cliente_data["estado"]);
						$cliente_obj->settoken($token);
						
						$cliente_res = $cliente_obj->Guardar();
						
						if ($cliente_res == true){
							$this->cliente_id = mysql_insert_id();
							
							$enlace_activacion = Helpers::resolveLink(false, 'ext_usuarios', 'login', __('Validar_usuario'));
	
							$emailer = new Emailer();
						
							$to_array = array($new_cliente_data["user_email"]=>utf8_decode($new_cliente_data["user_login"]));
							$asunto = __('Activación de cuenta de usuario');
							
							$emailer->to = $to_array;
							$emailer->asunto = $asunto;
							
							$emailer->contenido = '<h1>'.$asunto.'</h1>';
							$emailer->contenido .= '<p>'.__('Para activar su cuenta de usuario, haga click en el siguiente enlace:');
							$emailer->contenido .= ' <a href="'.$enlace_activacion.'/?token='.$token.'">'.$enlace_activacion.'/?token='.$token.'</a>';
							
							if ($emailer->enviar()){
							} else {
								$this->setMsg(sprintf(__('Se ha producido un error al enviar el email de confirmación: %s'), $emailer->mailer->ErrorInfo), 'error');
								return false;
							}
						} else {
							$this->setMsg(__( 'No se pudo registrar; por favor, ponte en contacto con nosotros si sigues teniendo problemas.'));
							return false;
						}
						
					} else {
						return false;
					}
				}
				
				// Abort if errors are present
				if ( $this->hasError() ){
					//echo App::printMsg();
					return false;
				}
				
				// Create the pedido
				$pedido_id = $this->crear_pedido();
				
				$usr_id = Usuario::getLogedId();
				$usr_obj = Usuario::getLoged();
				if ($usr_id){
					$facturacion_pedido = $this->posted['facturacion'];
					$envio_pedido = $this->posted['envio'];
					
					$client = new ClienteShop();
					$client->setid_usuario($usr_id);
					if ($client = $client->BuscarIdUsuario()){
						$client->setdatos_facturacion(json_encode($facturacion_pedido));
						$client->setdatos_envio(json_encode($envio_pedido));
						$client->setdireccion_envio_igual_facturacion($this->posted['envioafacturacion']);
						$client->Modificar();
					} else {
						$client = new ClienteShop();
						$client->setid_usuario($usr_id);
						$client->setemail($usr_obj->email);
						$client->setestado(1);
						$client->setdatos_facturacion(json_encode($facturacion_pedido));
						$client->setdatos_envio(json_encode($envio_pedido));
						$client->setdireccion_envio_igual_facturacion($this->posted['envioafacturacion']);
						$client->Guardar();
					}
				}
				// Process payment
				if ( $shop->carrito->necesita_pago() ) {
					// Store Pedido ID in session so it can be re-used after payment failure
					Helpers::getSesion()->set('shop.pedido_esperando_pago', $pedido_id);
					// Process Payment
					$result = $available_gateways[ $this->posted['metodo_pago'] ]->procesar_pago( $pedido_id );
					// Redirect to success/confirmation/payment page
					if ( $result['result'] == 'info' || $result['result'] == 'success' ) {
						Helpers::redirect( $result['redirect'] );
						exit;
					}
				} else {
					if ( empty( $pedido ) ){
						$pedido = new PedidoShop();
						$pedido->setid($pedido_id);
						$pedido = $pedido->BuscarId();
					}
					// No payment was required for pedido
					$pedido->pago_completo();
					// Empty the Cart
					$shop->carrito->vaciar_carrito();
					// Get redirect
					$return_url = get_permalink( shop_get_page_id( 'thanks' ) );
					$return_url = add_query_arg( 'key', $pedido->pedido_key, add_query_arg( 'pedido', $pedido->id, $return_url ) );
					// Redirect to success/confirmation/payment page
					if ( is_ajax() ) {
						echo json_encode(
							array(
								'result' 	=> 'info',
								'redirect' => apply_filters( 'shop_checkout_no_payment_needed_redirect', $return_url, $pedido)
							)
						) . '<!--WC_END-->';
						exit;
					} else {
						wp_safe_redirect(
							apply_filters( 'shop_checkout_no_payment_needed_redirect', $return_url, $pedido)
						);
						exit;
					}
				}
			} catch ( Exception $e ) {
				if ( ! empty( $e ) )
					$this->setMsg( $e );
			}
		} // endif
	}
	
	private function setMsg($txt,$tipo='error'){
		if (empty($txt) || empty($tipo))
			return false;
			
		if ($tipo == 'error')
			$this->error = true;
			
		App::setMsg($txt,$tipo);
		
		return true;
	}
	
	private function hasError(){
		if ($this->error==true)
			return true;
		else
			return false;
	}
	/**
	 * Gets the value either from the posted data, or from the users meta data
	 *
	 * @access public
	 * @param string $input
	 * @return string
	 */
	public function get_value( $input ) {
		global $shop;
		if ( ! empty( $_POST[ $input ] ) ) {
			return esc_attr( $_POST[ $input ] );
		} else {
			if ( is_user_logged_in() ) {
				$current_user = wp_get_current_user();
				if ( $meta = get_user_meta( $current_user->ID, $input, true ) )
					return $meta;
				if ( $input == "facturacion_email" )
					return $current_user->user_email;
			}
			$default_facturacion_pais = apply_filters('default_checkout_pais', ($shop->cliente->get_pais()) ? $shop->cliente->get_pais() : $shop->paises->get_base_pais());
			$default_envio_pais = apply_filters('default_checkout_pais', ($shop->cliente->get_envio_pais()) ? $shop->cliente->get_envio_pais() : $shop->paises->get_base_pais());
			if ( $shop->cliente->has_calculated_envio() ) {
				$default_facturacion_state  = apply_filters('default_checkout_state', $shop->cliente->get_state());
				$default_envio_state = apply_filters('default_checkout_state', $shop->cliente->get_envio_state());
			} else {
				$default_facturacion_state  = apply_filters('default_checkout_state', '');
				$default_envio_state = apply_filters('default_checkout_state', '');
			}
			if ( $input == "facturacion_pais" )
				return $default_facturacion_pais;
			if ( $input == "facturacion_state" )
				return $default_facturacion_state;
			if ( $input == "facturacion_postcode" )
				return $shop->cliente->get_postcode() ? $shop->cliente->get_postcode() : '';
			if ( $input == "envio_pais" )
				return $default_envio_pais;
			if ( $input == "envio_state" )
				return $default_envio_state;
			if ( $input == "envio_postcode" )
				return $shop->cliente->get_envio_postcode() ? $shop->cliente->get_envio_postcode() : '';
		}
	}
}
