<?php
/**
 * Shop Extensions Connector Gateway class
 *
 * Extended by individual extensions connectors gateways to handle virtual products.
 *
 * @class 		ShopConector
 * @version		1.6.4
 * @package		Shop/Abstracts
 * @category	Abstract Class
 * @author 		Mideas
 */
class ShopConector {
	/** @var string Extension Connector method ID. */
	var $id;
	/** @var string Extension Connector method titulo. */
	var $titulo;
	/** @var string Extension carpeta. */
	var $extension;
	
	/** @var string Objeto Extension. */
	var $obj_extension;
	
	/** @var array Acciones del conector. */
	var $acciones;
	
	function is_available(){
		c_include('extension');
		$tmp_ext = Extension::BuscarPorCarpeta($this->extension);
		if (file_exists(CONF_REALPATH_APP . 'extensions/' . $this->extension . '/' . $this->id . '.php') && $tmp_ext != false && $tmp_ext->activo == 1){
			$this->obj_extension = $tmp_ext;
			return true;
		}
	}
	
	protected static function loadFieldClass($entity, $type)
	{
		$class = 'Form'.ucfirst($entity).ucfirst($type);
		if (class_exists($class)) return $class;
		// Get the field search path array.
		$paths = array();
		$paths[] = CONF_REALPATH_APP.'includes/class/forms/fields';
		// If the type is complex, add the base type to the paths.
		if ($pos = strpos($type, '_')) {
			// Add the complex type prefix to the paths.
			for ($i = 0, $n = count($paths); $i < $n; $i++) {
				// Derive the new path.
				$path = $paths[$i].'/'.strtolower(substr($type, 0, $pos));
				// If the path does not exist, add it.
				if (!in_array($path, $paths)) {
					array_unshift($paths, $path);
				}
			}
			// Break off the end of the complex type.
			$type = substr($type, $pos+1);
		}
		
		// Try to find the class file.
		$file = $paths[0].'/'.strtolower($type).'.php';
		if (is_file($file)) {
			require_once $file;
		}
		
		// Check for all if the class exists.
		return class_exists($class) ? $class : false;
	}
	
	function renderCampo($campo=null,$valor='',$grupo=''){
		if (empty($campo['type']))
			return false;
			
		if ($campo['type'] == 'titulo'){
			return '<h4>'.$campo['titulo'].'</h4>';
		}
			
		c_include('forms/formField');
		
		$ff = self::loadFieldClass('field', $campo['type']);
		
		if (class_exists($ff)){
			$ff_obj = new $ff;
			
			$nombre = Helpers::parseaTexto($campo['titulo']);
			$id = Helpers::parseaTexto($campo['titulo']);
			
			if (!empty($grupo)){
				$nombre = $grupo."[".$nombre."]";
				//$id = $grupo.'_'.$id;
			}
			
			$ff_obj->__set('id', $id);
			$ff_obj->__set('name', $nombre);
			$ff_obj->__set('fieldname', $nombre);
			$ff_obj->__set('label', $campo['label']);
			$ff_obj->__set('description', $campo['descripcion']);
			if ($valor != ''){
				$ff_obj->__set('value', $valor);
			} /*elseif (!empty($campo['default'])){
				$ff_obj->__set('value', $campo['default']);
				$valor = $campo['default'];
			}*/
			
			//$ff_obj->__set('validate', $this->tipovalidacion);
			
			if ($campo['type'] == 'hidden'){
				$ff_obj->__set('hidden', true);
			}
			//$params_lng = SistemaIdioma::Imprimir($this->params);
			//parse_str($params_lng, $element);
			
			$element = $campo['attributes'];
			
			$element["name"] = $nombre;
			$element["type"] = $campo['type'];
			$element["label"] = $campo['label'];
			$element["description"] = $campo['descripcion'];
			$element["value"] = $campo['default'];
			//$element["validate"] = $this->tipovalidacion;
	
			$element_xml = new SimpleXMLElement('<root></root>');
			
			foreach (array_keys($element) as $key){
				$cxml = $element_xml->addAttribute($key, $element[$key]);
			}
			
			$ff_obj->element = $element_xml;
			
			$input = $ff_obj->getInput($valor);
			
			if (empty($formato_label)){
				$formato_label = '<label for="##ID##">##LABEL##</label>
				';
			}
			if (empty($formato_input)){
				$formato_input = '##INPUT##
				';
			}
			
			switch ($campo['type']){ 
				case 'hidden':
					$formato_label = '';
					$formato_input = "##INPUT##";
					break;
				case 'submit':
					$formato_label = '';
					break;
				case 'checkbox':
					$formato_label = '<label for="##ID##">##INPUT## ##LABEL##</label>';
					$formato_input = ' ';
					break;
				case 'button':
					$formato_label = '';
					break;
			}
			$formato = $formato_label.$formato_input;
			$out = str_replace("##ID##", $id, $formato);
			$out = str_replace("##LABEL##", $campo['label'], $out);
			$out = str_replace("##INPUT##", $input, $out);
			
			return $out;
		}
	}
	
	/**
     * Generate Settings HTML.
     *
     * Generate the HTML for the fields on the "settings" screen.
     *
     * @access public
     * @param bool $form_fields (default: false)
     * @since 1.0.0
     * @uses method_exists()
	 * @access public
	 * @return string the html for the settings
     */
    public function generate_settings_html ( $form_fields = false ) {
    	if ( ! $form_fields ) {
    		$form_fields = $this->form_fields;
    	}
    	$html = '';
    	foreach ( $form_fields as $k => $v ) {
    		if ( ! isset( $v['type'] ) || ( $v['type'] == '' ) ) { $v['type'] == 'text'; } // Default to "text" field type.
			
			$opt_v = $GLOBALS['opciones']['ext_shop_conectores'][$this->id][$k];
			
			
			
			if (is_null($opt_v) && $v["type"] == "checkbox" && isset($GLOBALS['opciones']['ext_shop_conectores'][$this->id])){
				//$opt_v = null;
			}
			
    		$html .= $this->renderCampo($v,$opt_v,'conector_extension');
    	}
    	return $html;
    }
}