<?php
/**
 * Shop Payment Gateways class
 *
 * Loads payment conectors via hooks for use in the store.
 *
 * @class 		ConectoresExtensiones
 * @version		1.0.0
 * @package		Shop/Classes/Payment
 * @category	Class
 * @author 		Mideas
 */
defined('CONF_URL_APP') or die;
class ConectoresExtensiones {
	/** @var array Array of payment conector classes. */
	var $conectores_extensiones;
	/**
	 * __construct function.
	 *
	 * @access public
	 * @return void
	 */
	public function __construct() {
		$this->init();
	}
    /**
     * Load conectors and hook in functions.
     *
     * @access public
     * @return void
     */
    function init() {
		$config_shop = App::getConfigValue('ext_shop');
    	$load_conectors = $this->load_conectores_extensiones();
		// Get order option
		/*$ordering 	= (array) get_option('woocommerce_conector_order');
		$order_end 	= 999;
		// Load conectors in order
		foreach ($load_conectors as $conector) :
			$load_conector = new $conector();
			if (isset($ordering[$load_conector->id]) && is_numeric($ordering[$load_conector->id])) :
				// Add in position
				$this->conectores_extensiones[$ordering[$load_conector->id]] = $load_conector;
			else :
				// Add to end of the array
				$this->conectores_extensiones[$order_end] = $load_conector;
				$order_end++;
			endif;
		endforeach;*/
		ksort( $this->conectores_extensiones );
    }
	
	private function load_conectores_extensiones() {
		if ( sizeof( $this->conectores_extensiones ) == 0 ){
			$path = CONF_REALPATH_APP . 'extensions/ext_shop/class/conectores_extensiones';
			$files = Helpers::dir_list($path, '','.class.php',false);
			
			if (sizeof( $files[CONF_REALPATH_APP . 'extensions/ext_shop/class/conectores_extensiones'] ) > 0){
				foreach ($files[CONF_REALPATH_APP . 'extensions/ext_shop/class/conectores_extensiones'] as $conector_class){
					$conector_name = substr($conector_class, 0, strlen($conector_class) - 10);
					$conector_class_name = "ConectoresExtensiones" . ucfirst($conector_name);
			
					include_once($path.'/'.$conector_class);
					
					$dummy_class = new $conector_class_name();
					
					$this->conectores_extensiones[ $dummy_class->id ] = $dummy_class;
				}
			}
		}
		
		return true;
	}
    /**
     * Get conectors.
     *
     * @access public
     * @return array
     */
    function conectores_extensiones() {
		$_available_conectores = array();
		if ( sizeof( $this->conectores_extensiones ) > 0 )
			foreach ( $this->conectores_extensiones as $conector )
				$_available_conectores[ $conector->id ] = $conector;
		return $_available_conectores;
	}
	/**
	 * Get available conectors.
	 *
	 * @access public
	 * @return array
	 */
	function get_available_conectores_extensiones() {
		$_available_conectores = array();
		foreach ( $this->conectores_extensiones as $conector ) :
			if ($conector->is_available()) $_available_conectores[$conector->id] = $conector;
		endforeach;
		return $_available_conectores;
	}
	/**
	 * Save options in admin.
	 *
	 * @access public
	 * @return void
	 */
	function process_admin_options() {
		$default_conector = ( isset( $_POST['default_conector'] ) ) ? esc_attr( $_POST['default_conector'] ) : '';
		$conector_order = ( isset( $_POST['conector_order'] ) ) ? $_POST['conector_order'] : '';
		$order = array();
		if ( is_array( $conector_order ) && sizeof( $conector_order ) > 0 ) {
			$loop = 0;
			foreach ( $conector_order as $conector_id ) {
				$order[ esc_attr( $conector_id ) ] = $loop;
				$loop++;
			}
		}
		update_option( 'woocommerce_default_conector', $default_conector );
		update_option( 'woocommerce_conector_order', $order );
	}
}