<?php
/**
 * Free Shipping Method
 *
 * A simple shipping method for free shipping
 *
 * @class 		EnvioGratis
 * @version		2.0.0
 * @package		Shop/Classes/Shipping
 * @author 		Mideas
 */
class EnvioGratis extends MetodoEnvio {
	/**
	 * __construct function.
	 *
	 * @access public
	 * @return void
	 */
	function __construct() {
        $this->id 			= 'envioGratis';
        $this->titulo_metodo = __( 'Envío gratuito' );
		$this->init();
    }
    /**
     * init function.
     *
     * @access public
     * @return void
     */
    function init() {
		// Load the settings.
		//$this->init_form_fields();
		$this->init_settings();
		// Define user set variables
		$this->id			= $this->settings['id'];
        $this->activo		= $this->settings['activo'];
		$this->titulo 		= $this->settings['titulo'];
		$this->cuota_minima 	= isset($this->settings['cuota_minima']) ? $this->settings['cuota_minima'] : ''; 
		$this->tasa_manipulacion 	= isset($this->settings['tasa_manipulacion']) ? $this->settings['tasa_manipulacion'] : ''; 
		$this->disponibilidad = $this->settings['disponibilidad'];
		$this->paises 	= json_decode($this->settings['paises']);
		$this->zona = $this->settings['zona'];
		$this->opciones 		= isset($this->settings['opciones']) ? $this->settings['opciones'] : '';
		
		// Get opciones
		$this->opciones		= (array) explode("\n", $this->opciones);
		// Actions
		//add_action( 'shop_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
    }
    /**
     * Initialise Gateway Settings Form Fields
     *
     * @access public
     * @return void
     */
    function init_form_fields() {
    	global $shop;
    	// Backwards compat
    	if ( $this->get_option( 'requires_coupon' ) && $this->min_amount )
			$default_requires = 'either';
		elseif ( $this->get_option( 'requires_coupon' ) )
			$default_requires = 'coupon';
		elseif ( $this->min_amount )
			$default_requires = 'min_amount';
		else
			$default_requires = '';
    	$this->form_fields = array(
			'activo' => array(
							'title' 		=> __( 'Enable/Disable', 'shop' ),
							'type' 			=> 'checkbox',
							'label' 		=> __( 'Enable Free Shipping', 'shop' ),
							'default' 		=> 'yes'
						),
			'title' => array(
							'title' 		=> __( 'Method Title', 'shop' ),
							'type' 			=> 'text',
							'description' 	=> __( 'This controls the title which the user sees during checkout.', 'shop' ),
							'default'		=> __( 'Free Shipping', 'shop' ),
							'desc_tip'      => true,
						),
			'disponibilidad' => array(
							'title' 		=> __( 'Method disponibilidad', 'shop' ),
							'type' 			=> 'select',
							'default' 		=> 'all',
							'class'			=> 'disponibilidad',
							'options'		=> array(
								'all' 		=> __( 'All allowed paises', 'shop' ),
								'specific' 	=> __( 'Specific Countries', 'shop' )
							)
						),
			'paises' => array(
							'title' 		=> __( 'Specific Countries', 'shop' ),
							'type' 			=> 'multiselect',
							'class'			=> 'chosen_select',
							'css'			=> 'width: 450px;',
							'default' 		=> '',
							'options'		=> $shop->paises->paises
						),
			'requires' => array(
							'title' 		=> __( 'Free Shipping Requires...', 'shop' ),
							'type' 			=> 'select',
							'default' 		=> $default_requires,
							'options'		=> array(
								'' 				=> __( 'N/A', 'shop' ),
								'coupon'		=> __( 'A valid free shipping coupon', 'shop' ),
								'min_amount' 	=> __( 'A minimum order amount (defined below)', 'shop' ),
								'either' 		=> __( 'A minimum order amount OR a coupon', 'shop' ),
								'both' 			=> __( 'A minimum order amount AND a coupon', 'shop' ),
							)
						),
			'min_amount' => array(
							'title' 		=> __( 'Minimum Order Amount', 'shop' ),
							'type' 			=> 'number',
							'custom_attributes' => array(
								'step'	=> 'any',
								'min'	=> '0'
							),
							'description' 	=> __( 'Users will need to spend this amount to get free shipping (if activo above).', 'shop' ),
							'default' 		=> '0',
							'desc_tip'      => true,
							'placeholder'	=> '0.00'
						)
			);
    }
	/**
	 * Admin Panel Options
	 * - Options for bits like 'title' and disponibilidad on a pais-by-pais basis
	 *
	 * @since 1.0.0
	 * @access public
	 * @return void
	 */
	public function admin_options() {
    	?>
    	<h3><?php _e( 'Free Shipping', 'shop' ); ?></h3>
    	<table class="form-table">
    	<?php
    		// Generate the HTML For the settings form.
    		$this->generate_settings_html();
    	?>
		</table><!--/.form-table-->
    	<?php
    }
    /**
     * is_available function.
     *
     * @access public
     * @param mixed $paquete
     * @return bool
     */
    function is_available( $paquete ) {
    	$shop_config = $GLOBALS['opciones']['ext_shop'];
		
    	if ( $this->activo == "no" ) return false;
		$ship_to_paises = '';
		
		global $paises_defecto_shop;
		
		if (is_array($paises_defecto_shop)){
			$paises_defecto = $paises_defecto_shop;
		} else {
			$paises_defecto_shop = Helpers::getPaisesWithArgs($config_shop['envios']['paises'], $config_shop['envios']['disponibilidad']);
			$paises_defecto = $paises_defecto_shop;
		}
		
		if ( $this->disponibilidad == 'specific' ) {
			if ($this->zona != "0"){
				e_include("ext_shop/class/zonaEnvios");
				$zona = new ZonaEnviosShop();
				$zona->setid($this->zona);
				if ($zona = $zona->BuscarId()){
					$ship_to_paises = Helpers::toArray(json_decode($zona->paises));
				}
			} else {
				$ship_to_paises = $this->paises;
			}
		} else {
			if ( sizeof($paises_defecto) > 0 )
				$ship_to_paises = $paises_defecto;
		}
		if ( is_array( $ship_to_paises ) )
			if ( ! in_array( $paquete['destino']['pais'].'.'.$paquete['destino']['estado'], $ship_to_paises ) )
				return false;
		// Enabled logic
		$is_available 		= false;
		$has_coupon 		= false;
		$has_met_min_amount = false;
		if ( in_array( $this->opciones, array( 'coupon', 'either', 'both' ) ) ) {
			/*if ( $shop->cart->applied_coupons ) {
				foreach ($shop->cart->applied_coupons as $code) {
					$coupon = new WC_Coupon( $code );
					if ( $coupon->is_valid() && $coupon->enable_free_shipping() )
						$has_coupon = true;
				}
			}*/
		}
		if ( in_array( $this->opciones, array( 'min_amount', 'either', 'both' ) ) ) {
			/*if ( isset( $shop->cart->cart_contents_total ) ) {
				if ( $shop->cart->prices_include_tax )
					$total = $shop->cart->tax_total + $shop->cart->cart_contents_total;
				else
					$total = $shop->cart->cart_contents_total;
				if ( $total >= $this->min_amount )
					$has_met_min_amount = true;
			}*/
		}
		switch ( $this->opciones ) {
			case 'min_amount' :
				if ( $has_met_min_amount ) $is_available = true;
			break;
			case 'coupon' :
				if ( $has_coupon ) $is_available = true;
			break;
			case 'both' :
				if ( $has_met_min_amount && $has_coupon ) $is_available = true;
			break;
			case 'either' :
				if ( $has_met_min_amount || $has_coupon ) $is_available = true;
			break;
			default :
				$is_available = true;
			break;
		}
		//return apply_filters( 'shop_shipping_' . $this->id . '_is_available', $is_available );
    }
    /**
     * calcular_envio function.
     *
     * @access public
     * @return array
     */
    function calcular_envio() {
    	$args = array(
    		'id' 	=> $this->id,
    		'nombre' => $this->titulo,
    		'coste' 	=> 0,
    		'impuestos' => false,
			'tipo' => 'envioGratis',
			'tasa_manipulacion'	=> 0
    	);
    	$this->add_tarifa( $args );
    }
}