<?php
/**
 * Flat Rate Shipping Method
 * 
 * A simple envio method for a flat cuota per item or per orden
 *
 * @class 		TarifaPlana
 * @package		Shop
 * @category	Shipping
 * @author		Mideas
 */  
class TarifaPlana extends MetodoEnvio {
	
	function __construct() { 
        $this->id 						= 'tarifaPlana';
        $this->titulo_metodo 			= __('Tarifa plana');
    	$this->init();
    } 
    
    function init() {
    	
		// Load the settings.
		$this->init_settings();
		
		// Define user set variables
		$this->id			= $this->settings['id'];
        $this->activo		= $this->settings['activo'];
		$this->titulo 		= $this->settings['titulo'];
		$this->disponibilidad = $this->settings['disponibilidad'];
		$this->paises 	= $this->settings['paises'];
		$this->zona 	= $this->settings['zona'];
		$this->tipo 		= $this->settings['tipo'];
		$this->estado_impuestos	= $this->settings['estado_impuestos'];
		$this->coste 		= $this->settings['coste'];
		$this->tasa_manipulacion 		= $this->settings['tasa_manipulacion'];
		$this->cuota 			= $this->settings['cuota']; 
		$this->cuota_minima 	= isset($this->settings['cuota_minima']) ? $this->settings['cuota_minima'] : ''; 
		$this->opciones 		= isset($this->settings['opciones']) ? $this->settings['opciones'] : ''; 
		
		// Get opciones
		$this->opciones		= (array) json_decode($this->opciones);
		
		// Load Tarifa planas
		//$this->get_tarifas_planas();
    }
    
    function calcular_envio( $paquete = array() ) {
    	
    	$this->tarifas = array();
    	
    	if ( $this->tipo == 'orden' ) {
    		
    		$envio_total = $this->orden_envio( $paquete );
			$tasa_total = $this->calc_tasa_manipulacion( $paquete );
			$envio_total += $tasa_total;
    		$tarifa = array(
				'id' 	=> $this->id,
				'nombre' => $this->titulo,
				'coste' 	=> $envio_total,
				'tasa_manipulacion'	=> $tasa_total
			);
			
		} elseif ( $this->tipo == 'class' ) {
		
			$envio_total = $this->class_envio( $paquete );
			$tasa_total = $this->calc_tasa_manipulacion( $paquete );
			$envio_total += $tasa_total;
    		
    		$tarifa = array(
				'id' 	=> $this->id,
				'nombre' => $this->titulo,
				'coste' 	=> $envio_total,
				'tasa_manipulacion'	=> $tasa_total
			);
		} elseif ( $this->tipo == 'item' ) {
			
			$costes = $this->item_envio( $paquete );
			$tasa_total = $this->calc_tasa_manipulacion_item( $paquete );
			
			$tarifa = array(
				'id' 		=> $this->id,
				'nombre' 	=> $this->titulo,
				'coste' 		=> $costes,
				'tasa_manipulacion'	=> $tasa_total,
				'calc_impuesto' 	=> 'per_item',
				'tipo' => 'tarifaPlana'
			);
		}
		
		// Register the tarifa
		$this->add_tarifa( $tarifa );
		
		// Add any extra tarifas
		if ( sizeof( $this->opciones ) > 0) foreach ( $this->opciones as $opcion ) {
			
			$this_opcion = explode('|', $opcion);
			
			if (sizeof($this_opcion)!==3) continue;
			
			$extra_tarifa = $tarifa;
			
			$extra_tarifa['id'] 		= $this->id . ':' . $this_opcion[0];
			$extra_tarifa['nombre'] 	= $this_opcion[0];
			$extra_tarifa['tipo'] = 'tarifaPlana';
			
			$per_orden_coste = ($this_opcion[2]=='1') ? 1 : 0;
			$this_coste = $this_opcion[1];
			
			if (is_array($extra_tarifa['coste'])) {
				
				if ($per_orden_coste) {
					$extra_tarifa['coste']['orden'] = $this_coste;
				} else {
					// Per-product envio
					$extra_tarifa['coste']['orden'] = $this_coste * $cantidad_total;
				}
				
			} else {
				// If using envio per class, multiple the coste by the classes we found
				if (!$per_orden_coste && $this->tipo=='class') {
					$this_coste = $this_coste * $found_envio_classes;
				}
				
				$extra_tarifa['coste'] = $extra_tarifa['coste'] + $this_coste;
			}
			
			$this->add_tarifa( $extra_tarifa );  
			
		}
		
		
    } 
	
	function calc_tasa_manipulacion( $paquete ){
		$coste_tasa_manipulacion = null;
		$tasa = null;
		
		if ( sizeof( $this->tarifas_planas ) > 0 ) {
			
    		$found_envio_classes = array();
    		
    		// Find envio classes for products in the cart
    		if ( sizeof( $paquete['contenido'] ) > 0 ) {
    			foreach ( $paquete['contenido'] as $item_id => $values ) {
    				if ( $values['data']->necesita_envio() )
    					$found_envio_classes[] = $values['data']->get_envio_class(); 
    			}
    		}
    		$found_envio_classes = array_unique( $found_envio_classes );
			
    		// Find most expensive class (if found)
    		foreach ( $found_envio_classes as $envio_class ) {
    			if (isset($this->tarifas_planas[$envio_class])) {
    				if ($this->tarifas_planas[$envio_class]['coste'] > $coste) {
    					$tasa 	= $this->tarifas_planas[$envio_class]['tasa_manipulacion'];
    				}
    			} else {
    				// No matching classes so use defaults
    				if ($this->coste > $coste) {
    					$tasa 	= $this->tasa_manipulacion;
    				}
    			}
    		}
		}
		
		// Default tarifas
		if ( is_null( $tasa ) ) {
			$tasa = $this->tasa_manipulacion;
		}
		
		$coste_tasa_manipulacion = (float) ( $paquete['contenido_coste'] / 100 ) * $tasa;
		
		$ctm_exp = explode(".", str_replace(",",".", (string) $coste_tasa_manipulacion));
		
		if (is_string($ctm_exp[1]))
			$ctm_exp[1] = substr($ctm_exp[1], 0, 2);
		else
			$ctm_exp[1] = '00';
			
		$coste_tasa_manipulacion = $ctm_exp[0] . '.' . $ctm_exp[1];
		$coste_tasa_manipulacion = (float) $coste_tasa_manipulacion;
		
		// Shipping for whole orden
		return $coste_tasa_manipulacion;
	}
    
    function orden_envio( $paquete ) {
    	$coste 	= null;
    	$cuota 	= null;
    		
		if ( sizeof( $this->tarifas_planas ) > 0 ) {
			
    		$found_envio_classes = array();
    		
    		// Find envio classes for products in the cart
    		if ( sizeof( $paquete['contenido'] ) > 0 ) {
    			foreach ( $paquete['contenido'] as $item_id => $values ) {
    				if ( $values['data']->necesita_envio() )
    					$found_envio_classes[] = $values['data']->get_envio_class(); 
    			}
    		}
    		$found_envio_classes = array_unique( $found_envio_classes );
    		
    		// Find most expensive class (if found)
    		foreach ( $found_envio_classes as $envio_class ) {
    			if (isset($this->tarifas_planas[$envio_class])) {
    				if ($this->tarifas_planas[$envio_class]['coste'] > $coste) {
    					$coste 	= $this->tarifas_planas[$envio_class]['coste'];
    					$cuota	= $this->tarifas_planas[$envio_class]['cuota'];
    				}
    			} else {
    				// No matching classes so use defaults
    				if ($this->coste > $coste) {
    					$coste 	= $this->coste;
    					$cuota	= $this->cuota;
    				}
    			}
    		}
		}
		
		// Default tarifas
		if ( is_null( $coste ) ) {
			$coste = $this->coste;
			$cuota = $this->cuota;
		}
		
		// Shipping for whole orden
		return $coste + $this->get_cuota( $cuota, $paquete['contenido_coste'] );
    }
    
    function class_envio( $paquete ) {
		$coste 	= null;
    	$cuota 	= null;
    		
		if ( sizeof( $this->tarifas_planas ) > 0 ) {
		
    		$found_envio_classes = array();
    		
    		// Find envio classes for products in the cart. Store prices too, so we can calc a cuota for the class.
    		if ( sizeof( $paquete['contenido'] ) > 0 ) {
    			foreach ( $paquete['contenido'] as $item_id => $values ) {
    				if ( $values['data']->necesita_envio() ) :
    					if ( isset( $found_envio_classes[$values['data']->get_envio_class()] ) ) {
    						$found_envio_classes[$values['data']->get_envio_class()] = ($values['data']->getprecio() * $values['quantity']) + $found_envio_classes[$values['data']->get_envio_class()];
    					} else {
    						$found_envio_classes[$values['data']->get_envio_class()] = ($values['data']->getprecio() * $values['quantity']);
    					}
    				endif;
    			}
    		}
    		$found_envio_classes = array_unique($found_envio_classes);
    		
    		// For each found class, add up the costes and cuotas
    		foreach ($found_envio_classes as $envio_class => $class_price) :
    			if (isset($this->tarifas_planas[$envio_class])) :
    				$coste 	+= $this->tarifas_planas[$envio_class]['coste'];
    				$cuota	+= $this->get_cuota( $this->tarifas_planas[$envio_class]['cuota'], $class_price );
    			else :
    				// Class not set so we use default tarifa
    				$coste 	+= $this->coste;
    				$cuota	+= $this->get_cuota( $this->cuota, $class_price );
    			endif;
    		endforeach;
		}
			
		// Total
		return $coste + $cuota;
    }
	
	function calc_tasa_manipulacion_item( $paquete ) {
		
		// Per item envio so we pass an array of costes (per item) instead of a single value
		$costes = array();
		$cantidad_total = 0;
		
		// Shipping per item
		foreach ( $paquete['contenido'] as $item_id => $values ) {
			
			$_product = $values['data'];
			
			if ( $values['cantidad'] > 0 && $_product->necesita_envio() ) {
				
				$envio_class = $_product->get_envio_class();
				
				if ( isset( $this->tarifas_planas[$envio_class] ) ) {
					$tasa 	= $this->tarifas_planas[$envio_class]['tasa_manipulacion'];
				} else {
					$tasa 	= $this->tasa_manipulacion;
				}
				
				$coste_tasa_manipulacion = (float) ( $values['linea_total'] / 100 ) * $tasa;
		
				$ctm_exp = explode(".", str_replace(",",".", (string) $coste_tasa_manipulacion));
				
				if (is_string($ctm_exp[1]))
					$ctm_exp[1] = substr($ctm_exp[1], 0, 2);
				else
					$ctm_exp[1] = '00';
					
				$coste_tasa_manipulacion = $ctm_exp[0] . '.' . $ctm_exp[1];
				$coste_tasa_manipulacion = (float) $coste_tasa_manipulacion;
				
				$costes[$item_id] = $coste_tasa_manipulacion;
				
				$cantidad_total += $values['cantidad'];
				
			}
		}
		
		return $costes;
    }
    
    function item_envio( $paquete ) {
		
		// Per item envio so we pass an array of costes (per item) instead of a single value
		$costes = array();
		$cantidad_total = 0;
		
		// Shipping per item
		foreach ( $paquete['contenido'] as $item_id => $values ) {
			
			$_product = $values['data'];
			
			if ( $values['cantidad'] > 0 && $_product->necesita_envio() ) {
				
				$envio_class = $_product->get_envio_class();
				
				if ( isset( $this->tarifas_planas[$envio_class] ) ) {
					$coste 	= $this->tarifas_planas[$envio_class]['coste'];
    				$cuota	= $this->get_cuota( $this->tarifas_planas[$envio_class]['cuota'], $_product->getprecio(true) );
					$tasa 	= $this->tarifas_planas[$envio_class]['tasa_manipulacion'];
				} else {
					$coste 	= $this->coste;
					$cuota	= $this->get_cuota( $this->cuota, $_product->getprecio(true) );
					$tasa 	= $this->tasa_manipulacion;
				}
				
				$costes[$item_id] = ( ( $coste + $cuota ) * $values['cantidad'] );
				
				if (!empty($tasa)){
					
					$coste_tasa_manipulacion = (float) $tasa * ($values['linea_total'] / 100);
		
					$ctm_exp = explode(".", str_replace(",",".", (string) $coste_tasa_manipulacion));
					
					if (is_string($ctm_exp[1]))
						$ctm_exp[1] = substr($ctm_exp[1], 0, 2);
					else
						$ctm_exp[1] = '00';
						
					$coste_tasa_manipulacion = $ctm_exp[0] . '.' . $ctm_exp[1];
					$coste_tasa_manipulacion = (float) $coste_tasa_manipulacion;
					$costes[$item_id] += $coste_tasa_manipulacion;
				}
				
				$cantidad_total += $values['cantidad'];
				
			}
		}
		
		return $costes;
    }
    
    function process_tarifas_planas() {
   		
		// Save the tarifas
		$tarifa_plana_class = array();
		$tarifa_plana_coste = array();
		$tarifa_plana_cuota = array();
		$tarifas_planas = array();
		
		if (isset($_POST[$this->id . '_class']))	$tarifa_plana_class	= array_map('woocommerce_clean', $_POST[$this->id . '_class']);
		if (isset($_POST[$this->id . '_coste'])) 	$tarifa_plana_coste 	= array_map('woocommerce_clean', $_POST[$this->id . '_coste']);
		if (isset($_POST[$this->id . '_cuota'])) 	$tarifa_plana_cuota 		= array_map('woocommerce_clean', $_POST[$this->id . '_cuota']);
		
		// Get max key
		$values = $tarifa_plana_class;
		ksort($values);   
		$value = end($values);  
		$key = key($values);
		
		for ($i=0; $i<=$key; $i++) :
		
			if (isset($tarifa_plana_class[$i]) && isset($tarifa_plana_coste[$i]) && isset($tarifa_plana_cuota[$i])) :
				
				$tarifa_plana_coste1 = (float) $tarifa_plana_coste[$i] ;
		
				$ctm_exp = explode(".", str_replace(",",".", (string) $tarifa_plana_coste1));
				
				if (is_string($ctm_exp[1]))
					$ctm_exp[1] = substr($ctm_exp[1], 0, 2);
				else
					$ctm_exp[1] = '00';
					
				$tarifa_plana_coste1 = $ctm_exp[0] . '.' . $ctm_exp[1];
				$tarifa_plana_coste1 = (float) $tarifa_plana_coste1;
				
				
				$tarifa_plana_coste[$i] = $tarifa_plana_coste1;
				
				// Add to flat tarifas array
				$tarifas_planas[ $tarifa_plana_class[$i] ] = array(
					'coste' 				=> $tarifa_plana_coste[$i],
					'cuota' 				=> $tarifa_plana_cuota[$i]
				);  
				
			endif;
		endfor;
		
		$this->tarifa_plana_opcion = $tarifas_planas;
		
		$this->get_tarifas_planas();
    }
    
    function get_tarifas_planas() {
		$tp = new EnvioShop();
		$tp = $tp->Listar("metodo_envio='tarifaPlana'");
    	$this->tarifas_planas = $tp;
    }
}
function add_tarifa_plana_method( $methods ) {
	$methods[] = 'TarifaPlana'; return $methods;
}
