<?php
/**
 * Flat Rate Shipping Method
 * 
 * A simple envio method for a flat cuota per item or per orden
 *
 * @class 		TarifaPorPeso
 * @package		Shop
 * @category	Shipping
 * @author		Mideas
 */  
class TarifaPorPeso extends MetodoEnvio {
	
	function __construct() { 
        $this->id 						= 'tarifaPorPeso';
        $this->titulo_metodo 			= __('Tarifa por peso');
    	$this->init();
    } 
    
    function init() {
    	
		// Load the settings.
		$this->init_settings();
		$this->init_form_fields();
		
		// Define user set variables
		$this->id			= $this->settings['id'];
        $this->activo		= $this->settings['activo'];
		$this->titulo 		= $this->settings['titulo'];
		$this->disponibilidad = $this->settings['disponibilidad'];
		$this->paises 	= $this->settings['paises'];
		$this->zona 	= $this->settings['zona'];
		$this->tipo 		= $this->settings['tipo'];
		$this->estado_impuestos	= $this->settings['estado_impuestos'];
		$this->coste 		= $this->settings['coste'];
		$this->tasa_manipulacion 		= $this->settings['tasa_manipulacion'];
		$this->cuota 			= $this->settings['cuota']; 
		$this->cuota_minima 	= isset($this->settings['cuota_minima']) ? $this->settings['cuota_minima'] : ''; 
		$this->opciones 		= isset($this->settings['opciones']) ? $this->settings['opciones'] : ''; 
		
		// Get opciones
		$this->opciones		= (array) json_decode($this->opciones);
		
		if (is_array($this->opciones) && sizeof($this->opciones) > 0){
			foreach ($this->opciones as $kopt=>$opt){				
				$this->settings[$kopt] = $opt;
			}
		}
		$this->peso_minimo 			= isset($this->settings['peso_minimo']) ? $this->settings['peso_minimo'] : ''; 
		$this->peso_maximo 	= isset($this->settings['peso_maximo']) ? $this->settings['peso_maximo'] : ''; 
		$this->rango_peso 	= isset($this->settings['rango_peso']) ? $this->settings['rango_peso'] : ''; 
		// Load Tarifa planas
		//$this->get_tarifas_por_peso();
    }
	
	function init_form_fields()
	{
		$this->form_fields = array
		(
			'peso_minimo' => array
			(
				'titulo' => __( 'peso_minimo' ),
				'label' => __( 'Peso mínimo' ),
				'type' => 'text',
				'descripcion' => __( 'Peso mínimo' ),
				'default' => __( '0' ),
				'desc_tip'      => true,
				'attributes'	=>	array('multilanguage' => 'false')
			),
			'peso_maximo' => array
			(
				'titulo' => __( 'peso_maximo' ),
				'label' => __( 'Peso máximo' ),
				'type' => 'text',
				'descripcion' => __( 'Peso máximo' ),
				'default' => __( '*' ),
				'desc_tip'      => true,
				'attributes'	=>	array('multilanguage' => 'false')
			),
			'rango_peso' => array
			(
				'titulo' => __( 'rango_peso' ),
				'label' => __( 'Rango de peso' ),
				'type' => 'text',
				'descripcion' => __( 'Rango de peso' ),
				'default' => __( '*' ),
				'desc_tip'      => true,
				'attributes'	=>	array('multilanguage' => 'false')
			)
		);
	}
    
    function calcular_envio( $paquete = array() ) {
		e_include('ext_shop/class/carrito');
		
		global $shop;
		global $carrito_shop;
		
		if (is_object($shop)){
			$carrito = $shop->carrito;
		} elseif (is_object($carrito_shop)){
			$carrito = $carrito_shop;
		} else {
			$carrito = new Carrito();
			$carrito_shop = $carrito;
		}
		//$carrito->calcular_totales();
		
		$peso_total = $carrito->contenido_carrito_peso;
		
		if ($this->peso_minimo && $peso_total <= $this->peso_minimo) {
			return false;
		}
		if ($this->peso_maximo && $peso_total >= $this->peso_maximo) {
			return false;
		}
		
		$peso_calculado = max(0, $peso_total - $this->peso_minimo);
		
		if ($this->rango_peso != 0){
			$peso_calculado = ceil(round($peso_calculado / $this->rango_peso, 5));
		}		
		
		$coste   = (float)@$this->settings['coste'];
		$precio  = $peso_calculado * $coste;
		
		if ($this->cuota > 0 && $paquete['destino']['pais']) {
			$precio = $precio + $this->cuota;
		}
		
		if ($precio <= 0) {
			return false;
		}
		
		$coste_tasa_manipulacion = (float) ( $paquete['contenido_coste'] / 100 ) * $this->tasa_manipulacion;
		
		$ctm_exp = explode(".", str_replace(",",".", (string) $coste_tasa_manipulacion));
		
		if (is_string($ctm_exp[1]))
			$ctm_exp[1] = substr($ctm_exp[1], 0, 2);
		else
			$ctm_exp[1] = '00';
			
		$coste_tasa_manipulacion = $ctm_exp[0] . '.' . $ctm_exp[1];
		$coste_tasa_manipulacion = (float) $coste_tasa_manipulacion;
		
		$tasa_total = $coste_tasa_manipulacion ;
		
		$precio += $tasa_total;
		$this->add_tarifa(array
		(
			'id'       => $this->id,
			'nombre'    => $this->titulo,
			'coste'     => $precio,
			'impuestos'    => '',
			'calc_impuesto' => 'per_order',
			'tipo' => 'tarifaPorPeso',
			'tasa_manipulacion'	=> $tasa_total
		));
    } 
    
    function process_tarifas_por_peso() {
   		
		// Save the tarifas
		$tarifa_por_peso_class = array();
		$tarifa_por_peso_coste = array();
		$tarifa_por_peso_cuota = array();
		$tarifas_por_peso = array();
		
		if (isset($_POST[$this->id . '_class']))	$tarifa_por_peso_class	= array_map('woocommerce_clean', $_POST[$this->id . '_class']);
		if (isset($_POST[$this->id . '_coste'])) 	$tarifa_por_peso_coste 	= array_map('woocommerce_clean', $_POST[$this->id . '_coste']);
		if (isset($_POST[$this->id . '_cuota'])) 	$tarifa_por_peso_cuota 		= array_map('woocommerce_clean', $_POST[$this->id . '_cuota']);
		
		// Get max key
		$values = $tarifa_por_peso_class;
		ksort($values);   
		$value = end($values);  
		$key = key($values);
		
		for ($i=0; $i<=$key; $i++) :
		
			if (isset($tarifa_por_peso_class[$i]) && isset($tarifa_por_peso_coste[$i]) && isset($tarifa_por_peso_cuota[$i])) :
			
				$tarifa_por_peso_coste1 = (float) $tarifa_por_peso_coste[$i];
		
				$ctm_exp = explode(".", str_replace(",",".", (string) $tarifa_por_peso_coste1));
				
				if (is_string($ctm_exp[1]))
					$ctm_exp[1] = substr($ctm_exp[1], 0, 2);
				else
					$ctm_exp[1] = '00';
					
				$tarifa_por_peso_coste1 = $ctm_exp[0] . '.' . $ctm_exp[1];
				$tarifa_por_peso_coste1 = (float) $tarifa_por_peso_coste1;
				
				$tarifa_por_peso_coste[$i] = $tarifa_por_peso_coste1;
				
				// Add to flat tarifas array
				$tarifas_por_peso[ $tarifa_por_peso_class[$i] ] = array(
					'coste' 				=> $tarifa_por_peso_coste[$i],
					'cuota' 				=> $tarifa_por_peso_cuota[$i]
				);  
				
			endif;
		endfor;
		
		$this->tarifa_por_peso_opcion = $tarifas_por_peso;
		
		$this->get_tarifas_por_peso();
    }
    
    function get_tarifas_por_peso() {
		$tp = new EnvioShop();
		$tp = $tp->Listar("metodo_envio='tarifaPorPeso'");
    	$this->tarifas_por_peso = $tp;
    }
}
function add_tarifa_por_peso_method( $methods ) {
	$methods[] = 'TarifaPorPeso'; return $methods;
}
