<?php 
defined('CONF_URL_APP') or die;
abstract class ShopHelper {
	/**
	 * Get Base Currency Code.
	 * @return string
	 */
	public static function get_shop_moneda() {
		return $GLOBALS['opciones']['ext_shop']['shop']['moneda'];
	}
	
	/**
	 * Get full list of moneda codes.
	 * @return array
	 */
	public static function get_shop_monedas() {
		return array_unique(
			array(
				'AED' => __( 'Dirham de Emiratos Árabes Unidos' ),
				'AUD' => __( 'Dólar Australiano' ),
				'BDT' => __( 'Taka Bangladeshí' ),
				'BRL' => __( 'Real Brasileño' ),
				'BGN' => __( 'Lev Búlgaro' ),
				'CAD' => __( 'Dólar Canadiense' ),
				'CLP' => __( 'Peso Chileno' ),
				'CNY' => __( 'Yuan Chino' ),
				'COP' => __( 'Peso Colombiano' ),
				'CZK' => __( 'Corona Checa' ),
				'DKK' => __( 'Corona Danesa' ),
				'DOP' => __( 'Peso Dominicano' ),
				'EUR' => __( 'Euro' ),
				'HKD' => __( 'Dólar de Hong Kong' ),
				'HRK' => __( 'Kuna Croata' ),
				'HUF' => __( 'Florín Húngaro' ),
				'ISK' => __( 'Corona Islandesa' ),
				'IDR' => __( 'Rupia Indonesa' ),
				'INR' => __( 'Rupia India' ),
				'NPR' => __( 'Rupia Nepalí' ),
				'ILS' => __( 'Shekel Israelí' ),
				'JPY' => __( 'Yen Japonés' ),
				'KIP' => __( 'Lao Kip' ),
				'KRW' => __( 'Won Surcoreano' ),
				'MYR' => __( 'Ringgit Malasio' ),
				'MXN' => __( 'Peso Mexicano' ),
				'NGN' => __( 'Naira Nigeriano' ),
				'NOK' => __( 'Corona Noruega' ),
				'NZD' => __( 'Dólar Neozelandés' ),
				'PYG' => __( 'Guaraní Paraguayo' ),
				'PHP' => __( 'Peso Filipino' ),
				'PLN' => __( 'Zloty Polaco' ),
				'GBP' => __( 'Libra Esterlina' ),
				'RON' => __( 'Leu Rumano' ),
				'RUB' => __( 'Rublo Ruso' ),
				'SGD' => __( 'Dólar de Singapur' ),
				'ZAR' => __( 'Rand Sudafricano' ),
				'SEK' => __( 'Corona Sueca' ),
				'CHF' => __( 'Franco Suizo' ),
				'TWD' => __( 'Dólar Taiwanés' ),
				'THB' => __( 'Baht Tailandés' ),
				'TRY' => __( 'Lira Turca' ),
				'UAH' => __( 'Hryvnia Ucraniano' ),
				'USD' => __( 'Dólar Americano' ),
				'VND' => __( 'Dong Vietnamita' ),
				'EGP' => __( 'Libra Egipcia' ),
			)
		);
	}
	
	/**
	 * Get Currency simbolo.
	 * @param string $moneda (default: '')
	 * @return string
	 */
	public static function get_shop_moneda_simbolo( $moneda = '' ) {
		if ( ! $moneda ) {
			$moneda = self::get_shop_moneda();
		}
		
		switch ( $moneda ) {
			case 'AED' :
				$moneda_simbolo = 'د.إ';
				break;
			case 'AUD' :
			case 'CAD' :
			case 'CLP' :
			case 'COP' :
			case 'HKD' :
			case 'MXN' :
			case 'NZD' :
			case 'SGD' :
			case 'USD' :
				$moneda_simbolo = '&#36;';
				break;
			case 'BDT':
				$moneda_simbolo = '&#2547;&nbsp;';
				break;
			case 'BGN' :
				$moneda_simbolo = '&#1083;&#1074;.';
				break;
			case 'BRL' :
				$moneda_simbolo = '&#82;&#36;';
				break;
			case 'CHF' :
				$moneda_simbolo = '&#67;&#72;&#70;';
				break;
			case 'CNY' :
			case 'JPY' :
			case 'RMB' :
				$moneda_simbolo = '&yen;';
				break;
			case 'CZK' :
				$moneda_simbolo = '&#75;&#269;';
				break;
			case 'DKK' :
				$moneda_simbolo = 'kr.';
				break;
			case 'DOP' :
				$moneda_simbolo = 'RD&#36;';
				break;
			case 'EGP' :
				$moneda_simbolo = 'EGP';
				break;
			case 'EUR' :
				$moneda_simbolo = '&euro;';
				break;
			case 'GBP' :
				$moneda_simbolo = '&pound;';
				break;
			case 'HRK' :
				$moneda_simbolo = 'Kn';
				break;
			case 'HUF' :
				$moneda_simbolo = '&#70;&#116;';
				break;
			case 'IDR' :
				$moneda_simbolo = 'Rp';
				break;
			case 'ILS' :
				$moneda_simbolo = '&#8362;';
				break;
			case 'INR' :
				$moneda_simbolo = 'Rs.';
				break;
			case 'ISK' :
				$moneda_simbolo = 'Kr.';
				break;
			case 'KIP' :
				$moneda_simbolo = '&#8365;';
				break;
			case 'KRW' :
				$moneda_simbolo = '&#8361;';
				break;
			case 'MYR' :
				$moneda_simbolo = '&#82;&#77;';
				break;
			case 'NGN' :
				$moneda_simbolo = '&#8358;';
				break;
			case 'NOK' :
				$moneda_simbolo = '&#107;&#114;';
				break;
			case 'NPR' :
				$moneda_simbolo = 'Rs.';
				break;
			case 'PHP' :
				$moneda_simbolo = '&#8369;';
				break;
			case 'PLN' :
				$moneda_simbolo = '&#122;&#322;';
				break;
			case 'PYG' :
				$moneda_simbolo = '&#8370;';
				break;
			case 'RON' :
				$moneda_simbolo = 'lei';
				break;
			case 'RUB' :
				$moneda_simbolo = '&#1088;&#1091;&#1073;.';
				break;
			case 'SEK' :
				$moneda_simbolo = '&#107;&#114;';
				break;
			case 'THB' :
				$moneda_simbolo = '&#3647;';
				break;
			case 'TRY' :
				$moneda_simbolo = '&#8378;';
				break;
			case 'TWD' :
				$moneda_simbolo = '&#78;&#84;&#36;';
				break;
			case 'UAH' :
				$moneda_simbolo = '&#8372;';
				break;
			case 'VND' :
				$moneda_simbolo = '&#8363;';
				break;
			case 'ZAR' :
				$moneda_simbolo = '&#82;';
				break;
			default :
				$moneda_simbolo = '';
				break;
		}
	
		return $moneda_simbolo;
	}
	
	public static function renderDynamicAdminWidgets(){
		c_include('app','widget','helpers/widgetHelper');
		e_include('ext_shop/class/pedidoShop', 'ext_shop/class/producto', 'ext_shop/models/producto');
		$opts = $GLOBALS['opciones']['ext_shop'];
		
		if ($_GET['extension'] == 'dashboard'){
			$cm_txt = '';
			if (Usuario::isLoged() && !Usuario::getLoged()->es_superadmin()){
				$cm_txt = '&custom_menu=-1';
			}
			
			$pedidos_mes = new PedidoShop();
			$pedidos_mes = $pedidos_mes->Listar('fecha > "'.strftime('%Y-%m-%d', time()-2592000).'"');
			
			$total_mes = 0.00;
			
			foreach ($pedidos_mes as $p){
				if ($p->estado != $opts['venta']['estado_fallido'] && $p->estado != $opts['venta']['estado_cancelado']){
					$total_mes += $p->datos['total']['total'];
				}
			}
			
			$pedidos = new PedidoShop();
			$pedidos_procesando = $pedidos->ListarCount('estado = "'.$opts['venta']['estado_procesando'].'"');
			$pedidos_creado = $pedidos->ListarCount('estado = "'.$opts['venta']['estado_creado'].'"');
			
			$estadisticas_pedidos = $pedidos->get_estadisticas('dia');
			$dias = Helpers::createDateRangeArray(strftime("%Y-%m-%d", time()-2592000), strftime("%Y-%m-%d", time()));
			$graph = array();
			$n_ped = 0;
			$total_pedidos = 0;
			$importe_pedidos = 0;
			foreach ($dias as $dia){
				$graph[$n_ped][0] = strtotime($dia) * 1000;
				$graph[$n_ped][1] = 0;
				foreach ($estadisticas_pedidos as $est){
					if ($est->fecha == $dia){
						$graph[$n_ped][1] = (int) $est->num;
						$total_pedidos += (int) $est->num;
						$importe_pedidos += $est->total;
					}
				}
				$n_ped ++;
			}
			
			$total_mes_format = Helpers::Importe($total_mes).ShopHelper::get_shop_moneda_simbolo();
			
			$productos = new Producto();
			$productos = $productos->Listar();
			
			$productos_casi_sin_stock = 0;
			$productos_sin_stock = 0;
			
			foreach ($productos as $pr){
				$pm = new ProductoModel($pr->id, false);
				
				if (sizeof($pm->variaciones_atributos) > 0 && is_array($pm->variaciones_atributos)){
					$val_tmp = 0;
					foreach ($pm->variaciones_atributos as $va){
						$val_tmp += $va->stock;
					}
					if ($val_tmp == 0){
						$productos_sin_stock ++;
					} elseif ($val_tmp < 5){
						$productos_casi_sin_stock ++;
					}
				} else {
					if ($pm->objeto->stock == 0){
						$productos_sin_stock ++;
					} elseif ($pm->objeto->stock < 5){
						$productos_casi_sin_stock ++;
					}
				}
			}
			
			$widget_resumen = new Widget();
			$widget_resumen->setnombre("Estado de la tienda");
			$widget_resumen->settitulo(json_encode(array(SistemaIdioma::getIdiomaSesion()=>__('Estado de la tienda'))));
			$widget_resumen->settipo('wdg_admin_custom');
			$widget_resumen->setposicion('dashboard_01');
			$widget_resumen->setactivo(1);
			$widget_resumen->setnivel_acceso(0);
			
			$params_wr = array("contenido"=>array(SistemaIdioma::getIdiomaSesion()=>'<ul class="list-group">
  <li class="list-group-item"> <a href="'.CONF_URL_APP_ADMIN.'?extension=ext_shop&vista=lista_pedidos'.$cm_txt.'"> <span class="pull-left" style="font-size: 35px; margin-right: 10px;"><i class="fa fa-bar-chart"></i> </span> <span class="pull-left"><strong style="font-size: 35px; display: block; margin-bottom: -5px;"><span class="amount">'.$total_mes_format.'</span></strong> '.__('ventas este mes').'</span> <div class="flot-chart pull-right" id="stats-chart" style="background-color: #eeffff;border: 1px solid #eeeeee;"> </div></a><div class="clearfix"></div> </li>
  <li class="list-group-item col-xs-6"> <a href="'.CONF_URL_APP_ADMIN.'?extension=ext_shop&vista=lista_pedidos&filtro_estado=1'.$cm_txt.'"> <span class="pull-left" style="font-size: 18px; margin-right: 10px;"><i class="fa fa-circle"></i> </span> <span class="pull-left"><strong style="font-size: 18px; display: block; margin-bottom: -5px;">'.sprintf(__('%s pedidos'), number_format($pedidos_procesando, 0, '.', ',')).'</strong> '.__('esperan ser procesados').'</span> </a><div class="clearfix"></div> </li>
  <li class="list-group-item col-xs-6"> <a href="'.CONF_URL_APP_ADMIN.'?extension=ext_shop&vista=lista_pedidos&filtro_estado=3'.$cm_txt.'"> <span class="pull-left" style="font-size: 18px; margin-right: 10px;"><i class="fa fa-circle-o"></i> </span> <span class="pull-left"><strong style="font-size: 18px; display: block; margin-bottom: -5px;">'.sprintf(__('%s pedidos'), number_format($pedidos_creado, 0, '.', ',')).'</strong> '.__('en espera').'</span> </a> </li>
  <li class="list-group-item col-xs-6"> <a href="'.CONF_URL_APP_ADMIN.'?extension=ext_shop&vista=lista&stock=low'.$cm_txt.'"> <span class="pull-left" style="font-size: 18px; margin-right: 10px;"><i class="fa fa-square"></i> </span> <span class="pull-left"><strong style="font-size: 18px; display: block; margin-bottom: -5px;">'.sprintf(__('%s productos'), $productos_casi_sin_stock).'</strong> '.__('casi sin existencias').'</span> </a> </li>
  <li class="list-group-item col-xs-6"> <a href="'.CONF_URL_APP_ADMIN.'?extension=ext_shop&vista=lista&stock=out'.$cm_txt.'"> <span class="pull-left" style="font-size: 18px; margin-right: 10px;"><i class="fa fa-square-o"></i> </span> <span class="pull-left"><strong style="font-size: 18px; display: block; margin-bottom: -5px;">'.sprintf(__('%s productos'), $productos_sin_stock).'</strong> '.__('agotados').'</span> </a> </li>
</ul>
<div class="clearfix"></div>'), "class"=>"no-margin no-padding");

$GLOBALS['AssetBundles']->require_bundle('flot');

$GLOBALS['AppDoc']->AddScriptDeclaration("
$(document).ready(function() {
	var visits = ".json_encode($graph).";
	$('#stats-chart').css({
		height: '63px',
		width: '150px'
	});
	$.plot($('#stats-chart'),[
			{ data: visits }
		],{
	        lines: { show: true },
	        points: { show: false },
			xaxis: { ticks: 0, mode: 'time', timeformat: '".SistemaIdioma::formatoFecha('corta',true)."' },
			yaxis: { ticks: 0 },
			colors: ['#1ab394'],
			grid: {
				color: '#999999',
				hoverable: true,
				clickable: true,
				tickColor: '#D4D4D4',
				borderWidth:0
			}
	});
	
	$('<div id=\'tooltip\'></div>').css({
			position: 'absolute',
			display: 'none',
			border: '1px solid #fdd',
			padding: '2px',
			'background-color': '#fee',
			opacity: 0.80
		}).appendTo('body');
		$('#stats-chart').bind('plothover', function (event, pos, item) {
			if ($('#enablePosition:checked').length > 0) {
				var str = '(' + pos.x.toFixed(2) + ', ' + pos.y.toFixed(2) + ')';
				$('#hoverdata').text(str);
			}
			if (item) {
				var x = item.datapoint[0],
					y = parseInt(item.datapoint[1]);
					
				var fecha_obj = new Date(x);
				var fecha_format = fecha_obj.getDate() + '-' + (fecha_obj.getMonth()+1) + '-' + fecha_obj.getFullYear();
				$('#tooltip').html('".__('Pedidos del')." ' + fecha_format + ' = ' + y)
					.css({top: item.pageY+5, left: item.pageX+5})
					.fadeIn(200);
			} else {
				$('#tooltip').hide();
			}
		});
		$('#stats-chart').bind('plotclick', function (event, pos, item) {
			if (item) {
				$('#clickdata').text(' - click point ' + item.dataIndex + ' in ' + item.series.label);
				plot.highlight(item.series, item.datapoint);
			}
		});
	
});
");
			
			$widget_resumen->setparams(json_encode($params_wr));
			
			$widget_pedidos = new Widget();
			$widget_pedidos->setnombre("Últimos pedidos");
			$widget_pedidos->settitulo(json_encode(array(SistemaIdioma::getIdiomaSesion()=>__('Últimos pedidos'))));
			$widget_pedidos->settipo('wdg_admin_custom');
			$widget_pedidos->setposicion('dashboard_01');
			$widget_pedidos->setactivo(1);
			$widget_pedidos->setnivel_acceso(0);
			
			$ultimos_pedidos_c = $pedidos->ListarCount("","id","DESC");

			if ($ultimos_pedidos_c > 0){
				$ultimos_pedidos = $pedidos->Listar("","id","DESC",0,10);
				
				$wp_html = '<table class="table table-hover no-margins">
							<thead><tr><th class="order_id">'.__('ID').'</th><th class="order_date">'.__('Fecha').'</th><th class="order_customer">'.__('Cliente').'</th><th class="order_status">'.__('Estado').'</th><th class="order_total">'.__('Total').'</th></tr><tr></tr></thead>
							<tbody>';
						if (is_array($ultimos_pedidos) && sizeof($ultimos_pedidos) > 0){
							foreach ($ultimos_pedidos as $ped){
								$wp_html .= '<tr><td class="order_td_first order_id">'.$ped->id.'</td><td class="order_date"><a href="'.CONF_URL_APP_ADMIN.'?extension=ext_shop&vista=editar_pedido&id='.$ped->id.$cm_txt.'">'.strftime("%d-%m-%Y %H:%M",strtotime($ped->fecha)).'</a></td><td class="order_customer">'.$ped->datos['facturacion']['nombre'] . ' ' . $ped->datos['facturacion']['apellidos'].'</td><td class="order_status">'.$ped->get_estado_con_pago().'</td><td class="order_total">'.number_format($ped->datos['total']['total'], 2, ',', '.').' '.ShopHelper::get_shop_moneda_simbolo().'</td></tr>';
							}
						} else {
							$wp_html .= '<tr><td colspan="6" align="center">'.__('No hay pedidos').'</td></tr>';
						}
				$wp_html .= '</tbody></table><hr><div class="panel-body"><a href="'.CONF_URL_APP_ADMIN.'?extension=ext_shop&vista=lista_pedidos'.$cm_txt.'" class="btn btn-default btn-block">'.__('Ver todos los pedidos').'</a></div>';
			}
			
			$params_wp = array("contenido"=>array(SistemaIdioma::getIdiomaSesion()=>$wp_html), "class"=>"no-margin no-padding");
			$widget_pedidos->setparams(json_encode($params_wp));
			
			if ($opts['admin']['widget_estado'] != '1')
				WidgetHelper::$_custom_widgets['dashboard_01'][] = $widget_resumen;
				
			if ($opts['admin']['widget_pedidos'] != '1')	
				WidgetHelper::$_custom_widgets['dashboard_02'][] = $widget_pedidos;
		}
	}
}