<?php
/**
 * Clase MetodoEnvio
 * 
 * Extendida por los métodos de envío para manejar los cálculos de envío, etc
 *
 * @class 		MetodoEnvio
 * @package		Shop
 * @category	Envio
 * @author		Mideas
 */  
class MetodoEnvio {
	
	var $id;
	var $titulo_metodo; 	// Method titulo
	var $titulo;			// User set titulo
	var $estado_impuestos			= 'taxable';
	var $disponibilidad;
	var $paises;
	var $zona;
	var $tipo;
	var $cuota				= 0;
	var $cuota_minima		= null;
	var $cantidad_minima			= null;
	var $tasa_manipulacion			= null;
	var $activo			= false;
	var $form_fields = array();
	var $db_object		= null;
	
	/**
	 * Rates
	 *
	 * This is an array of tarifas - methods must populate this array to register envio costes
	 */
	var $tarifas 				= array(); // This is an array of tarifas - methods must populate this array to register envio costes
	
	var $settings				= array();
	
	function init_settings(){
		$variables = get_object_vars($this->db_object);
		
		$this->settings = $variables;
	}
	
	/**
	 * Add a rate
	 *
	 * A a envio rate. If impuestos are not set they will be calculated based on coste.
	 */
	function add_tarifa( $args = array() ) {
		e_include('ext_shop/class/producto');
		$config_shop = App::getConfigValue('ext_shop');
		
		$defaults = array(
			'id' 		=> '',			// ID for the rate
			'nombre' 	=> '',			// Label for the rate
			'coste' 		=> '0',			// Amount or array of costes (per item envio)
			'tasa_manipulacion' 		=> '0',			// Amount or array of costes (per item envio)
			'impuestos' 	=> '',			// Pass impuestos, nothing to have it calculated for you, or 'false' to calc no tax
			'calc_impuesto'	=> 'per_order',	// Calc tax per_order or per_item. Per item needs an array of costes
			'tipo' => ''
		);
		
		if (empty($args))
			$args = $defaults;
		extract( $args );
		
		// Id and nombre are required
		if (!$id || !$nombre) return;
		
		// Handle coste
		$total_coste = (is_array($coste)) ? array_sum($coste) : $coste;
		
		// Taxes - if not an array and not set to false, calc tax based on coste and passed calc_impuesto variable
		// This saves envio methods having to do compelex tax calculations
		if (!is_array($impuestos) && $impuestos!==false && $total_coste>0 && $config_shop['impuestos']['activar_impuestos']=='1' && $this->estado_impuestos=='taxable' ) :
		
			e_include('ext_shop/class/impuestoTarifa');
			
			$_impuesto 	= new ImpuestoTarifa();
			$impuestos 	= array();
			
			switch ($calc_impuesto) :
				
				case "per_item" :
				
					// If we have an array of costes we can look up each items tax class and add tax accordingly
					if (is_array($coste)) :
						
						$cart = Helpers::getSesion()->get('shop.cesta');
						
						foreach ($coste as $coste_key => $cantidad) :
							
							if (!isset($cart[$coste_key])) continue;
							
							$_product = new Producto();
							$_product->setid($cart[$coste_key]['product_id']);
							$_product = $_product->BuscarId();
							
							$tarifas = $_impuesto->get_envio_impuesto_tarifas( $_product->impuesto );
							$item_impuestos = $_impuesto->calc_envio_impuesto( $cantidad, $tarifas );
							
							// Sum the item impuestos
							foreach (array_keys($impuestos + $item_impuestos) as $key) :
								$impuestos[$key] = (isset($item_impuestos[$key]) ? $item_impuestos[$key] : 0) + (isset($impuestos[$key]) ? $impuestos[$key] : 0);
							endforeach;
							
						endforeach;
						
						// Add any coste for the order - order costes are in the key 'order' 
						if (isset($coste['order'])) {
							
							$tarifas = $_impuesto->get_envio_impuesto_tarifas();
							$item_impuestos = $_impuesto->calc_envio_impuesto( $coste['order'], $tarifas );
					
							// Sum the item impuestos
							foreach (array_keys($impuestos + $item_impuestos) as $key) :
								$impuestos[$key] = (isset($item_impuestos[$key]) ? $item_impuestos[$key] : 0) + (isset($impuestos[$key]) ? $impuestos[$key] : 0);
							endforeach;
						}
					endif;
				break;
				
				default :
					
					$tarifas = $_impuesto->get_envio_impuesto_tarifas();
					$impuestos = $_impuesto->calc_envio_impuesto( $total_coste, $tarifas );
					
				break;
				
			endswitch;
			
		endif;
		$this->tarifas[] = new TarifaEnvioShop( $id, $nombre, $total_coste, $impuestos, $tipo, $tasa_manipulacion );
	}
	
    function is_available( $package ) {
    	$config_shop = App::getConfigValue('ext_shop');
    	
    	if ($this->activo=="0") 
    		return false;
    	
		$ship_to_paises = '';
		
		global $paises_defecto_shop;
			
		if (is_array($paises_defecto_shop)){
			$paises_globales = $paises_defecto_shop;
		} else {
			$paises_defecto_shop = Helpers::getPaisesWithArgs($config_shop['envios']['paises'], $config_shop['envios']['disponibilidad']);
			$paises_globales = $paises_defecto_shop;
		}
				
		$this_paises = Helpers::getPaisesWithArgs((array) json_decode($this->paises), $this->disponibilidad, $paises_globales);
		
		//if ( $this->disponibilidad == 'specific' ) :
		if ($this->zona != "0" && !empty($this->zona)) {
			e_include("ext_shop/class/zonaEnvios");
			$zona = new ZonaEnviosShop();
			$zona->setid($this->zona);
			if ($zona = $zona->BuscarId()){
				//$ship_to_paises = (array) json_decode($zona->paises);
				$ship_to_paises = Helpers::getPaisesWithArgs((array) json_decode($zona->paises), $this->disponibilidad, $paises_globales);
			}
		} elseif ( !empty($this_paises) ) {
			$ship_to_paises = $this_paises;
		} else {
			if ( !empty($paises_globales) ) {
				$ship_to_paises = $paises_globales;
			}
		}
		
		if ( is_array( $ship_to_paises ) ) :
			$prov_pack = $package['destino']['estado'];
			if (empty($prov_pack))
				$prov_pack = '0';
				
			if ( ! in_array( $package['destino']['pais'].'.'.$package['destino']['estado'], $ship_to_paises ) && ! in_array( $package['destino']['pais'].'.0', $ship_to_paises ) ) return false;
		endif;
		
		return true;
    } 
    
    function get_cuota( $cuota, $total ) {
		if ( strstr( $cuota, '%' ) ) :
			$cuota = ( $total / 100 ) * str_replace( '%', '', $cuota );
		endif;
		if ( ! empty( $this->cuota_minima ) && $this->cuota_minima > $cuota ) $cuota = $this->cuota_minima;
		return $cuota;
	}
	
	/**
     * Generate Settings HTML.
     *
     * Generate the HTML for the fields on the "settings" screen.
     *
     * @access public
     * @param bool $form_fields (default: false)
     * @since 1.0.0
     * @uses method_exists()
	 * @access public
	 * @return string the html for the settings
     */
    public function generate_settings_html ( $nombre='metodo_envio', $valor=array(), $form_fields = false ) {
    	if ( ! $form_fields ) {
    		$form_fields = $this->form_fields;
    	}
    	$html = '<div class="row">';
    	foreach ( $form_fields as $k => $v ) {
    		if ( ! isset( $v['type'] ) || ( $v['type'] == '' ) ) { $v['type'] == 'text'; } // Default to "text" field type.
			
			$opt_v = $valor[$k];
			
    		$html .= $this->renderCampo($v,$opt_v,$nombre);
    	}
		$html .= '</div>';
		
    	return $html;
    }
	
	protected static function loadFieldClass($entity, $type)
	{
		$class = 'Form'.ucfirst($entity).ucfirst($type);
		if (class_exists($class)) return $class;
		// Get the field search path array.
		$paths = array();
		$paths[] = CONF_REALPATH_APP.'includes/class/forms/fields';
		// If the type is complex, add the base type to the paths.
		if ($pos = strpos($type, '_')) {
			// Add the complex type prefix to the paths.
			for ($i = 0, $n = count($paths); $i < $n; $i++) {
				// Derive the new path.
				$path = $paths[$i].'/'.strtolower(substr($type, 0, $pos));
				// If the path does not exist, add it.
				if (!in_array($path, $paths)) {
					array_unshift($paths, $path);
				}
			}
			// Break off the end of the complex type.
			$type = substr($type, $pos+1);
		}
		
		// Try to find the class file.
		$file = $paths[0].'/'.strtolower($type).'.php';
		if (is_file($file)) {
			require_once $file;
		}
		
		// Check for all if the class exists.
		return class_exists($class) ? $class : false;
	}
	function renderCampo($campo=null,$valor='',$grupo=''){
		if (empty($campo['type']))
			return false;
			
		if ($campo['type'] == 'titulo'){
			return '<h4>'.$campo['titulo'].'</h4>';
		}
		
		c_include('forms/formField');
		
		$ff = self::loadFieldClass('field', $campo['type']);
		
		if (class_exists($ff)){
			$ff_obj = new $ff;
			
			$nombre = Helpers::parseaTexto($campo['titulo'], true, '_');
			$id = Helpers::parseaTexto($campo['titulo'], true, '_');
			
			if (!empty($grupo)){
				$nombre = $grupo."[".$nombre."]";
			}
			
			$ff_obj->__set('id', $id);
			$ff_obj->__set('name', $nombre);
			$ff_obj->__set('fieldname', $nombre);
			$ff_obj->__set('label', $campo['label']);
			$ff_obj->__set('description', $campo['descripcion']);
			if ($valor != ''){
				$ff_obj->__set('value', $valor);
			}
			
			if ($campo['type'] == 'hidden'){
				$ff_obj->__set('hidden', true);
			}
			
			$element = $campo['attributes'];
			
			$element["name"] = $nombre;
			$element["type"] = $campo['type'];
			$element["label"] = $campo['label'];
			$element["description"] = $campo['descripcion'];
			$element["value"] = $valor;
			
			if ($campo["type"] == 'checkbox'){
				$ff_obj->__set('value', $campo['default']);
			}
	
			$element_xml = new SimpleXMLElement('<root></root>');
			
			foreach (array_keys($element) as $key) {
				if (is_array($element[$key])) {
					$child_xml = $element_xml->addChild($key);
					foreach ($element[$key] as $kval => $cval) {
						$cxml1 = $child_xml->addAttribute($kval, $cval);
					}
				} else {
					$cxml = $element_xml->addAttribute($key, $element[$key]);
				}
			}
			if (is_array($campo['options']) && sizeof($campo['options']) > 0) {
				foreach ($campo['options'] as $kopt => $opt) {
					$child_element = $element_xml->AddChild('option', $opt);
					$child_element->AddAttribute('value', $kopt);
				}
			}
			
			$ff_obj->element = $element_xml;
			
			$input = $ff_obj->getInput($valor);
			
			if (empty($formato_label)){
				$formato_label = '<label for="##ID##">##LABEL##</label>
				';
			}
			if (empty($formato_input)){
				$formato_input = '##INPUT##
				';
			}
			
			switch ($campo['type']){ 
				case 'hidden':
					$formato_label = '';
					$formato_input = "##INPUT##";
					break;
				case 'submit':
					$formato_label = '';
					break;
				case 'checkbox':
					$formato_label = '##INPUT##';
					$formato_input = ' ';
					break;
				case 'button':
					$formato_label = '';
					break;
			}
			$formato = '<div class="form-group col-sm-6">'.$formato_label.$formato_input.'</div>';
			$out = str_replace("##ID##", $id, $formato);
			$out = str_replace("##LABEL##", $campo['label'], $out);
			$out = str_replace("##INPUT##", $input, $out);
			
			return $out;
		}
	}
}
/**
 * Shop Shipping Rate Class
 * 
 * Simple Class for storing tarifas. 
 */ 
class TarifaEnvioShop {
	var $id 	= '';
	var $nombre 	= '';
	var $coste 	= 0;
	var $tasa_manipulacion 	= 0;
	var $impuestos 	= array();
	var $tipo = '';
	
	public function __construct( $id, $nombre, $coste, $impuestos , $tipo, $tasa_manipulacion) {
		$this->id 		= $id;
		$this->nombre 	= $nombre;
		$this->coste 	= $coste;
		$this->tasa_manipulacion 	= $tasa_manipulacion;
		$this->impuestos 	= ($impuestos) ? $impuestos : array();
		$this->tipo = $tipo;
	}
	
	function get_envio_impuesto() {
		$impuestos = 0;
		if ($this->impuestos && sizeof($this->impuestos)>0) :
			$impuestos = array_sum($this->impuestos);
		endif;
		return $impuestos;
	}
}