<?php
/**
* Shop Payment Gateway class
*
* Extended by individual payment gateways to handle payments.
*
* @class 		MetodoPago
* @version		1.6.4
* @package		Shop/Abstracts
* @category	Abstract Class
* @author 		Mideas
*/
class MetodoPago
{
    /** @var string Payment method ID. */
    var $id;
    /** @var string Payment method titulo. */
    var $titulo;
    /** @var string Chosen payment method id. */
    var $elegido;
    /** @var bool True if the gateway shows fields on the checkout. */
    var $tiene_campos;
    /** @var array Array of paises this gateway is allowed for. */
    var $paises;
    /** @var string Available for all counties or specific. */
    var $disponibilidad;
    /** @var bool True if the method is activo. */
    var $activo;
    /** @var string Icon for the gateway. */
    var $icono;
    /** @var string Description for the gateway. */
    var $descripcion;
    /** @var array Array of supported features. */
    var $soporta = array('productos');
    function init_settings()
    {
        e_include('ext_shop/class/helper');
        $variables = array();
        if (!empty($this->id))
            $variables = $GLOBALS['opciones']['ext_shop_pasarelas_pago'][$this->id];
        $this->settings = $variables;
    }
    /**
    
    
    * Get the return url (thank you page)
    
    
    *
    
    
    * @access public
    
    
    * @param string $pedido (default: '')
    
    
    * @return string
    
    
    */
    function get_return_url($pedido = '')
    {
        global $shop;
        //$thanks_page_id = shop_get_page_id('thanks');
        $thanks_page_id = $shop->config['venta']['pagina_agradecimiento'];
        if ($thanks_page_id) {
            $pagina_obj = new Enlace();
            $pagina_obj->setid($thanks_page_id);
            $pagina_obj = $pagina_obj->BuscarId();
            $return_url = $pagina_obj->printLink($GLOBALS['opciones']['metadatos']['urls_amigables_metadatos']);
            if (substr($return_url, -1) != '/') {
                $return_url .= '/';
            }
        } else {
            $return_url = CONF_URL_APP;
        }
        if ($pedido):
            $return_url = Helpers::add_query_arg('pedido', $pedido->id, $return_url);
        endif;
        /*if ( is_ssl() || get_option('shop_force_ssl_checkout') == 'yes' )
        
        
        $return_url = str_replace( 'http:', 'https:', $return_url );*/
        return $return_url;
    }
    /**
    
    
    * Check If The Gateway Is Available For Use
    
    
    *
    
    
    * @access public
    
    
    * @return bool
    
    
    */
    function is_available()
    {
        if ($this->activo == "yes" || $this->activo == "1")
            return true;
    }
    /**
    
    
    * tiene_campos function.
    
    
    *
    
    
    * @access public
    
    
    * @return bool
    
    
    */
    function tiene_campos()
    {
        return $this->tiene_campos ? true : false;
    }
    /**
    
    
    * Return the gateways titulo
    
    
    *
    
    
    * @access public
    
    
    * @return string
    
    
    */
    function get_titulo()
    {
        return $this->titulo;
    }
    /**
    
    
    * Return the gateways descripcion
    
    
    *
    
    
    * @access public
    
    
    * @return string
    
    
    */
    function get_descripcion()
    {
        return $this->descripcion;
    }
    /**
    
    
    * get_icono function.
    
    
    *
    
    
    * @access public
    
    
    * @return string
    
    
    */
    function get_icono()
    {
        global $shop;
        $icono = $this->icono ? '<img src="' . $shop->force_ssl($this->icono) . '" alt="' . $this->titulo . '" />' : '';
        return $icono;
    }
    /**
    
    
    * Set As Current Gateway.
    
    
    *
    
    
    * Set this as the current gateway.
    
    
    *
    
    
    * @access public
    
    
    * @return void
    
    
    */
    function set_current()
    {
        $this->elegido = true;
    }
    /**
    
    
    * Process Payment
    
    
    *
    
    
    * Process the payment. Override this in your gateway.
    
    
    *
    
    
    * @access public
    
    
    * @return void
    
    
    */
    function procesar_pago()
    {
    }
    /**
    
    
    * Validate Frontend Fields
    
    
    *
    
    
    * Validate payment fields on the frontend.
    
    
    *
    
    
    * @access public
    
    
    * @return bool
    
    
    */
    function validar_campos()
    {
        return true;
    }
    /**
    
    
    * If There are no payment fields show the descripcion if set.
    
    
    * Override this in your gateway if you have some.
    
    
    *
    
    
    * @access public
    
    
    * @return void
    
    
    */
    function campos_pago()
    {
    }
    /**
    
    
    * Check if a gateway soporta a given feature.
    
    
    *
    
    
    * Gateways should override this to declare support (or lack of support) for a feature.
    
    
    * For backward compatibility, gateways support 'productos' by default, but nothing else.
    
    
    *
    
    
    * @access public
    
    
    * @param $feature string The name of a feature to test support for.
    
    
    * @return bool True if the gateway soporta the feature, false otherwise.
    
    
    * @since 1.5.7
    
    
    */
    function soporta($feature)
    {
        return in_array($feature, $this->soporta) ? true : false;
    }
	
	public function get_option( $key, $empty_value = null ) {
        if ( empty( $this->settings ) ) {
            $this->init_settings();
        }
        // Get option default if unset.
        if ( ! isset( $this->settings[ $key ] ) ) {
            $form_fields            = $this->form_fields;
            $this->settings[ $key ] = isset( $form_fields[ $key ]['default'] ) ? $form_fields[ $key ]['default'] : '';
        }
        if ( ! is_null( $empty_value ) && empty( $this->settings[ $key ] ) && '' === $this->settings[ $key ] ) {
            $this->settings[ $key ] = $empty_value;
        }
        return $this->settings[ $key ];
    }
    /**
    
    
    * Generate Settings HTML.
    
    
    *
    
    
    * Generate the HTML for the fields on the "settings" screen.
    
    
    *
    
    
    * @access public
    
    
    * @param bool $form_fields (default: false)
    
    
    * @since 1.0.0
    
    
    * @uses method_exists()
    
    
    * @access public
    
    
    * @return string the html for the settings
    
    
    */
    public function generate_settings_html($form_fields = false)
    {
        if (!$form_fields) {
            $form_fields = $this->form_fields;
        }
        $html = '';
        foreach ($form_fields as $k => $v) {
            if (!isset($v['type']) || ($v['type'] == '')) {
                $v['type'] == 'text';
            } // Default to "text" field type.
            $opt_v = $GLOBALS['opciones']['ext_shop_pasarelas_pago'][$this->id][$k];
            if (is_null($opt_v) && $v["type"] == "checkbox" && isset($GLOBALS['opciones']['ext_shop_pasarelas_pago'][$this->id])) {
                //$opt_v = null;
            }
            $html .= $this->renderCampo($v, $opt_v, 'metodo_pago');
        }
        echo $html;
    }
    protected static function loadFieldClass($entity, $type)
    {
        $class = 'Form' . ucfirst($entity) . ucfirst($type);
        if (class_exists($class))
            return $class;
        // Get the field search path array.
        $paths   = array();
        $paths[] = CONF_REALPATH_APP . 'includes/class/forms/fields';
        // If the type is complex, add the base type to the paths.
        if ($pos = strpos($type, '_')) {
            // Add the complex type prefix to the paths.
            for ($i = 0, $n = count($paths); $i < $n; $i++) {
                // Derive the new path.
                $path = $paths[$i] . '/' . strtolower(substr($type, 0, $pos));
                // If the path does not exist, add it.
                if (!in_array($path, $paths)) {
                    array_unshift($paths, $path);
                }
            }
            // Break off the end of the complex type.
            $type = substr($type, $pos + 1);
        }
        // Try to find the class file.
        $file = $paths[0] . '/' . strtolower($type) . '.php';
        if (is_file($file)) {
            require_once $file;
        }
        // Check for all if the class exists.
        return class_exists($class) ? $class : false;
    }
    function renderCampo($campo = null, $valor = '', $grupo = '')
    {
        if (empty($campo['type']))
            return false;
        if ($campo['type'] == 'titulo') {
            return '<h4>' . $campo['titulo'] . '</h4>';
        }
        c_include('forms/formField');
        $ff = self::loadFieldClass('field', $campo['type']);
        if (class_exists($ff)) {
            $ff_obj = new $ff;
            $nombre = Helpers::parseaTexto($campo['titulo'], true, '_');
            $id     = Helpers::parseaTexto($campo['titulo'], true, '_');
            if (!empty($grupo)) {
                $nombre = $grupo . "[" . $nombre . "]";
            }
            $ff_obj->__set('id', $id);
            $ff_obj->__set('name', $nombre);
            $ff_obj->__set('fieldname', $nombre);
            $ff_obj->__set('label', $campo['label']);
            $ff_obj->__set('description', $campo['descripcion']);
            if ($valor != '') {
                $ff_obj->__set('value', $valor);
            }
            if ($campo['type'] == 'hidden') {
                $ff_obj->__set('hidden', true);
            }
            $element                = $campo['attributes'];
            $element["name"]        = $nombre;
            $element["type"]        = $campo['type'];
            $element["label"]       = $campo['label'];
            $element["description"] = $campo['descripcion'];
            $element["value"]       = $valor;
            if ($campo["type"] == 'checkbox') {
                $ff_obj->__set('valor', $valor);
            }
            $element_xml = new SimpleXMLElement('<root></root>');
            foreach (array_keys($element) as $key) {
                if (is_array($element[$key])) {
                    $child_xml = $element_xml->addChild($key);
                    foreach ($element[$key] as $kval => $cval) {
                        $cxml1 = $child_xml->addAttribute($kval, $cval);
                    }
                } else {
                    $cxml = $element_xml->addAttribute($key, $element[$key]);
                }
            }
            if (is_array($campo['options']) && sizeof($campo['options']) > 0) {
                foreach ($campo['options'] as $kopt => $opt) {
                    $child_element = $element_xml->AddChild('option', $opt);
                    $child_element->AddAttribute('value', $kopt);
                }
            }
            $ff_obj->element = $element_xml;
            $input           = $ff_obj->getInput($valor);
			
            if (empty($formato_label)) {
                $formato_label = '<label for="##ID##">##LABEL##</label>
				';
            }
            if (empty($formato_input)) {
                $formato_input = '##INPUT##
				';
            }
            switch ($campo['type']) {
                case 'hidden':
                    $formato_label = '';
                    $formato_input = "##INPUT##";
                    break;
                case 'submit':
                    $formato_label = '';
                    break;
                case 'checkbox':
                    $formato_label = '##INPUT##';
                    $formato_input = ' ';					
                    break;
                case 'button':
                    $formato_label = '';
                    break;
            }
            $formato = '<div class="form-group">' . $formato_label . $formato_input . '</div>';
            $out     = str_replace("##ID##", $id, $formato);
            $out     = str_replace("##LABEL##", $campo['label'], $out);
            $out     = str_replace("##INPUT##", $input, $out);
            return $out;
        }
    }
    function printDeleteButton($filename = null, $plantilla = '')
    {
        if (empty($this->id) || $filename == null)
            return false;
        $output = '';
        if (empty($plantilla)) {
            $output .= '<a id="deleteButton_' . $this->id . '" href="javascript:;"><img src="' . CONF_URL_APP_ADMIN . CONF_DIR_THEMES_ADMIN . '/' . TEMPLATE_ADMIN . '/' . 'gfx/icon-delete.gif" alt="' . __('Borrar') . '" title="' . __('Borrar') . '" /></a>';
        } else {
            $aid = "deleteButton_" . $this->id;
            $txt = __('Borrar');
            $tmp = str_replace('##ID##', $aid, $plantilla);
            $tmp = str_replace('##TXT##', $txt, $tmp);
            $output .= $tmp;
        }
        return $output;
    }
    function getTitulo()
    {
        return $this->titulo;
    }
    /**
    
    
    * Returns a property of the object or the default value if the property is not set.
    
    
    *
    
    
    * @param   string  $property  The name of the property.
    
    
    * @param   mixed   $default   The default value.
    
    
    *
    
    
    * @return  mixed    The value of the property.
    
    
    *
    
    
    * @since   11.1
    
    
    *
    
    
    * @see     getProperties()
    
    
    */
    public function get($property, $default = null)
    {
        if (isset($this->$property)) {
            return $this->$property;
        }
        return $default;
    }
    /**
    
    
    * Returns a property of the object or the default value if the property is not set.
    
    
    *
    
    
    * @param   string  $property  The name of the property.
    
    
    * @param   mixed   $default   The default value.
    
    
    *
    
    
    * @return  mixed    The value of the property.
    
    
    *
    
    
    * @since   11.1
    
    
    *
    
    
    * @see     getProperties()
    
    
    */
    public function set($property, $default = null)
    {
        $this->$property = $default;
        return $default;
    }
}