<?php
/**
 * Shop Payment Gateways class
 *
 * Loads payment gateways via hooks for use in the store.
 *
 * @class 		Pago
 * @version		1.0.0
 * @package		Shop/Classes/Payment
 * @category	Class
 * @author 		Mideas
 */
defined('CONF_URL_APP') or die;
c_include('app');
e_include('ext_shop/class/metodoPago');
class Pago {
	/** @var array Array of payment gateway classes. */
	var $pasarelas_pago;
	/**
	 * __construct function.
	 *
	 * @access public
	 * @return void
	 */
	public function __construct() {
		$this->init();
	}
    /**
     * Load gateways and hook in functions.
     *
     * @access public
     * @return void
     */
    function init() {
		$config_shop = App::getConfigValue('ext_shop');
    	$load_gateways = $this->load_pasarelas_pago();
		// Get order option
		/*$ordering 	= (array) get_option('woocommerce_gateway_order');
		$order_end 	= 999;
		// Load gateways in order
		foreach ($load_gateways as $gateway) :
			$load_gateway = new $gateway();
			if (isset($ordering[$load_gateway->id]) && is_numeric($ordering[$load_gateway->id])) :
				// Add in position
				$this->pasarelas_pago[$ordering[$load_gateway->id]] = $load_gateway;
			else :
				// Add to end of the array
				$this->pasarelas_pago[$order_end] = $load_gateway;
				$order_end++;
			endif;
		endforeach;*/
		ksort( $this->pasarelas_pago );
    }
	
	private function load_pasarelas_pago() {
		if ( sizeof( $this->pasarelas_pago ) == 0 ){
			$path = CONF_REALPATH_APP . 'extensions/ext_shop/class/pasarelas_pago';
			$files = Helpers::dir_list($path, '','.class.php',false);
			
			if (sizeof( $files[CONF_REALPATH_APP . 'extensions/ext_shop/class/pasarelas_pago'] ) > 0){
				foreach ($files[CONF_REALPATH_APP . 'extensions/ext_shop/class/pasarelas_pago'] as $pasarela_class){
					$pasarela_name = substr($pasarela_class, 0, strlen($pasarela_class) - 10);
					$pasarela_class_name = "Pago" . ucfirst($pasarela_name);
					
					if (!class_exists($pasarela_class_name))
						include_once($path.'/'.$pasarela_class);
					
					$dummy_class = new $pasarela_class_name();
					
					$this->pasarelas_pago[ $dummy_class->id ] = $dummy_class;
				}
			}
		}
		
		return true;
	}
    /**
     * Get gateways.
     *
     * @access public
     * @return array
     */
    function pasarelas_pago() {
		$_available_gateways = array();
		if ( sizeof( $this->pasarelas_pago ) > 0 )
			foreach ( $this->pasarelas_pago as $gateway )
				$_available_gateways[ $gateway->id ] = $gateway;
		return $_available_gateways;
	}
	/**
	 * Get available gateways.
	 *
	 * @access public
	 * @return array
	 */
	function get_available_pasarelas_pago($admin=false) {
		$_available_gateways = array();
		foreach ( $this->pasarelas_pago as $gateway ) :
			if ($gateway->is_available() || $admin == true) $_available_gateways[$gateway->id] = $gateway;
		endforeach;
		return $_available_gateways;
	}
	/**
	 * Save options in admin.
	 *
	 * @access public
	 * @return void
	 */
	function process_admin_options() {
		$default_gateway = ( isset( $_POST['default_gateway'] ) ) ? esc_attr( $_POST['default_gateway'] ) : '';
		$gateway_order = ( isset( $_POST['gateway_order'] ) ) ? $_POST['gateway_order'] : '';
		$order = array();
		if ( is_array( $gateway_order ) && sizeof( $gateway_order ) > 0 ) {
			$loop = 0;
			foreach ( $gateway_order as $gateway_id ) {
				$order[ esc_attr( $gateway_id ) ] = $loop;
				$loop++;
			}
		}
		update_option( 'woocommerce_default_gateway', $default_gateway );
		update_option( 'woocommerce_gateway_order', $order );
	}
}