<?php
/**
 * BBVA Standard Payment Gateway
 *
 * Provides a PayPal Standard Payment Gateway.
 *
 * @class 		WC_Paypal
 * @extends		PagoPaypal
 * @version		2.0.0
 * @package		Shop/Classes/Payment
 * @author 		WooThemes
 */
 defined('CONF_URL_APP') or die;
 
class PagoBbva extends MetodoPago {
	var $notificar_url;
    /**
     * Constructor for the gateway.
     *
     * @access public
     * @return void
     */
	public function __construct() {
		global $shop;
        $this->id           = 'bbva';
        $this->icono         = CONF_URL_APP . 'extensions/ext_shop/assets/img/iconos/bbva.png';
        $this->tiene_campos   = false;
        $this->liveurl      = 'https://w3.grupobbva.com/TLPV/tlpv/TLPV_pub_RecepOpModeloServidor';
		$this->testurl      = 'https://w3.grupobbva.com/TLPV/tlpv/TLPV_pub_RecepOpModeloServidor';
        $this->titulo_metodo = __( 'TPV BBVA' );
        $this->notificar_url   = Helpers::add_query_arg('payment','bbva',Helpers::resolveLink(false, 'ext_shop', 'cesta_fin', __('Procesando_pago')).'/') . '&';
		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();
		
		if (sizeof($this->settings) == 0){
			$this->settings = array(	'titulo' 			=> 	__('TPV BBVA'),
										'descripcion'		=>	__('Pasarela de pago del BBVA.'),
										'codigo_comercio'	=> '175356769D00001',
										'terminal'			=>	'999999',
										'palabra_secreta'	=>	'',
										'clave_xor'			=>	'',
										'debug'				=>	'no',
										'metodo_envio_formulario'	=>	'yes',
										'prefijo_factura'	=>	'SH_',
										'activo'			=>	'no'
									);
		}
		
		// Define user set variables
		$this->titulo 			= $this->settings['titulo'];
		$this->descripcion 		= $this->settings['descripcion'];
		$this->codigo_comercio 	= $this->settings['codigo_comercio'];
		$this->terminal  		= $this->settings['terminal'];
		$this->palabra_secreta	= $this->settings['palabra_secreta'];
		$this->clave_xor		= $this->settings['clave_xor'];
		$this->debug			= $this->settings['debug'];
		$this->metodo_envio_formulario = $this->settings['metodo_envio_formulario'] == 'no' ? false : true;
		$this->prefijo_factura	= $this->settings['prefijo_factura'];
		$this->activo	= $this->settings['activo'];
		
		
		// Logs
		if ( 'yes' == $this->debug )
			$this->log = $shop->logger();
		// Actions
		//add_action( 'valid-paypal-standard-ipn-request', array( $this, 'solicitud_exitosa' ) );
		//add_action( 'shop_receipt_paypal', array( $this, 'pagina_recibo' ) );
		//add_action( 'shop_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		// Payment listener/API hook
		//add_action( 'shop_api_wc_gateway_paypal', array( $this, 'comprobar_respuesta_ipn' ) );
		if ( !$this->es_valido_para_uso() ) $this->activo = false;
    }
    /**
     * Check if this gateway is activo and available in the user's country
     *
     * @access public
     * @return bool
     */
    function es_valido_para_uso() {
        if ( ! in_array( ShopHelper::get_shop_moneda(), array( 'USD', 'EUR' ) ) ) return false;
        return true;
    }
	
	private function get_bbva_moneda($moneda=''){
		if (empty($moneda)){
			$moneda = ShopHelper::get_shop_moneda();
		}
		switch ($moneda){
			case 'EUR':
				return 978;
				break;
			case 'USD':
				return 840;
				break;
			default:
				return false;
				break;
		}
	}
	/**
	 * Admin Panel Options
	 * - Options for bits like 'titulo' and availability on a country-by-country basis
	 *
	 * @since 1.0.0
	 */
	public function admin_options() {
		?>
		<h3><?php echo __( 'TPV de BBVA' ); ?></h3>
		<p><?php echo __( 'Este módulo te permite pagar a través del TPV de BBVA. Si el cliente elije este modo de pago, podrá pagar de forma con tarjeta de crédito, débito y Mobipay.'); ?></p>
    	<?php if ( $this->es_valido_para_uso() ) : ?>
			<table class="form-table">
			<?php
    			// Generate the HTML For the settings form.
    			$this->generate_settings_html();
			?>
			</table><!--/.form-table-->
		<?php else : ?>
            <div class="inline error"><p><strong><?php echo __( 'Gateway Disabled' ); ?></strong>: <?php echo __( 'TPV BBVA does not support your store currency.' ); ?></p></div>
		<?php
			endif;
	}
    /**
     * Initialise Gateway Settings Form Fields
     *
     * @access public
     * @return void
     */
    function init_form_fields() {
    	$this->form_fields = array(
			'activo' => array(
							'titulo' => __( 'activo' ),
							'type' => 'checkbox',
							'label' => __( 'Activar TPV de BBVA' ),
							'default' => 'yes'
						),
			'titulo' => array(
							'titulo' => __( 'titulo' ),
							'label' => __( 'Título' ),
							'type' => 'text',
							'descripcion' => '',
							'default' => __( 'TPV de BBVA' ),
							'desc_tip'      => true,
							'attributes'	=>	array('multilanguage' => 'true')
						),
			'descripcion' => array(
							'titulo' => __( 'descripcion' ),
							'label' => __( 'Descripción' ),
							'type' => 'textarea',
							'descripcion' => '',
							'default' => __( 'Pasarela de pago del BBVA.' ),
							'attributes'	=>	array('multilanguage' => 'true')
						),
			'codigo_comercio' => array(
							'titulo' => __( 'codigo_comercio' ),
							'label' => __( 'Código de comercio' ),
							'type' 			=> 'text',
							'descripcion' => '',
							'default' => '',
							'desc_tip'      => true,
							'placeholder'	=> ''
						),
			'terminal' => array(
							'titulo' => __( 'terminal' ),
							'label' => __( 'Identificador del terminal' ),
							'type' 			=> 'text',
							'descripcion' => '',
							'default' => '',
							'desc_tip'      => true,
							'placeholder'	=> ''
						),
			'palabra_secreta' => array(
							'titulo' => __( 'palabra_secreta' ),
							'label' => __( 'Palabra secreta' ),
							'type' => 'text',
							'descripcion' => '',
							'default' => '',
							'desc_tip'      => true,
						),
			'clave_xor' => array(
							'titulo' => __( 'clave_xor' ),
							'label' => __( 'Clave XOR' ),
							'type' => 'text',
							'descripcion' => '',
							'default' => '',
							'desc_tip'      => true,
						),
			/*'metodo_envio_formulario' => array(
							'titulo' => __( 'metodo_envio_formulario' ),
							'type' => 'checkbox',
							'label' => __( 'Usar el método de envío de formulario' ),
							'descripcion' => '',
							'default' => 'no'
						),*/
			'testing' => array(
							'titulo' => __( 'Modo pruebas' ),
							'type' => 'titulo',
							'descripcion' => '',
						),
			'debug' => array(
							'titulo' => __( 'debug' ),
							'type' => 'checkbox',
							'label' => __( 'Activar debug' ),
							'default' => 'no',
							'descripcion' => __( 'Log PayPal events, such as IPN requests, inside <code>shop/logs/paypal.txt</code>' ),
						)
			);
    }
	
	private function desobfuscate($pal_sec_ofuscada,$clave_xor) 
    { 
        $trozos = explode (";", $pal_sec_ofuscada); 
        $tope = count($trozos); 
         
        $res=""; 
         
        for ($i=0; $i<$tope; $i++) 
        { 
                $x1=ord($clave_xor[$i]); 
                $x2=hexdec($trozos[$i]); 
                $r=$x1 ^ $x2; 
                $res.=chr($r); 
        } 
        return($res); 
     }
	/**
	 * Get PayPal Args for passing to PP
	 *
	 * @access public
	 * @param mixed $order
	 * @return array
	 */
	function get_bbvaargs( $order ) {
		global $shop;
		$id_transaccion = $order->id;
		
		if (strlen($id_transaccion) < 12){
			$yep = strlen($id_transaccion);
			for ($i=0; $i < 12 - $yep; $i++){
				$id_transaccion = '0'.$id_transaccion;
			}
		}
		
		//Datos proporcionados por el BBVA: 
		$MODULE_PAYMENT_BBVA_SIGNATURE_KEY=$this->clave_xor; 
		$MODULE_PAYMENT_BBVA_COMMERCE_ID=$this->codigo_comercio; 
		$MODULE_PAYMENT_BBVA_SIGNATURE=$this->palabra_secreta;
		$MODULE_PAYMENT_BBVA_TERMINAL_ID=$this->terminal; 
		
		$des_key=$MODULE_PAYMENT_BBVA_SIGNATURE_KEY.substr($MODULE_PAYMENT_BBVA_COMMERCE_ID,0,9)."***"; 
		$obfuscated=$MODULE_PAYMENT_BBVA_SIGNATURE; 
		$desobfuscated=$this->desobfuscate($obfuscated, $des_key);
		
		$importe=$order->datos['total']['total']; 
		$importe=round($importe,2); 
	  	$importe=number_format($importe, 2, '.', '');       
	   	$nf_importe=str_replace('.','',$importe);
		
		$nf_importe=str_replace('.','',$importe);                    
		$lt="&lt;"; 
		$gt="&gt;"; 
		$moneda=(string)$this->get_bbva_moneda();
		
		//Session id to return back 
		$localizador=$order->id; 
		
		//Signature calculation 
		$datosfirma = $MODULE_PAYMENT_BBVA_TERMINAL_ID.$MODULE_PAYMENT_BBVA_COMMERCE_ID.$id_transaccion.$nf_importe.$moneda.$localizador.$desobfuscated; 
		$datosfirma2 = $MODULE_PAYMENT_BBVA_TERMINAL_ID."-".$MODULE_PAYMENT_BBVA_COMMERCE_ID."-".$id_transaccion."-".$nf_importe."-".$moneda."-".$desobfuscated; 
		$firma=strtoupper(sha1($datosfirma));                   
		$url_redir=$this->get_return_url( $order ); //url donde redireccionas despues de realizar el pago. 
		
		$url_comercio=$this->notificar_url;//url donde compruebas que el pago se realiza correctamente. 
		
		$xml.=$lt."tpv".$gt; 
		$xml.=$lt."oppago".$gt; 
		$xml.=$lt."idterminal".$gt.$MODULE_PAYMENT_BBVA_TERMINAL_ID.$lt."/idterminal".$gt; 
		$xml.=$lt."idcomercio".$gt.$MODULE_PAYMENT_BBVA_COMMERCE_ID.$lt."/idcomercio".$gt;                     
		$xml.=$lt."idtransaccion".$gt.$id_transaccion.$lt."/idtransaccion".$gt; 
		$xml.=$lt."moneda".$gt.$moneda.$lt."/moneda".$gt;             
		$xml.=$lt."importe".$gt.$importe.$lt."/importe".$gt;                         
		$xml.=$lt."urlcomercio".$gt.$url_comercio.$lt."/urlcomercio".$gt;             
		$xml.=$lt."idioma".$gt."es".$lt."/idioma".$gt;                         
		$xml.=$lt."pais".$gt."ES".$lt."/pais".$gt;                                     
		$xml.=$lt."urlredir".$gt.$url_redir.$lt."/urlredir".$gt;                                     
		$xml.=$lt."firma".$gt.$firma.$lt."/firma".$gt;                                                             
		$xml.=$lt."localizador".$gt.$localizador.$lt."/localizador".$gt; 
		$xml.=$lt."/oppago".$gt; 
		$xml.=$lt."/tpv".$gt;
		
		// BBVA Args
		$bbvaargs = 
			array(
				'peticion'				=> $xml
				);
		return $bbvaargs;
	}
    /**
	 * Generate the paypal button link
     *
     * @access public
     * @param mixed $order_id
     * @return string
     */
    function generar_formulario_bbva( $order ) {
		global $shop;
		//$order = new PedidoShop( $order_id );
		
		$bbvaadr = $this->liveurl;
		$bbvaargs = $this->get_bbvaargs( $order );
		$bbvaargs_array = array();
		foreach ($bbvaargs as $key => $value) {
			$bbvaargs_array[] = '<input type="hidden" name="'.$key.'" value="'.$value.'" />';
		}
		/*$shop->add_inline_js( '
			jQuery("body").block({
					message: "<img src=\"' . esc_url( apply_filters( 'shop_ajax_loader_url', $shop->plugin_url() . '/assets/img/ajax-loader.gif' ) ) . '\" alt=\"Redirecting&hellip;\" style=\"float:left; margin-right: 10px;\" />'.__( 'Thank you for your order. We are now redirecting you to PayPal to make payment.' ).'",
					overlayCSS:
					{
						background: "#fff",
						opacity: 0.6
					},
					css: {
				        padding:        20,
				        textAlign:      "center",
				        color:          "#555",
				        border:         "3px solid #aaa",
				        backgroundColor:"#fff",
				        cursor:         "wait",
				        lineHeight:		"32px"
				    }
				});
			jQuery("#submit_bbvapayment_form").click();
		' );*/
		return '<form action="'. $bbvaadr .'" method="post" id="bbvapayment_form" target="popup" class="popupform">
				' . implode( '', $bbvaargs_array) . '
				<input type="submit" class="btn btn-large btn-inverse" id="submit_bbvapayment_form" value="'.__( 'Pagar a través de BBVA' ).'" /> <input type="button" class="btn  btn-large btn-inverse cancel" value="'.__( 'Cancelar pedido &amp; vaciar el carrito' ).'" />
			</form>';
	}
    /**
     * Process the payment and return the result
     *
     * @access public
     * @param int $order_id
     * @return array
     */
	function procesar_pago( $order_id ) {
		
		$order = new PedidoShop();
		$order->setid($order_id);
		$order = $order->BuscarId();
		/*if ( ! $this->metodo_envio_formulario ) {
			$bbvaargs = $this->get_bbvaargs( $order );
			$bbvaargs = http_build_query( $bbvaargs, '', '&' );
			$bbvaadr = $this->liveurl . '?';
			return array(
				'result' 	=> 'info',
				'redirect'	=> $bbvaadr . $bbvaargs
			);
		} else {*/
			return array(
				'result' 	=> 'info',
				'redirect'	=> Helpers::add_query_arg('pedido', $order->id, Helpers::resolveLink(false,'ext_shop','cesta_pago_form',__('Finalizar_pago')).'/')
			);
		//}
	}
    /**
     * Output for the order received page.
     *
     * @access public
     * @return void
     */
	function pagina_recibo( $order ) {
		
		echo '<p>'.__( 'Gracias por su pedido, haz clic en el botón de abajo para pagar con tarjeta de crédito.' ).'</p>';
		echo $this->generar_formulario_bbva( $order );
	}
	/**
	 * Check PayPal IPN validity
	 **/
	function comprobar_solicitud_ipn_valida() {
		global $shop;
		/*if ( 'yes' == $this->debug )
			$this->log->add( 'paypal', 'Checking IPN response is valid...' );*/
    	// Get recieved values from post data
		$received_values = array( 'cmd' => '_notify-validate' );
		$received_values += Helpers::stripslashes_deep( $_POST );
        // Send back post vars to paypal
        $params = array(
        	'body' 			=> $received_values,
        	'sslverify' 	=> false,
        	'timeout' 		=> 60,
        	'user-agent'	=> 'Shop/1.0'
        );
        /*if ( 'yes' == $this->debug )
			$this->log->add( 'paypal', 'IPN Request: ' . print_r( $params, true ) );*/
        // Get url
       	if ( $this->testmode == 'yes' )
			$bbvaadr = $this->testurl;
		else
			$bbvaadr = $this->liveurl;
			
		return true;
		// Post back to get a response
        //$response = $this->_post( $bbvaadr, $params );
	
			/*if ( 'yes' == $this->debug )
				$this->log->add( 'paypal', 'IPN Response: ' . print_r( $response, true ) );*/
	
			// check to see if the request was valid
			if ( /*! is_wp_error( $response ) && */$response['response']['code'] >= 200 && $response['response']['code'] < 300 && ( strcmp( $response['body'], "VERIFIED" ) == 0 ) ) {
				/*if ( 'yes' == $this->debug )
					$this->log->add( 'paypal', 'Received valid response from PayPal' );*/
	
				return true;
			}
        /*if ( 'yes' == $this->debug ) {
        	$this->log->add( 'paypal', 'Received invalid response from PayPal' );
        	if ( is_wp_error( $response ) )
        		$this->log->add( 'paypal', 'Error response: ' . $response->get_error_message() );
        }*/
        return false;
    }
	/**
	 * Check for BBVA IPN Response
	 *
	 * @access public
	 * @return void
	 */
	function comprobar_respuesta_ipn() {
		if (empty($_GET['?peticion'])){
			if (!empty($_GET['amp;?peticion'])){
				$peticion = $_GET['amp;?peticion'];
			} else {
				return false;
			}
		} else {
			$peticion = $_GET['?peticion'];
		}
		$tmp_xml = new SimpleXMLElement($peticion);
		
		var_dumpre($tmp_xml);
		exit;
		
		@ob_clean();
    	if ( ! empty( $_POST ) && $this->comprobar_solicitud_ipn_valida() ) {
    		header( 'HTTP/1.1 200 OK' );
        	//do_action( "valid-paypal-standard-ipn-request", $_POST );
			
			$this->solicitud_exitosa( $_POST );
		} else {
			//wp_die( "PayPal IPN Request Failure" );
			die( "BBVA IPN Request Failure" );
   		}
	}
	/**
	 * Successful Payment!
	 *
	 * @access public
	 * @param array $posted
	 * @return void
	 */
	function solicitud_exitosa( $posted ) {
		global $shop;
		$posted = Helpers::stripslashes_deep( $posted );
		// Custom holds post ID
	    if ( ! empty( $posted['invoice'] ) && ! empty( $posted['custom'] ) ) {
		    $order = $this->get_pedido_paypal( $posted );
		     /*if ( 'yes' == $this->debug )
	        	$this->log->add( 'paypal', 'Found order #' . $order->id );*/
		    // Lowercase returned variables
	        $posted['payment_status'] 	= strtolower( $posted['payment_status'] );
	        $posted['txn_type'] 		= strtolower( $posted['txn_type'] );
			
			$order->datos['metodo_pago']['estado'] = $posted['payment_status'];
	        /*if ( 'yes' == $this->debug )
	        	$this->log->add( 'paypal', 'Payment status: ' . $posted['payment_status'] );*/
	        // We are here so lets check status and do actions
	        switch ( $posted['payment_status'] ) {
				case 'pending' :
					$order->datos['metodo_pago']['razon_estado'] = $posted['pending_reason'];
					
					$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_creado'] );
					$order->Modificar();
					
					$emailer = new Emailer();
				
					$to_array = array($GLOBALS['opciones']['emailer']['email_emailer']=>$GLOBALS['opciones']['emailer']['nombre_emailer']);
					$asunto = sprintf(__('Pedido # %s procesado mediante PayPal pendiente'), $order->id);
					
					$emailer->to = $to_array;
					$emailer->asunto = $asunto;
					
					$emailer->contenido = '<h1>'.$asunto.'</h1>';
					//$emailer->contenido .= print_r($_POST, true);
					//$emailer->contenido .= print_r($metodo_pago->id, true);
					$emailer->contenido .= '<p>' . sprintf(__('El pedido # %s se ha procesado mediante PayPal con el siguiente resultado: %s (%s)'), $order->id, $posted['payment_status'], $posted['pending_reason']) . '</p>';
					
					if ($emailer->enviar()){
					}
					break;
					
	            case 'completed' :
	            	// Check order not already completed
	            	if ( $order->estado == 'completado' ) {
	            		 /*if ( 'yes' == $this->debug )
	            		 	$this->log->add( 'paypal', 'Aborting, Order #' . $order->id . ' is already complete.' );*/
	            		 exit;
	            	}
	            	// Check valid txn_type
	            	$accepted_types = array( 'cart', 'instant', 'express_checkout', 'web_accept', 'masspay', 'send_money' );
					if ( ! in_array( $posted['txn_type'], $accepted_types ) ) {
						/*if ( 'yes' == $this->debug )
							$this->log->add( 'paypal', 'Aborting, Invalid type:' . $posted['txn_type'] );*/
						exit;
					}
					// Validate Amount
				    if ( (float) $order->datos['total']['total'] != (float) $posted['mc_gross'] ) {
				    	/*if ( 'yes' == $this->debug )
				    		$this->log->add( 'paypal', 'Payment error: Amounts do not match (gross ' . $posted['mc_gross'] . ')' );*/
				    	// Put this order en-espera for manual checking
				    	$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_creado'] );
						$order->Modificar();
				    	exit;
				    }
					
				    // Validate Email Address
					if ( strcasecmp( trim( $posted['receiver_email'] ), trim( $this->receptor_email ) ) != 0 ) {
						/*if ( 'yes' == $this->debug )
							$this->log->add( 'paypal', "IPN Response is for another one: {$posted['receptor_email']} our email is {$this->receptor_email}" );*/
						// Put this order en-espera for manual checking
				    	$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_creado'] );
						$order->Modificar();
				    	exit;
					}
					
					$emailer = new Emailer();
				
					$to_array = array($GLOBALS['opciones']['emailer']['email_emailer']=>$GLOBALS['opciones']['emailer']['nombre_emailer']);
					$asunto = sprintf(__('Pedido # %s pagado mediante PayPal'), $order->id);
					
					$emailer->to = $to_array;
					$emailer->asunto = $asunto;
					
					$emailer->contenido = '<h1>'.$asunto.'</h1>';
					//$emailer->contenido .= print_r($_POST, true);
					//$emailer->contenido .= print_r($metodo_pago->id, true);
					$emailer->contenido .= '<p>' . sprintf(__('El pedido # %s se ha pagado mediante PayPal con el siguiente resultado: %s'), $order->id, $posted['payment_status']) . '</p>';
					
					if ($emailer->enviar()){
					}
					 // Store PP Details
	                /*if ( ! empty( $posted['payer_email'] ) )
	                	update_post_meta( $order_id, 'Payer PayPal address', $posted['payer_email'] );
	                if ( ! empty( $posted['txn_id'] ) )
	                	update_post_meta( $order_id, 'Transaction ID', $posted['txn_id'] );
	                if ( ! empty( $posted['first_name'] ) )
	                	update_post_meta( $order_id, 'Payer first name', $posted['first_name'] );
	                if ( ! empty( $posted['last_name'] ) )
	                	update_post_meta( $order_id, 'Payer last name', $posted['last_name'] );
	                if ( ! empty( $posted['payment_type'] ) )
	                	update_post_meta( $order_id, 'Payment type', $posted['payment_type'] );*/
	            	// Payment completed
	                //$order->add_order_note( __( 'IPN payment completed' ) );
	                $order->pago_completo();
					//$order->setestado('completado');
					//$order->Modificar();
	                /*if ( 'yes' == $this->debug )
	                	$this->log->add( 'paypal', 'Payment complete.' );*/
	            break;
	            case 'denied' :
	            case 'expired' :
	            case 'failed' :
	            case 'voided' :
	                // Order failed
	                $order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_fallido'] );
					$order->Modificar();
	            break;
	            case "refunded" :
	            	// Only handle full refunds, not partial
	            	if ( (float) $order->datos['total']['total'] == (float) ( $posted['mc_gross'] * -1 ) ) {
		            	// Mark order as refunded
		            	$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_cancelado'] );
						$order->Modificar();
						
						$emailer = new Emailer();
				
						$to_array = array($GLOBALS['opciones']['emailer']['email_emailer']=>$GLOBALS['opciones']['emailer']['nombre_emailer']);
						$asunto = sprintf(__('Pago del pedido # %s reembolsado/revertido'), $order->id);
						
						$emailer->to = $to_array;
						$emailer->asunto = $asunto;
						
						$emailer->contenido = '<h1>'.$asunto.'</h1>';
						//$emailer->contenido .= print_r($_POST, true);
						//$emailer->contenido .= print_r($metodo_pago->id, true);
						$emailer->contenido .= '<p>' . sprintf(__('El pedido # %s ha sido marcado como reembolsado - PayPal reason code: %s'), $order->id, $posted['reason_code']) . '</p>';
						
						if ($emailer->enviar()){
						}
		            	/*$mailer = $shop->mailer();
		            	$message = $mailer->wrap_message(
		            		__( 'Order refunded/reversed' ),
		            		sprintf( __( 'Order %s has been marked as refunded - PayPal reason code: %s' ), $order->get_order_number(), $posted['reason_code'] )
						);
						$mailer->send( get_option( 'admin_email' ), sprintf( __( 'Payment for order %s refunded/reversed' ), $order->get_order_number() ), $message );*/
					}
	            break;
	            case "reversed" :
	            case "chargeback" :
	            	// Mark order as refunded
	            	$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_cancelado'] );
					$order->Modificar();
					
					$emailer = new Emailer();
				
					$to_array = array($GLOBALS['opciones']['emailer']['email_emailer']=>$GLOBALS['opciones']['emailer']['nombre_emailer']);
					$asunto = sprintf(__('Pago del pedido # %s reembolsado/revertido'), $order->id);
					
					$emailer->to = $to_array;
					$emailer->asunto = $asunto;
					
					$emailer->contenido = '<h1>'.$asunto.'</h1>';
					//$emailer->contenido .= print_r($_POST, true);
					//$emailer->contenido .= print_r($metodo_pago->id, true);
					$emailer->contenido .= '<p>' . sprintf(__('El pedido # %s ha sido marcado como reembolsado - PayPal reason code: %s'), $order->id, $posted['reason_code']) . '</p>';
					
					if ($emailer->enviar()){
					}
	            	/*$mailer = $shop->mailer();
	            	$message = $mailer->wrap_message(
	            		__( 'Order refunded/reversed' ),
	            		sprintf(__( 'Order %s has been marked as refunded - PayPal reason code: %s' ), $order->get_order_number(), $posted['reason_code'] )
					);
					$mailer->send( get_option( 'admin_email' ), sprintf( __( 'Payment for order %s refunded/reversed' ), $order->get_order_number() ), $message );*/
	            break;
	            default :
	            	// No action
	            break;
	        }
			
			exit;
	    }
	}
	/**
	 * get_pedido_paypal function.
	 *
	 * @access public
	 * @param mixed $posted
	 * @return void
	 */
	function get_pedido_paypal( $posted ) {
		$custom = Helpers::maybe_unserialize( $posted['custom'] );
    	// Backwards comp for IPN requests
    	if ( is_numeric( $custom ) ) {
	    	$order_id = (int) $custom;
	    	$order_key = $posted['invoice'];
    	} elseif( is_string( $custom ) ) {
	    	$order_id = (int) str_replace( $this->prefijo_factura, '', $custom );
	    	$order_key = $custom;
    	} else {
    		list( $order_id, $order_key ) = $custom;
		}
		$order = new PedidoShop();
		$order->setid($order_id);
		$order = $order->BuscarId();
		if ( ! isset( $order->id ) ) {
			// We have an invalid $order_id, probably because prefijo_factura has changed
			$order_id 	= shop_get_order_id_by_order_key( $order_key );
			$order = new PedidoShop();
			$order->setid($order_id);
			$order = $order->BuscarId();
		}
		// Validate key
		/*if ( $order->order_key !== $order_key ) {
        	if ( $this->debug=='yes' )
        		$this->log->add( 'paypal', 'Error: Order Key does not match invoice.' );
        	exit;
        }*/
        return $order;
	}
}