<?php
/**
 * PayPal Standard Payment Gateway
 *
 * Provides a PayPal Standard Payment Gateway.
 *
 * @class 		PagoPaypal
 * @extends		MetodoPago
 * @version		1.0.0
 * @package		Shop/Classes/Payment
 * @author 		Mideas
 */
 defined('CONF_URL_APP') or die;
 
class PagoPaypal extends MetodoPago {
	var $notificar_url;
    /**
     * Constructor for the gateway.
     *
     * @access public
     * @return void
     */
	public function __construct() {
		global $shop;
        $this->id           = 'paypal';
        $this->icono         = CONF_URL_APP . 'extensions/ext_shop/assets/img/iconos/paypal.png';
        $this->tiene_campos   = false;
        $this->liveurl      = 'https://www.paypal.com/cgi-bin/webscr';
		$this->testurl      = 'https://www.sandbox.paypal.com/cgi-bin/webscr';
        $this->titulo_metodo = __( 'PayPal' );
        $this->notificar_url   = Helpers::add_query_arg('payment','paypal',Helpers::resolveLink(false, 'ext_shop', 'cesta_fin', __('Procesando_pago')).'/');
		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();
		
		if (sizeof($this->settings) == 0){
			$this->settings = array(	'titulo' 			=> 	__('Paypal'),
										'descripcion'		=>	__('Pasarela de pago de Paypal.'),
										'email'				=> 'test@email.com',
										'receptor_email'	=>	'test@email.com',
										'enviar_envio'		=>	'yes',
										'reemplazar_direccion'	=>	'no',
										'debug'				=>	'no',
										'metodo_envio_formulario'	=>	'yes',
										'estilo_pagina'		=>	'',
										'prefijo_factura'	=>	'SH_',
										'activo'			=>	'no',
										'recargo'			=>	''
									);
		}
		
		// Define user set variables
		$this->titulo 			= $this->settings['titulo'];
		$this->descripcion 		= $this->settings['descripcion'];
		$this->email 			= $this->settings['email'];
		$this->receptor_email   = $this->settings['receptor_email'];
		$this->testmode			= $this->settings['testmode'];
		$this->enviar_envio	= $this->settings['enviar_envio'];
		$this->reemplazar_direccion	= $this->settings['reemplazar_direccion'];
		$this->debug			= $this->settings['debug'];
		$this->metodo_envio_formulario = $this->settings['metodo_envio_formulario'] == 'no' ? false : true;
		$this->estilo_pagina 		= $this->settings['estilo_pagina'];
		$this->prefijo_factura	= $this->settings['prefijo_factura'];
		$this->activo	= $this->settings['activo'];
		$this->recargo			= $this->settings['recargo'];
		// Logs
		if ( 'yes' == $this->debug )
			$this->log = $shop->logger();
		// Actions
		//add_action( 'valid-paypal-standard-ipn-request', array( $this, 'solicitud_exitosa' ) );
		//add_action( 'shop_receipt_paypal', array( $this, 'pagina_recibo' ) );
		//add_action( 'shop_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		// Payment listener/API hook
		//add_action( 'shop_api_wc_gateway_paypal', array( $this, 'comprobar_respuesta_ipn' ) );
		if ( !$this->es_valido_para_uso() ) $this->activo = false;
    }
    /**
     * Check if this gateway is activo and available in the user's country
     *
     * @access public
     * @return bool
     */
    function es_valido_para_uso() {
        if ( ! in_array( ShopHelper::get_shop_moneda(), array( 'AUD', 'BRL', 'CAD', 'MXN', 'NZD', 'HKD', 'SGD', 'USD', 'EUR', 'JPY', 'TRY', 'NOK', 'CZK', 'DKK', 'HUF', 'ILS', 'MYR', 'PHP', 'PLN', 'SEK', 'CHF', 'TWD', 'THB', 'GBP', 'RMB' ) ) ) return false;
        return true;
    }
	/**
	 * Admin Panel Options
	 * - Options for bits like 'titulo' and availability on a country-by-country basis
	 *
	 * @since 1.0.0
	 */
	public function admin_options() {
		?>
		<h3><?php echo __( 'PayPal estándar' ); ?></h3>
		<p><?php echo __( 'PayPal estándar funciona mediante el envío al usuario de PayPal para introducir su información de pago.'); ?></p>
    	<?php if ( $this->es_valido_para_uso() ) : ?>
			<table class="form-table">
			<?php
    			// Generate the HTML For the settings form.
    			$this->generate_settings_html();
			?>
			</table><!--/.form-table-->
		<?php else : ?>
            <div class="inline error"><p><strong><?php echo __( 'Gateway Disabled' ); ?></strong>: <?php echo __( 'PayPal does not support your store currency.' ); ?></p></div>
		<?php
			endif;
	}
    /**
     * Initialise Gateway Settings Form Fields
     *
     * @access public
     * @return void
     */
    function init_form_fields() {
    	$this->form_fields = array(
			'activo' => array(
							'titulo' => __( 'activo' ),
							'type' => 'checkbox',
							'label' => __( 'Activar PayPal estándar' ),
							'default' => 'yes'
						),
			'titulo' => array(
							'titulo' => __( 'titulo' ),
							'label' => __( 'Título' ),
							'type' => 'text',
							'descripcion' => __( 'This controls the titulo which the user sees during checkout.' ),
							'default' => __( 'PayPal' ),
							'desc_tip'      => true,
							'attributes'	=>	array('multilanguage' => 'true')
						),
			'descripcion' => array(
							'titulo' => __( 'descripcion' ),
							'label' => __( 'Descripción' ),
							'type' => 'textarea',
							'descripcion' => __( 'This controls the descripcion which the user sees during checkout.' ),
							'default' => __( 'Pay via PayPal; you can pay with your credit card if you don\'t have a PayPal account' ),
							'attributes'	=>	array('multilanguage' => 'true')
						),
			'email' => array(
							'titulo' => __( 'email' ),
							'label' => __( 'Email PayPal' ),
							'type' 			=> 'email',
							'descripcion' => __( 'Please enter your PayPal email address; this is needed in order to take payment.' ),
							'default' => '',
							'desc_tip'      => true,
							'attributes'	=> array('placeholder'	=> 'you@youremail.com','class'	=> 'form-control')
							
						),
			'receptor_email' => array(
							'titulo' => __( 'receptor_email' ),
							'label' => __( 'Email del receptor' ),
							'type' 			=> 'email',
							'descripcion' => __( 'If this differs from the email entered above, input your main receiver email for your PayPal account. This is used to validate IPN requests.' ),
							'default' => '',
							'desc_tip'      => true,
							'attributes'	=> array('placeholder'	=> 'you@youremail.com','class'	=> 'form-control')
						),
			'prefijo_factura' => array(
							'titulo' => __( 'prefijo_factura' ),
							'label' => __( 'Prefijo de la factura' ),
							'type' => 'text',
							'descripcion' => __( 'Please enter a prefix for your invoice numbers. If you use your PayPal account for multiple stores ensure this prefix is unique as PayPal will not allow orders with the same invoice number.' ),
							'default' => 'WC-',
							'desc_tip'      => true,
						),
			'metodo_envio_formulario' => array(
							'titulo' => __( 'metodo_envio_formulario' ),
							'type' => 'checkbox',
							'label' => __( 'Usar el método de envío de formulario' ),
							'descripcion' => __( 'Enable this to post order data to PayPal via a form instead of using a redirect/querystring.' ),
							'default' => 'no'
						),
			'estilo_pagina' => array(
							'titulo' => __( 'estilo_pagina' ),
							'label' => __( 'Estilo de página' ),
							'type' => 'text',
							'descripcion' => __( 'Optionally enter the name of the page style you wish to use. These are defined within your PayPal account.' ),
							'default' => '',
							'desc_tip'      => true,
							'placeholder'	=> __( 'Optional' )
						),
			'shipping' => array(
							'titulo' => __( 'Opciones de envío' ),
							'type' => 'titulo',
							'descripcion' => '',
						),
			'enviar_envio' => array(
							'titulo' => __( 'enviar_envio' ),
							'type' => 'checkbox',
							'label' => __( 'Enviar datos de envío a PayPal en lugar de facturación.' ),
							'descripcion' => '',
							'descripcion' => __( 'PayPal allows us to send 1 address. If you are using PayPal for shipping labels you may prefer to send the shipping address rather than billing.' ),
							'default' => 'no'
						),
			'reemplazar_direccion' => array(
							'titulo' => __( 'reemplazar_direccion' ),
							'type' => 'checkbox',
							'label' => __( 'Activar "address override" para evitar que la información de dirección se cambie.' ),
							'descripcion' => __( 'PayPal verifies addresses therefore this setting can cause errors (we recommend keeping it disabled).' ),
							'default' => 'no'
						),
			'recargo' => array(
							'titulo'	  => __('recargo'),
							'label'       => __( 'Recargo (%)' ),
							'type'        => 'text',
							'descripcion' => __( 'Recargo del comercio.' ),
							'default'     => '00',
							'desc_tip'    => true,
						),
			'testing' => array(
							'titulo' => __( 'Modo pruebas' ),
							'type' => 'titulo',
							'descripcion' => '',
						),
			'testmode' => array(
							'titulo' => __( 'testmode' ),
							'type' => 'checkbox',
							'label' => __( 'Activar sandbox de PayPal' ),
							'default' => 'yes',
							'descripcion' => sprintf( __( 'PayPal sandbox can be used to test payments. Sign up for a developer account <a href="%s">here</a>.' ), 'https://developer.paypal.com/' ),
						),
			'debug' => array(
							'titulo' => __( 'debug' ),
							'type' => 'checkbox',
							'label' => __( 'Activar debug' ),
							'default' => 'no',
							'descripcion' => __( 'Log PayPal events, such as IPN requests, inside <code>shop/logs/paypal.txt</code>' ),
						)
			);
    }
	/**
	 * Get PayPal Args for passing to PP
	 *
	 * @access public
	 * @param mixed $order
	 * @return array
	 */
	function get_paypal_args( $order ) {
		global $shop;
		$order_id = $order->id;
		if ( 'yes' == $this->debug )
			$this->log->add( 'paypal', 'Generating payment form for order ' . $order->getid() . '. Notify URL: ' . $this->notificar_url );
			
		if ( in_array( $order->datos['facturacion']['pais'], array( 'US','CA' ) ) ) {
			$order->datos['facturacion']['telefono'] = str_replace( array( '( ', '-', ' ', ' )', '.' ), '', $order->datos['facturacion']['telefono'] );
			$phone_args = array(
				'night_phone_a' => substr( $order->datos['facturacion']['telefono'], 0, 3 ),
				'night_phone_b' => substr( $order->datos['facturacion']['telefono'], 3, 3 ),
				'night_phone_c' => substr( $order->datos['facturacion']['telefono'], 6, 4 ),
				'day_phone_a' 	=> substr( $order->datos['facturacion']['telefono'], 0, 3 ),
				'day_phone_b' 	=> substr( $order->datos['facturacion']['telefono'], 3, 3 ),
				'day_phone_c' 	=> substr( $order->datos['facturacion']['telefono'], 6, 4 )
			);
		} else {
			$phone_args = array(
				'night_phone_b' => $order->datos['facturacion']['telefono'],
				'day_phone_b' 	=> $order->datos['facturacion']['telefono']
			);
		}
		// PayPal Args
		$paypal_args = array_merge(
			array(
				'cmd' 					=> '_cart',
				'business' 				=> $this->email,
				'no_note' 				=> 1,
				'currency_code' 		=> ShopHelper::get_shop_moneda(),
				'charset' 				=> 'UTF-8',
				'rm' 					=> /*is_ssl() ? 2 : */1,
				'upload' 				=> 1,
				'return' 				=> /*add_query_arg( 'utm_nooverride', '1', */$this->get_return_url( $order ) /*)*/,
				'cancel_return'			=> /*$order->get_cancel_order_url()*/'',
				'page_style'			=> $this->estilo_pagina,
				// Order key + ID
				'invoice'				=> $this->prefijo_factura . ltrim( $order->getid(), '#' ),
				'custom' 				=> $order_id,
				// IPN
				'notify_url'			=> $this->notificar_url,
				// Billing Address info
				'first_name'				=> $order->datos['facturacion']['nombre'],
				'last_name'				=> $order->datos['facturacion']['apellidos'],
				'company'				=> $order->datos['facturacion']['empresa'],
				'nif'					=> $order->datos['facturacion']['nif'],
				'address1'				=> $order->datos['facturacion']['direccion'],
				'city'				=> $order->datos['facturacion']['localidad'],
				'state'				=> $order->datos['facturacion']['provincia'],
				'zip'			=> $order->datos['facturacion']['codigo_postal'],
				'country'					=> $order->datos['facturacion']['pais'],
				'email'					=> $order->email
			),
			$phone_args
		);
		// Shipping
		if ( $this->enviar_envio=='yes' ) {
			$paypal_args['reemplazar_direccion'] = ( $this->reemplazar_direccion == 'yes' ) ? 1 : 0;
			$paypal_args['no_shipping'] = 0;
			// If we are sending shipping, send shipping address instead of billing
			$paypal_args['first_name']			= $order->datos['envio']['nombre'];
			$paypal_args['last_name']		= $order->datos['envio']['apellidos'];
			$paypal_args['company']			= $order->datos['envio']['empresa'];
			$paypal_args['nif']				= '';
			$paypal_args['address1']		= $order->datos['envio']['direccion'];
			$paypal_args['city']		= $order->datos['envio']['localidad'];
			$paypal_args['state']		= $order->datos['envio']['provincia'];
			$paypal_args['country']			= $order->datos['envio']['codigo_postal'];
			$paypal_args['zip']	= $order->datos['envio']['pais'];
		} else {
			$paypal_args['no_shipping'] = 1;
		}
		
		// If prices include tax or have order discounts, send the whole order as a single item
		//if ( $shop->config['impuestos']['impuestos_incluidos'] == '1' || $order->datos['total']['descuento_total'] > 0 ) {
			// Discount
			$paypal_args['discount_amount_cart'] = str_replace(',', '.', (string) $order->datos['total']['descuento_total']);
			// Don't pass items - paypal borks tax due to prices including tax. PayPal has no option for tax inclusive pricing sadly. Pass 1 item for the order items overall
			$item_names = array();
			if ( sizeof( $order->lineas ) > 0 )
				foreach ( $order->lineas as $item )
					if ( $item->cantidad ){
						$_pr = unserialize(utf8_decode($item->data));
						$item_names[] = SistemaIdioma::Imprimir($_pr->nombre) . ' x ' . $item->cantidad;
					}
			$paypal_args['item_name_1'] 	= sprintf( __( 'Pedido %s'  ), $order->getid() ) . " - " . implode( ', ', $item_names );
			$paypal_args['quantity_1'] 		= 1;
			$paypal_args['amount_1'] 		= number_format( $order->datos['total']['total'] - $order->datos['total']['envio_total'] - $order->datos['total']['envio_impuesto_total'] + $order->datos['total']['descuento_total'], 2, '.', '' );
			// Shipping Cost
			// No longer using shipping_1 because
			//		a) paypal ignore it if *any* shipping rules are within paypal
			//		b) paypal ignore anything over 5 digits, so 999.99 is the max
			// $paypal_args['shipping_1']		= number_format( $order->get_shipping() + $order->get_shipping_tax() , 2, '.', '' );
			if ( ( $order->datos['total']['envio_total'] + $order->datos['total']['envio_impuesto_total'] ) > 0 ) {
				$paypal_args['item_name_2'] = __( 'Gastos de envío' ) . ': ' . ucwords( SistemaIdioma::Imprimir($order->datos['total']['envio_nombre']) );
				$paypal_args['quantity_2'] 	= '1';
				$paypal_args['amount_2'] 	= number_format( $order->datos['total']['envio_total'] + $order->datos['total']['envio_impuesto_total'] , 2, '.', '' );
			}
			
			// Descuentos aplicados
			/*if ($order->datos['total']['descuento_total'] <> 0){
				$desc_total = substr($order->datos['total']['descuento_total'], 0, 1) == '-' ? number_format(substr($order->datos['total']['descuento_total'], 1), 2, '.', '') : number_format($order->datos['total']['descuento_total'], 2, '.', '');
				
				$paypal_args['item_name_3'] = __( 'Descuento' );
				$paypal_args['quantity_3'] 	= '1';
				$paypal_args['amount_3'] 	= $desc_total;
			}*/
			
		/*} else {
			// Tax
			$paypal_args['tax_cart'] = $order->datos['total']['impuesto_total'];
			// Cart Contents
			$item_loop = 0;
			if ( sizeof( $order->lineas ) > 0 ) {
				foreach ( $order->lineas as $item ) {
					if ( $item->cantidad ) {
						$item_loop++;
						$product = unserialize($item->data);
						$item_name 	= SistemaIdioma::Imprimir($product->nombre);
						if ( $meta = implode(',', Helpers::toArray(json_decode($item->variation))) )
							$item_name .= ' ( ' . $meta . ' )';
						$paypal_args[ 'item_name_' . $item_loop ] 	= $item_name;
						$paypal_args[ 'quantity_' . $item_loop ] 	= $item->cantidad;
						$paypal_args[ 'amount_' . $item_loop ] 		= $item->linea_subtotal;
						if ( $product->getsku() )
							$paypal_args[ 'item_number_' . $item_loop ] = $product->getsku();
					}
				}
			}*/
			// Fees
			/*if ( sizeof( $order->get_fees() ) > 0 ) {
				foreach ( $order->get_fees() as $item ) {
					$item_loop++;
					$paypal_args[ 'item_name_' . $item_loop ] 	= $item['name'];
					$paypal_args[ 'quantity_' . $item_loop ] 	= 1;
					$paypal_args[ 'amount_' . $item_loop ] 		= $item['line_total'];
				}
			}
			// Shipping Cost item - paypal only allows shipping per item, we want to send shipping for the order
			if ( $order->get_shipping() > 0 ) {
				$item_loop++;
				$paypal_args[ 'item_name_' . $item_loop ] 	= __( 'Shipping via' ) . ' ' . ucwords( $order->shipping_titulo_metodo );
				$paypal_args[ 'quantity_' . $item_loop ] 	= '1';
				$paypal_args[ 'amount_' . $item_loop ] 		= number_format( $order->get_shipping(), 2, '.', '' );
			}*/
		//}
		//$paypal_args = apply_filters( 'shop_paypal_args', $paypal_args );
		
		return $paypal_args;
	}
    /**
	 * Generate the paypal button link
     *
     * @access public
     * @param mixed $order_id
     * @return string
     */
    function generar_formulario_paypal( $order ) {
		global $shop;
		//$order = new PedidoShop( $order_id );
		if ( $this->testmode == 'yes' ):
			$paypal_adr = $this->testurl . '?test_ipn=1&';
		else :
			$paypal_adr = $this->liveurl . '?';
		endif;
		$paypal_args = $this->get_paypal_args( $order );
		$paypal_args_array = array();
		foreach ($paypal_args as $key => $value) {
			$paypal_args_array[] = '<input type="hidden" name="'.$key.'" value="'.$value.'" />';
		}
		/*$shop->add_inline_js( '
			jQuery("body").block({
					message: "<img src=\"' . esc_url( apply_filters( 'shop_ajax_loader_url', $shop->plugin_url() . '/assets/img/ajax-loader.gif' ) ) . '\" alt=\"Redirecting&hellip;\" style=\"float:left; margin-right: 10px;\" />'.__( 'Thank you for your order. We are now redirecting you to PayPal to make payment.' ).'",
					overlayCSS:
					{
						background: "#fff",
						opacity: 0.6
					},
					css: {
				        padding:        20,
				        textAlign:      "center",
				        color:          "#555",
				        border:         "3px solid #aaa",
				        backgroundColor:"#fff",
				        cursor:         "wait",
				        lineHeight:		"32px"
				    }
				});
			jQuery("#submit_paypal_payment_form").click();
		' );*/
		return '<form action="'. $paypal_adr .'" method="post" id="paypal_payment_form" target="_top">
				' . implode( '', $paypal_args_array) . '
				<input type="submit" class="button-alt" id="submit_paypal_payment_form" value="'.__( 'Pay via PayPal' ).'" /> <input type="button" class="button cancel" value="'.__( 'Cancel order &amp; restore cart' ).'" />
			</form>';
	}
    /**
     * Process the payment and return the result
     *
     * @access public
     * @param int $order_id
     * @return array
     */
	function procesar_pago( $order_id ) {
		
		$order = new PedidoShop();
		$order->setid($order_id);
		$order = $order->BuscarId();
		
		//$order = new WC_Order( $order_id );
		
		if ( ! $this->metodo_envio_formulario ) {
			$paypal_args = $this->get_paypal_args( $order );
			$paypal_args = http_build_query( $paypal_args, '', '&' );
			if ( $this->testmode == 'yes' ):
				$paypal_adr = $this->testurl . '?test_ipn=1&';
			else :
				$paypal_adr = $this->liveurl . '?';
			endif;
			return array(
				'result' 	=> 'info',
				'redirect'	=> $paypal_adr . $paypal_args
			);
		} else {
			return array(
				'result' 	=> 'info',
				'redirect'	=> Helpers::add_query_arg('pedido', $order->id, Helpers::resolveLink(false,'ext_shop','cesta_pago_form',__('Finalizar_pago')).'/')
			);
		}
	}
    /**
     * Output for the order received page.
     *
     * @access public
     * @return void
     */
	function pagina_recibo( $order ) {
		
		echo '<p>'.__( 'Thank you for your order, please click the button below to pay with PayPal.' ).'</p>';
		echo $this->generar_formulario_paypal( $order );
	}
	/**
	 * Check PayPal IPN validity
	 **/
	function comprobar_solicitud_ipn_valida() {
		global $shop;
		/*if ( 'yes' == $this->debug )
			$this->log->add( 'paypal', 'Checking IPN response is valid...' );*/
    	// Get recieved values from post data
		$received_values = array( 'cmd' => '_notify-validate' );
		$received_values += Helpers::stripslashes_deep( $_POST );
        // Send back post vars to paypal
        $params = array(
        	'body' 			=> $received_values,
        	'sslverify' 	=> false,
        	'timeout' 		=> 60,
        	'user-agent'	=> 'Shop/1.0'
        );
        /*if ( 'yes' == $this->debug )
			$this->log->add( 'paypal', 'IPN Request: ' . print_r( $params, true ) );*/
        // Get url
       	if ( $this->testmode == 'yes' )
			$paypal_adr = $this->testurl;
		else
			$paypal_adr = $this->liveurl;
			
		return true;
		// Post back to get a response
        //$response = $this->_post( $paypal_adr, $params );
	
			/*if ( 'yes' == $this->debug )
				$this->log->add( 'paypal', 'IPN Response: ' . print_r( $response, true ) );*/
	
			// check to see if the request was valid
			if ( /*! is_wp_error( $response ) && */$response['response']['code'] >= 200 && $response['response']['code'] < 300 && ( strcmp( $response['body'], "VERIFIED" ) == 0 ) ) {
				/*if ( 'yes' == $this->debug )
					$this->log->add( 'paypal', 'Received valid response from PayPal' );*/
	
				return true;
			}
        /*if ( 'yes' == $this->debug ) {
        	$this->log->add( 'paypal', 'Received invalid response from PayPal' );
        	if ( is_wp_error( $response ) )
        		$this->log->add( 'paypal', 'Error response: ' . $response->get_error_message() );
        }*/
        return false;
    }
	/**
	 * Check for PayPal IPN Response
	 *
	 * @access public
	 * @return void
	 */
	function comprobar_respuesta_ipn() {
		@ob_clean();
    	if ( ! empty( $_POST ) && $this->comprobar_solicitud_ipn_valida() ) {
    		header( 'HTTP/1.1 200 OK' );
        	//do_action( "valid-paypal-standard-ipn-request", $_POST );
			
			$this->solicitud_exitosa( $_POST );
		} else {
			//wp_die( "PayPal IPN Request Failure" );
			die( "PayPal IPN Request Failure" );
   		}
	}
	/**
	 * Successful Payment!
	 *
	 * @access public
	 * @param array $posted
	 * @return void
	 */
	function solicitud_exitosa( $posted ) {
		global $shop;
		$posted = Helpers::stripslashes_deep( $posted );
		// Custom holds post ID
	    if ( ! empty( $posted['invoice'] ) && ! empty( $posted['custom'] ) ) {
		    $order = $this->get_pedido_paypal( $posted );
		     /*if ( 'yes' == $this->debug )
	        	$this->log->add( 'paypal', 'Found order #' . $order->id );*/
		    // Lowercase returned variables
	        $posted['payment_status'] 	= strtolower( $posted['payment_status'] );
	        $posted['txn_type'] 		= strtolower( $posted['txn_type'] );
			
			$order->datos['metodo_pago']['estado'] = $posted['payment_status'];
	        /*if ( 'yes' == $this->debug )
	        	$this->log->add( 'paypal', 'Payment status: ' . $posted['payment_status'] );*/
	        // We are here so lets check status and do actions
	        switch ( $posted['payment_status'] ) {
				case 'pending' :
					$order->datos['metodo_pago']['razon_estado'] = $posted['pending_reason'];
					
					$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_creado'] );
					$order->Modificar();
					
					$emailer = new Emailer();
				
					$to_array = array($GLOBALS['opciones']['emailer']['email_emailer']=>$GLOBALS['opciones']['emailer']['nombre_emailer']);
					$asunto = sprintf(__('Pedido # %s procesado mediante PayPal pendiente'), $order->id);
					
					$emailer->to = $to_array;
					$emailer->asunto = $asunto;
					
					$emailer->contenido = '<h1>'.$asunto.'</h1>';
					//$emailer->contenido .= print_r($_POST, true);
					//$emailer->contenido .= print_r($metodo_pago->id, true);
					$emailer->contenido .= '<p>' . sprintf(__('El pedido # %s se ha procesado mediante PayPal con el siguiente resultado: %s (%s)'), $order->id, $posted['payment_status'], $posted['pending_reason']) . '</p>';
					
					if ($emailer->enviar()){
					}
					break;
					
	            case 'completed' :
	            	// Check order not already completed
	            	if ( $order->estado == 'completado' ) {
	            		 /*if ( 'yes' == $this->debug )
	            		 	$this->log->add( 'paypal', 'Aborting, Order #' . $order->id . ' is already complete.' );*/
	            		 exit;
	            	}
	            	// Check valid txn_type
	            	$accepted_types = array( 'cart', 'instant', 'express_checkout', 'web_accept', 'masspay', 'send_money' );
					if ( ! in_array( $posted['txn_type'], $accepted_types ) ) {
						/*if ( 'yes' == $this->debug )
							$this->log->add( 'paypal', 'Aborting, Invalid type:' . $posted['txn_type'] );*/
						exit;
					}
					// Validate Amount
				    if ( (float) $order->datos['total']['total'] != (float) $posted['mc_gross'] ) {
				    	/*if ( 'yes' == $this->debug )
				    		$this->log->add( 'paypal', 'Payment error: Amounts do not match (gross ' . $posted['mc_gross'] . ')' );*/
				    	// Put this order en-espera for manual checking
				    	$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_creado'] );
						$order->Modificar();
				    	exit;
				    }
					
				    // Validate Email Address
					if ( strcasecmp( trim( $posted['receiver_email'] ), trim( $this->receptor_email ) ) != 0 ) {
						/*if ( 'yes' == $this->debug )
							$this->log->add( 'paypal', "IPN Response is for another one: {$posted['receptor_email']} our email is {$this->receptor_email}" );*/
						// Put this order en-espera for manual checking
				    	$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_creado'] );
						$order->Modificar();
				    	exit;
					}
					
					$emailer = new Emailer();
				
					$to_array = array($GLOBALS['opciones']['emailer']['email_emailer']=>$GLOBALS['opciones']['emailer']['nombre_emailer']);
					$asunto = sprintf(__('Pedido # %s pagado mediante PayPal'), $order->id);
					
					$emailer->to = $to_array;
					$emailer->asunto = $asunto;
					
					$emailer->contenido = '<h1>'.$asunto.'</h1>';
					//$emailer->contenido .= print_r($_POST, true);
					//$emailer->contenido .= print_r($metodo_pago->id, true);
					$emailer->contenido .= '<p>' . sprintf(__('El pedido # %s se ha pagado mediante PayPal con el siguiente resultado: %s'), $order->id, $posted['payment_status']) . '</p>';
					
					if ($emailer->enviar()){
					}
					 // Store PP Details
	                /*if ( ! empty( $posted['payer_email'] ) )
	                	update_post_meta( $order_id, 'Payer PayPal address', $posted['payer_email'] );
	                if ( ! empty( $posted['txn_id'] ) )
	                	update_post_meta( $order_id, 'Transaction ID', $posted['txn_id'] );
	                if ( ! empty( $posted['first_name'] ) )
	                	update_post_meta( $order_id, 'Payer first name', $posted['first_name'] );
	                if ( ! empty( $posted['last_name'] ) )
	                	update_post_meta( $order_id, 'Payer last name', $posted['last_name'] );
	                if ( ! empty( $posted['payment_type'] ) )
	                	update_post_meta( $order_id, 'Payment type', $posted['payment_type'] );*/
	            	// Payment completed
	                //$order->add_order_note( __( 'IPN payment completed' ) );
	                $order->pago_completo($GLOBALS['opciones']['ext_shop']['venta']['estado_completado']);
					//$order->setestado('completado');
					//$order->Modificar();
	                /*if ( 'yes' == $this->debug )
	                	$this->log->add( 'paypal', 'Payment complete.' );*/
	            break;
	            case 'denied' :
	            case 'expired' :
	            case 'failed' :
	            case 'voided' :
	                // Order failed
	                $order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_fallido'] );
					$order->Modificar();
	            break;
	            case "refunded" :
	            	// Only handle full refunds, not partial
	            	if ( (float) $order->datos['total']['total'] == (float) ( $posted['mc_gross'] * -1 ) ) {
		            	// Mark order as refunded
		            	$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_cancelado'] );
						$order->Modificar();
						
						$emailer = new Emailer();
				
						$to_array = array($GLOBALS['opciones']['emailer']['email_emailer']=>$GLOBALS['opciones']['emailer']['nombre_emailer']);
						$asunto = sprintf(__('Pago del pedido # %s reembolsado/revertido'), $order->id);
						
						$emailer->to = $to_array;
						$emailer->asunto = $asunto;
						
						$emailer->contenido = '<h1>'.$asunto.'</h1>';
						//$emailer->contenido .= print_r($_POST, true);
						//$emailer->contenido .= print_r($metodo_pago->id, true);
						$emailer->contenido .= '<p>' . sprintf(__('El pedido # %s ha sido marcado como reembolsado - PayPal reason code: %s'), $order->id, $posted['reason_code']) . '</p>';
						
						if ($emailer->enviar()){
						}
		            	/*$mailer = $shop->mailer();
		            	$message = $mailer->wrap_message(
		            		__( 'Order refunded/reversed' ),
		            		sprintf( __( 'Order %s has been marked as refunded - PayPal reason code: %s' ), $order->get_order_number(), $posted['reason_code'] )
						);
						$mailer->send( get_option( 'admin_email' ), sprintf( __( 'Payment for order %s refunded/reversed' ), $order->get_order_number() ), $message );*/
					}
	            break;
	            case "reversed" :
	            case "chargeback" :
	            	// Mark order as refunded
	            	$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_cancelado'] );
					$order->Modificar();
					
					$emailer = new Emailer();
				
					$to_array = array($GLOBALS['opciones']['emailer']['email_emailer']=>$GLOBALS['opciones']['emailer']['nombre_emailer']);
					$asunto = sprintf(__('Pago del pedido # %s reembolsado/revertido'), $order->id);
					
					$emailer->to = $to_array;
					$emailer->asunto = $asunto;
					
					$emailer->contenido = '<h1>'.$asunto.'</h1>';
					//$emailer->contenido .= print_r($_POST, true);
					//$emailer->contenido .= print_r($metodo_pago->id, true);
					$emailer->contenido .= '<p>' . sprintf(__('El pedido # %s ha sido marcado como reembolsado - PayPal reason code: %s'), $order->id, $posted['reason_code']) . '</p>';
					
					if ($emailer->enviar()){
					}
	            	/*$mailer = $shop->mailer();
	            	$message = $mailer->wrap_message(
	            		__( 'Order refunded/reversed' ),
	            		sprintf(__( 'Order %s has been marked as refunded - PayPal reason code: %s' ), $order->get_order_number(), $posted['reason_code'] )
					);
					$mailer->send( get_option( 'admin_email' ), sprintf( __( 'Payment for order %s refunded/reversed' ), $order->get_order_number() ), $message );*/
	            break;
	            default :
	            	// No action
	            break;
	        }
			
			exit;
	    }
	}
	/**
	 * get_pedido_paypal function.
	 *
	 * @access public
	 * @param mixed $posted
	 * @return void
	 */
	function get_pedido_paypal( $posted ) {
		$custom = Helpers::maybe_unserialize( $posted['custom'] );
    	// Backwards comp for IPN requests
    	if ( is_numeric( $custom ) ) {
	    	$order_id = (int) $custom;
	    	$order_key = $posted['invoice'];
    	} elseif( is_string( $custom ) ) {
	    	$order_id = (int) str_replace( $this->prefijo_factura, '', $custom );
	    	$order_key = $custom;
    	} else {
    		list( $order_id, $order_key ) = $custom;
		}
		$order = new PedidoShop();
		$order->setid($order_id);
		$order = $order->BuscarId();
		if ( ! isset( $order->id ) ) {
			// We have an invalid $order_id, probably because prefijo_factura has changed
			$order_id 	= shop_get_order_id_by_order_key( $order_key );
			$order = new PedidoShop();
			$order->setid($order_id);
			$order = $order->BuscarId();
		}
		// Validate key
		/*if ( $order->order_key !== $order_key ) {
        	if ( $this->debug=='yes' )
        		$this->log->add( 'paypal', 'Error: Order Key does not match invoice.' );
        	exit;
        }*/
        return $order;
	}
}