<?php
/**
 * PayPal Standard Payment Gateway
 *
 * Provides a PayPal Standard Payment Gateway.
 *
 * @class 		PagoSermepa
 * @extends		MetodoPago
 * @version		1.0.0
 * @package		Shop/Classes/Payment
 * @author 		Mideas
 */
defined('CONF_URL_APP') or die;
if(!function_exists("escribirLog")) {
	require_once('apiRedsys/redsysLibrary.php');
}
if(!class_exists("RedsysAPI")) {
	require_once('apiRedsys/apiRedsysFinal.php');
}
 
class PagoRedsys extends MetodoPago {
	var $notificar_url;
	const merchant_data = 'redsysNotification';
    /**
     * Constructor for the gateway.
     *
     * @access public
     * @return void
     */
	public function __construct() {
		global $shop;
        $this->id           = 'redsys';
        $this->icono         = CONF_URL_APP . 'extensions/ext_shop/assets/img/iconos/redsys.png';
        $this->tiene_campos   = false;
        $this->titulo_metodo = __( 'Tarjeta de crédito [REDSYS]' );
        $this->notificar_url   = Helpers::add_query_arg('payment','redsys',Helpers::resolveLink(false, 'ext_shop', 'cesta_fin', __('Procesando_pago')).'/');
		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();
		
		if (sizeof($this->settings) == 0){
			$this->settings = array(	'titulo' 					=> 	__('Tarjeta de crédito [REDSYS]'),
										'descripcion'				=>	__('Pasarela de pago de TPV Virtual de Redsys.'),
										'entorno'					=>	'',
										'nombre_comercio'			=>	'Nombre de comercio',
										'debug'						=>	'no',
										'fuc'						=>	'',
										'numero_terminal'			=>	'',
										'firma'						=>	'',
										'id_moneda'					=>	'',
										'clave_secreta'				=>	'',
										'tipo_pago'					=>	'',
										'trans'						=>	'',
										'metodo_envio_formulario'	=>	'yes',
										'activo'					=>	'no',
										'orden'						=>	0
									);
		}
		
		// Define user set variables
		$this->titulo 					= $this->settings['titulo'];
		$this->descripcion 				= $this->settings['descripcion'];
		$this->entorno					= $this->settings['entorno'];
		$this->nombre_comercio   		= $this->settings['nombre_comercio'];
		$this->debug					= $this->settings['debug'];
		$this->fuc						= $this->settings['fuc'];
		$this->numero_terminal			= $this->settings['numero_terminal'];
		$this->firma					= $this->settings['firma'];
		$this->id_moneda 				= $this->settings['id_moneda'];
		$this->clave_secreta 			= $this->settings['clave_secreta'];
		$this->tipo_pago				= $this->settings['tipo_pago'];
		$this->trans					= $this->settings['trans'];
		$this->recargo					= $this->settings['recargo'];
		$this->metodo_envio_formulario  = $this->settings['metodo_envio_formulario'] == 'yes' ? true : false;
		$this->activo					= $this->settings['activo'];
		$this->orden					= $this->settings['orden'];
		// Logs
		if ( 'yes' == $this->debug )
			$this->log = $shop->logger();
		if ( !$this->es_valido_para_uso() ) $this->activo = false;
    }
    /**
     * Check if this gateway is activo and available in the user's country
     *
     * @access public
     * @return bool
     */
    function es_valido_para_uso() {
        //if ( ! in_array( get_shop_currency(), apply_filters( 'shop_redsys_supported_currencies', array( 'AUD', 'BRL', 'CAD', 'MXN', 'NZD', 'HKD', 'SGD', 'USD', 'EUR', 'JPY', 'TRY', 'NOK', 'CZK', 'DKK', 'HUF', 'ILS', 'MYR', 'PHP', 'PLN', 'SEK', 'CHF', 'TWD', 'THB', 'GBP', 'RMB' ) ) ) ) return false;
        return true;
    }
	/**
	 * Admin Panel Options
	 * - Options for bits like 'titulo' and availability on a country-by-country basis
	 *
	 * @since 1.0.0
	 */
	public function admin_options() {
		?>
		<h3><?php echo __( 'Tarjeta de crédito [REDSYS]' ); ?></h3>
		<p><?php echo __( 'Pasarela de pago de TPV Virtual de Redsys.'); ?></p>
    	<?php if ( $this->es_valido_para_uso() ) : ?>
			<table class="form-table">
			<?php
    			// Generate the HTML For the settings form.
    			$this->generate_settings_html();
			?>
			</table><!--/.form-table-->
		<?php else : ?>
            <div class="inline error"><p><strong><?php echo __( 'Gateway Disabled' ); ?></strong>: <?php echo __( 'Redsys no soporta la moneda de tu tienda.' ); ?></p></div>
		<?php
			endif;
	}
    /**
     * Initialise Gateway Settings Form Fields
     *
     * @access public
     * @return void
     */
    function init_form_fields() {
    	$this->form_fields = array(
			'activo' => array(
							'titulo' => __( 'activo' ),
							'type' => 'checkbox',
							'label' => __( 'Activar TPV Redsys' ),
							'default' => 'yes'
						),
			'titulo' => array(
							'titulo' => __( 'titulo' ),
							'label' => __( 'Título' ),
							'type' => 'text',
							'descripcion' => __( 'This controls the titulo which the user sees during checkout.' ),
							'desc_tip'      => true,
							'attributes'	=>	array('multilanguage' => 'true')
						),
			'descripcion' => array(
							'titulo' => __( 'descripcion' ),
							'label' => __( 'Descripción' ),
							'type' => 'textarea',
							'descripcion' => __( 'This controls the descripcion which the user sees during checkout.' ),
							'attributes'	=>	array('multilanguage' => 'true')
						),
			'entorno' => array(
							'titulo' => __( 'entorno' ),
							'label' => __( 'Entorno de Redsys' ),
							'type' 			=> 'list',
							'descripcion' => __( 'Entorno del proceso de pago.' ),
							'options'     => array(
								'Sis-d' => __( 'Sis-d' ),
								'Sis-i' => __( 'Sis-i' ),
								'Sis-t' => __( 'Sis-t' ),
								'Sis' 	=> __( 'Sis' )
							)
						),
			'nombre_comercio' => array(
							'titulo' => __( 'nombre_comercio' ),
							'label' => __( 'Nombre del comercio' ),
							'type' 			=> 'text',
							'descripcion' => __( 'Please enter your commerce name.' )
						),
			'fuc' => array(
							'titulo' => __( 'fuc' ),
							'type' => 'text',
							'label' => __( 'Número de comercio' ),
							'descripcion' => __( 'PayPal verifies addresses therefore this setting can cause errors (we recommend keeping it disabled).' )
						),
			'tipo_pago' => array(
							'titulo' => __( 'tipo_pago' ),
							'type' => 'list',
							'label' => __( 'Tipos de pago permitidos' ),
							'descripcion' => __( 'Tipos de pago permitidos.' ),
							'options'     => array(
								' ' => __( 'Todos' ),
								'C' => __( 'Solo tarjeta' ),
								'T' => __( 'Tarjeta y Iupay' )
							)
						),
			'clave_secreta' => array(
							'titulo' => __( 'clave_secreta' ),
							'label' => __( 'Clave secreta de encriptación (SHA-256)' ),
							'type' => 'text',
							'descripcion' => __( 'Clave del comercio' )
						),
			'numero_terminal' => array(
							'titulo' => __( 'numero_terminal' ),
							'type' => 'text',
							'label' => __( 'Número de terminal' ),
							'default' => '001',
							'descripcion' => __( 'Log PayPal events, such as IPN requests, inside <code>shop/logs/redsys.txt</code>' )
						),
			'firma' => array(
							'title'       => __( 'firma' ),
							'type'        => 'list',
							'label'		  => __('Tipo de firma'),
							'description' => __( 'Firma del proceso de pago.' ),
							'default'     => 'ampliada',
							'desc_tip'    => true,
							'options'     => array(
								'ampliada' => __( 'Ampliada' ),
								'completa' => __( 'Completa' )
							)
						),
			'id_moneda' => array(
							'titulo' => __( 'id_moneda' ),
							'type' => 'text',
							'label' => __( 'Identificador de moneda' ),
							'descripcion' => __( 'Enable this to post order data to PayPal via a form instead of using a redirect/querystring.' )
						),
			'trans' => array(
							'titulo' => __( 'trans' ),
							'label' => __( 'Tipo de transacción' ),
							'type' => 'text',
							'descripcion' => __( 'Tipo de transacción del comercio.' ),
							'default' => '0'
						),
			'recargo' => array(
							'title'		  => __('recargo'),
							'label'       => __( 'Recargo (%)' ),
							'type'        => 'text',
							'description' => __( 'Recargo del comercio.' ),
							'default'     => '00',
							'desc_tip'    => true,
						),
			'testing' => array(
									'title' => __( 'Gateway Testing', 'wc_redsys_payment_gateway' ),
									'type' => 'title',
									'description' => '',
								),
			'metodo_envio_formulario' => array(
							'titulo' => __( 'metodo_envio_formulario' ),
							'type' => 'checkbox',
							'label' => __( 'Usar el método de envío de formulario' ),
							'descripcion' => '',
							'default' => 'yes',
							'value' => 'yes'
						),
			'debug' => array(
									'titulo' => __( 'Debug Log', 'wc_redsys_payment_gateway' ),
									'type' => 'checkbox',
									'label' => __( 'Enable logging', 'wc_redsys_payment_gateway' ),
									'value' => 'yes'
								),
			'orden' => array(
							'titulo' => __( 'orden' ),
							'label' => __( 'Orden' ),
							'type' 			=> 'text',
							'descripcion' => '',
							'default' => '0',
							'desc_tip'      => true,
							'placeholder'	=> ''
						)
			);
    }
	function generar_formulario_redsys( $order_id ) {
		$merchantModule = 'mideas_shop_redsys_2.8.3';
					
		//Recuperamos los datos de config.
		$nombre=$this->nombre_comercio;
		$codigo=$this->fuc;
		$terminal=$this->numero_terminal;
		$trans=$this->trans;
		$moneda=$this->id_moneda;
		$clave=$this->clave_secreta;	
		$tipopago=$this->tipo_pago;
		$recargo=$this->recargo;
		$idioma=$this->idioma;
		$firma=$this->firma;
		$entorno=$this->entorno;
		
		//Callback
		$urltienda = $this->notificar_url;		
		
		//Objeto tipo pedido
		$order = new PedidoShop();
		$order->setid($order_id);
		$order = $order->BuscarId();
		
		//Calculo del recargo
		$porcientorecargo = $recargo;
		$porcientorecargo = str_replace (',','.',$porcientorecargo);
		$totalcompra = floatval($order->datos['total']['total']);
		$fee = ($porcientorecargo / 100) * $totalcompra;
		
		//Calculo del precio total del pedido
		$transaction_amount = number_format( (float) ($order->datos['total']['total'] + $fee), 2, '.', '' );
		$transaction_amount = str_replace('.','',$transaction_amount);
		$transaction_amount = floatval($transaction_amount);
	
		// Descripción de los productos
		$products = $order->lineas;
		if (sizeof ( $products ) > 0){
			foreach ($products as $product) {
				$_pr = unserialize(utf8_decode($product->data));
				$productos .= $product->cantidad.'x'.htmlentities(SistemaIdioma::Imprimir($_pr->nombre)).'/';
			}
		}
	
		// El número de pedido es los 8 ultimos digitos del ID del carrito + el tiempo MMSS.
		$numpedido = str_pad($order_id, 8, "0", STR_PAD_LEFT) . date("is");
		
		$urlok = Helpers::add_query_arg('pedido',$order->id,Helpers::resolveLink(false, 'ext_shop', 'cesta_fin', __('Procesando_pago')).'/');
		
		$urlko = Helpers::add_query_arg('pedido_fallido',$order->id,Helpers::resolveLink(false, 'ext_shop', 'cesta_fin', __('Procesando_pago')).'/');
		
		// Obtenemos el valor de la config del idioma 
		if($idioma=="no"){
			$idiomaFinal="0";
		}
		else {
				$idioma_web = substr(SistemaIdioma::getIdiomaSesion(),0,2);
				switch ($idioma_web) {
					case 'es':
					$idiomaFinal='001';
					break;
					case 'en':
					$idiomaFinal='002';
					break;
					case 'ca':
					$idiomaFinal='003';
					break;
					case 'fr':
					$idiomaFinal='004';
					break;
					case 'de':
					$idiomaFinal='005';
					break;
					case 'nl':
					$idiomaFinal='006';
					break;
					case 'it':
					$idiomaFinal='007';
					break;
					case 'sv':
					$idiomaFinal='008';
					break;
					case 'pt':
					$idiomaFinal='009';
					break;
					case 'pl':
					$idiomaFinal='011';
					break;
					case 'gl':
					$idiomaFinal='012';
					break;
					case 'eu':
					$idiomaFinal='013';
					break;
					default:
					$idiomaFinal='002';
			}
		}
		
		// Generamos la firma	
		$miObj = new RedsysAPI;
		$miObj->setParameter("DS_MERCHANT_AMOUNT",$transaction_amount);
		$miObj->setParameter("DS_MERCHANT_ORDER",$numpedido);
		$miObj->setParameter("DS_MERCHANT_MERCHANTCODE",$codigo);
		$miObj->setParameter("DS_MERCHANT_CURRENCY",$moneda);
		$miObj->setParameter("DS_MERCHANT_TRANSACTIONTYPE",$trans);
		$miObj->setParameter("DS_MERCHANT_TERMINAL",$terminal);
		$miObj->setParameter("DS_MERCHANT_MERCHANTURL",$urltienda);
		$miObj->setParameter("DS_MERCHANT_URLOK",$urlok);
		$miObj->setParameter("DS_MERCHANT_URLKO",$urlko);
		$miObj->setParameter("Ds_Merchant_ConsumerLanguage",$idiomaFinal);
		$miObj->setParameter("Ds_Merchant_ProductDescription",$productos);
		$miObj->setParameter("Ds_Merchant_Titular",htmlentities($order->datos['facturacion']['nombre']).' '.htmlentities($order->datos['facturacion']['apellidos']));
		$miObj->setParameter("Ds_Merchant_MerchantData",sha1($urltienda));
		$miObj->setParameter("Ds_Merchant_MerchantName",$nombre);
		$miObj->setParameter("Ds_Merchant_PayMethods",$tipopago);
		$miObj->setParameter("Ds_Merchant_Module",$merchantModule);
		
		//Datos de configuración
		$version = getVersionClave();
		
		//Clave del comercio que se extrae de la configuración del comercio
		// Se generan los parámetros de la petición
		$request = "";
		$paramsBase64 = $miObj->createMerchantParameters();
		$signatureMac = $miObj->createMerchantSignature($this->clave_secreta);
		
		$resys_args = array(
			'Ds_SignatureVersion' => $version,
			'Ds_MerchantParameters' => $paramsBase64,
			'Ds_Signature' => $signatureMac
		  );
		 
		//Se establecen los input del formulario con los datos del pedido y la redirección
		$resys_args_array = array();
        foreach($resys_args as $key => $value){
          $resys_args_array[] = "<input type='hidden' name='$key' value='$value'/>";
        }
		
		//Se establece el entorno del SIS
		if($entorno=="Sis-d"){
			$action="http://sis-d.redsys.es/sis/realizarPago";
			}
		else if($entorno=="Sis-i"){
			$action="https://sis-i.redsys.es:25443/sis/realizarPago";
			}
		else if($entorno=="Sis-t"){
			$action="https://sis-t.redsys.es:25443/sis/realizarPago";	
		}
		else{
			$action="https://sis.redsys.es/sis/realizarPago";
			}	
		
		//Formulario que envía los datos del pedido y la redirección al formulario de acceso al TPV
		$return = '<form action="'.$action.'" method="post" id="redsys_payment_form">
		' . implode('', $resys_args_array) ;
		
		if ($this->metodo_envio_formulario == false){
		
			$return .= '<input type="submit" class="button-alt" id="submit_redsys_payment_form" value="'.__('Pagar con Tarjeta de Crédito').'" /> <a class="button cancel" href="javascript:;">'.__('Cancelar Pedido', 'redsys').'</a>
        ';
		} else {
			$return .= '<script>
			$(document).ready(function(){
				$("form#redsys_payment_form").submit();
			});
			</script>';
		}
		
		$return .= '</form>';
		
		return $return;
    }
	
	function pagina_recibo($order){
		//$this->log->add( 'redsys', 'Acceso a receipt page de REDSYS ');
		echo '<p>'.__('Gracias por su pedido, por favor pulsa el botón para pagar con Tarjeta de Crédito.').'</p>';
		echo $this->generar_formulario_redsys($order->id);
	}
	
	function procesar_pago($order_id){
		$order = new PedidoShop();
		$order->setid($order_id);
		$order = $order->BuscarId();
			
		// Return thankyou redirect
		return array(
			'result' 	=> 'success',
			'redirect'	=> Helpers::add_query_arg('pedido', $order->id, Helpers::resolveLink(false,'ext_shop','cesta_pago_form',__('Finalizar_pago')).'/')
		);
	}
	
	function comprobar_respuesta_ipn() {
		c_include('app');
		header( 'Content-Type:text/html; charset=UTF-8' );
    	if (!empty( $_REQUEST ) ) {
			if (!empty( $_POST ) ) {//URL DE RESP. ONLINE
				/** Recoger datos de respuesta **/
				$version     = $_POST["Ds_SignatureVersion"];
				$datos    = $_POST["Ds_MerchantParameters"];
				$firma_remota    = $_POST["Ds_Signature"];
				
				// Se crea Objeto
				$miObj = new RedsysAPI;
				
				/** Se decodifican los datos enviados y se carga el array de datos **/
				$decodec = $miObj->decodeMerchantParameters($datos);
				/** Clave **/
				$kc = $this->get_option( 'clave_secreta' );
				
				/** Se calcula la firma **/
				$firma_local = $miObj->createMerchantSignatureNotif($kc,$datos);
				
				/** Extraer datos de la notificación **/
				$total     = $miObj->getParameter('Ds_Amount');
				$pedido    = $miObj->getParameter('Ds_Order');
				$codigo    = $miObj->getParameter('Ds_MerchantCode');
				$moneda    = $miObj->getParameter('Ds_Currency');
				$respuesta = $miObj->getParameter('Ds_Response');
				$id_trans = $miObj->getParameter('Ds_AuthorisationCode');
								
				$pedido = substr($pedido,0,8);
				$pedido = intval($pedido);
				
					if ($firma_local == $firma_remota 
									&& checkRespuesta($respuesta)
									&& checkMoneda($moneda)
									&& checkFuc($codigo)
									&& checkPedidoNum($pedido)
									&& checkImporte($total)
					){
						
						// Formatear variables
						$respuesta = intval($respuesta);
						
						if ($respuesta < 101){
								$order = new PedidoShop();
								$order->setid($pedido);
								$order = $order->BuscarId();
								
								$order->datos['metodo_pago']['estado'] = 'pagado';
								
								$emailer = new Emailer();
				
								$to_array = array($GLOBALS['opciones']['emailer']['email_emailer']=>$GLOBALS['opciones']['emailer']['nombre_emailer']);
								$asunto = sprintf(__('Pedido # %s pagado mediante la pasarela de pago RedSys'), $order->id);
								
								$emailer->to = $to_array;
								$emailer->asunto = $asunto;
								
								$emailer->contenido = '<h1>'.$asunto.'</h1>';
								$emailer->contenido .= '<p>' . sprintf(__('El pedido # %s se ha pagado con tarjeta de crédito mediante la pasarela de pago RedSys con el siguiente resultado: Pagado'), $order->id) . '</p>';
								
								if ($emailer->enviar()){
								}
								
								App::setMsg( sprintf(__('Pedido #%s pagado correctamente a través de REDSYS'), $pedido), 'success' );
								
								//$order->pago_completo('procesando');
								//$order->pago_completo($GLOBALS['opciones']['ext_shop']['venta']['estado_completado']);
								$order->pago_completo();
						}
						else {
								$order = new PedidoShop();
								$order->setid($pedido);
								$order = $order->BuscarId();
								//$order->setestado( 'fallido' );
								$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_fallido'] );
								$order->Modificar();
								
								App::setMsg( sprintf(__('Se ha producido un error al intentar pagar el pedido #%s pagado a través de REDSYS'), $pedido), 'error' );
						}
					}// if (firma_local=firma_remota)
					else {
							$order = new PedidoShop();
							$order->setid($pedido);
							$order = $order->BuscarId();
							//$order->setestado( 'fallido' );
							$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_fallido'] );
							$order->Modificar();
							
							App::setMsg( sprintf(__('Se ha producido un error al intentar pagar el pedido #%s pagado a través de REDSYS'), $pedido), 'error' );
							
						}		
			}
			else {//URL OK Y KO
				/** Recoger datos de respuesta **/
				$version     = $_GET["Ds_SignatureVersion"];
				$datos    = $_GET["Ds_MerchantParameters"];
				$firma_remota    = $_GET["Ds_Signature"];
				
				// Se crea Objeto
				$miObj = new RedsysAPI;
				
				/** Se decodifican los datos enviados y se carga el array de datos **/
				$decodec = $miObj->decodeMerchantParameters($datos);
				/** Clave **/
				$kc = $this->get_option( 'clave_secreta' );
				
				/** Se calcula la firma **/
				$firma_local = $miObj->createMerchantSignatureNotif($kc,$datos);
				
				/** Extraer datos de la notificación **/
				$total     = $miObj->getParameter('Ds_Amount');
				$pedido    = $miObj->getParameter('Ds_Order');
				$codigo    = $miObj->getParameter('Ds_MerchantCode');
				$moneda    = $miObj->getParameter('Ds_Currency');
				$respuesta = $miObj->getParameter('Ds_Response');
				$id_trans = $miObj->getParameter('Ds_AuthorisationCode');
				
				$pedido = substr($pedido,0,8);
				$pedido = intval($pedido);
				if ($firma_local == $firma_remota){
					// Formatear variables
					$respuesta = intval($respuesta);
					
					if ($respuesta < 101){
							App::setMsg( sprintf(__('Pedido #%s pagado correctamente a través de REDSYS'), $pedido), 'success' );
					}
					else {
							App::setMsg( sprintf(__('Se ha producido un error al intentar pagar el pedido #%s pagado a través de REDSYS'), $pedido), 'error' );
					}
				}// if (firma_local=firma_remota)
				else {
						App::setMsg( sprintf(__('Se ha producido un error al intentar pagar el pedido #%s pagado a través de REDSYS'), $pedido), 'error' );
				}	
			}	
		} else{
				App::setMsg( '<b>REDSYS</b>: Fallo en el proceso de pago.<br>Su pedido ha sido cancelado.', 'error' );
		}
		
		exit;
								
	}
	/**
	 * get_pedido_redsys function.
	 *
	 * @access public
	 * @param mixed $posted
	 * @return void
	 */
	function get_pedido_redsys( $posted ) {
		$custom = Helpers::maybe_unserialize( $posted['custom'] );
    	// Backwards comp for IPN requests
    	if ( is_numeric( $custom ) ) {
	    	$order_id = (int) $custom;
	    	$order_key = $posted['invoice'];
    	} elseif( is_string( $custom ) ) {
	    	$order_id = (int) str_replace( $this->trans, '', $custom );
	    	$order_key = $custom;
    	} else {
    		list( $order_id, $order_key ) = $custom;
		}
		$order = new PedidoShop();
		$order->setid($order_id);
		$order = $order->BuscarId();
		if ( ! isset( $order->id ) ) {
			// We have an invalid $order_id, probably because trans has changed
			$order_id 	= shop_get_order_id_by_order_key( $order_key );
			$order = new PedidoShop();
			$order->setid($order_id);
			$order = $order->BuscarId();
		}
        return $order;
	}
}