<?php
/**
 * PayPal Standard Payment Gateway
 *
 * Provides a PayPal Standard Payment Gateway.
 *
 * @class 		PagoSermepa
 * @extends		MetodoPago
 * @version		1.0.0
 * @package		Shop/Classes/Payment
 * @author 		Mideas
 */
 defined('CONF_URL_APP') or die;
 
class PagoSermepa extends MetodoPago {
	var $notificar_url;
	const merchant_data = 'sermepaNotification';
    /**
     * Constructor for the gateway.
     *
     * @access public
     * @return void
     */
	public function __construct() {
		global $shop;
        $this->id           = 'sermepa';
        $this->icono         = CONF_URL_APP . 'extensions/ext_shop/assets/img/iconos/sermepa.png';
        $this->tiene_campos   = false;
        $this->liveurl      = 'https://sis.redsys.es/sis/realizarPago';
		$this->testurl      = 'https://sis-t.redsys.es:25443/sis/realizarPago';
        $this->titulo_metodo = __( 'Sermepa' );
        $this->notificar_url   = Helpers::add_query_arg('payment','sermepa',Helpers::resolveLink(false, 'ext_shop', 'cesta_fin', __('Procesando_pago')).'/');
		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();
		
		if (sizeof($this->settings) == 0){
			$this->settings = array(	'titulo' 					=> 	__('Sermepa'),
										'descripcion'				=>	__('Pasarela de pago de TPV Virtual de Sermepa.'),
										'nombre_propietario'		=> '',
										'nombre_comercio'			=>	'Nombre de comercio',
										'testmode'					=>	'yes',
										'debug'						=>	'no',
										'numero_comercio'			=>	'',
										'numero_terminal'			=>	'',
										'id_moneda'					=>	$this->get_sermepa_moneda(),
										'clave_secreta'				=>	'',
										'extended_sha1_algorithm'	=>	'',
										'prefijo_factura'			=>	'SE_',
										'activo'					=>	'no'
									);
		}
		
		// Define user set variables
		$this->titulo 					= $this->settings['titulo'];
		$this->descripcion 				= $this->settings['descripcion'];
		$this->nombre_propietario		= $this->settings['nombre_propietario'];
		$this->nombre_comercio   		= $this->settings['nombre_comercio'];
		$this->testmode					= $this->settings['testmode'];
		$this->debug					= $this->settings['debug'];
		$this->numero_comercio			= $this->settings['numero_comercio'];
		$this->numero_terminal			= $this->settings['numero_terminal'];
		$this->id_moneda 				= $this->get_sermepa_moneda();
		$this->clave_secreta 			= $this->settings['clave_secreta'];
		$this->extended_sha1_algorithm	= $this->settings['extended_sha1_algorithm'];
		$this->prefijo_factura			= $this->settings['prefijo_factura'];
		$this->activo					= $this->settings['activo'];
		// Logs
		if ( 'yes' == $this->debug )
			$this->log = $shop->logger();
		// Actions
		//add_action( 'valid-sermepa-standard-ipn-request', array( $this, 'solicitud_exitosa' ) );
		//add_action( 'shop_receipt_sermepa', array( $this, 'pagina_recibo' ) );
		//add_action( 'shop_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		// Payment listener/API hook
		//add_action( 'shop_api_wc_gateway_sermepa', array( $this, 'comprobar_respuesta_ipn' ) );
		if ( !$this->es_valido_para_uso() ) $this->activo = false;
    }
    /**
     * Check if this gateway is activo and available in the user's country
     *
     * @access public
     * @return bool
     */
    function es_valido_para_uso() {
        if ( ! in_array( ShopHelper::get_shop_moneda(), array( 'USD', 'EUR' ) ) ) return false;
        return true;
    }
	
	private function get_sermepa_moneda($moneda=''){
		if (empty($moneda)){
			$moneda = ShopHelper::get_shop_moneda();
		}
		switch ($moneda){
			case 'EUR':
				return 978;
				break;
			case 'USD':
				return 840;
				break;
			default:
				return false;
				break;
		}
	}
	/**
	 * Admin Panel Options
	 * - Options for bits like 'titulo' and availability on a country-by-country basis
	 *
	 * @since 1.0.0
	 */
	public function admin_options() {
		?>
		<h3><?php echo __( 'Sermepa' ); ?></h3>
		<p><?php echo __( 'Sermepa funciona enviando el usuario a Sermepa para introducir su información de pago.'); ?></p>
    	<?php if ( $this->es_valido_para_uso() ) : ?>
			<table class="form-table">
			<?php
    			// Generate the HTML For the settings form.
    			$this->generate_settings_html();
			?>
			</table><!--/.form-table-->
		<?php else : ?>
            <div class="inline error"><p><strong><?php echo __( 'Gateway Disabled' ); ?></strong>: <?php echo __( 'Sermepa no soporta la moneda de tu tienda.' ); ?></p></div>
		<?php
			endif;
	}
    /**
     * Initialise Gateway Settings Form Fields
     *
     * @access public
     * @return void
     */
    function init_form_fields() {
    	$this->form_fields = array(
			'activo' => array(
							'titulo' => __( 'activo' ),
							'type' => 'checkbox',
							'label' => __( 'Activar TPV Sermepa' ),
							'default' => 'yes'
						),
			'titulo' => array(
							'titulo' => __( 'titulo' ),
							'label' => __( 'Título' ),
							'type' => 'text',
							'descripcion' => __( 'This controls the titulo which the user sees during checkout.' ),
							'desc_tip'      => true,
							'attributes'	=>	array('multilanguage' => 'true')
						),
			'descripcion' => array(
							'titulo' => __( 'descripcion' ),
							'label' => __( 'Descripción' ),
							'type' => 'textarea',
							'descripcion' => __( 'This controls the descripcion which the user sees during checkout.' ),
							'attributes'	=>	array('multilanguage' => 'true')
						),
			'nombre_propietario' => array(
							'titulo' => __( 'nombre_propietario' ),
							'label' => __( 'Nombre de propietario' ),
							'type' 			=> 'text',
							'descripcion' => __( 'Please enter your owner name.' )
						),
			'nombre_comercio' => array(
							'titulo' => __( 'nombre_comercio' ),
							'label' => __( 'Nombre del comercio' ),
							'type' 			=> 'text',
							'descripcion' => __( 'Please enter your commerce name.' )
						),
			'prefijo_factura' => array(
							'titulo' => __( 'prefijo_factura' ),
							'label' => __( 'Prefijo de la factura' ),
							'type' => 'text',
							'descripcion' => __( 'Please enter a prefix for your invoice numbers. If you use your PayPal account for multiple stores ensure this prefix is unique as PayPal will not allow orders with the same invoice number.' )
						),
			'numero_comercio' => array(
							'titulo' => __( 'numero_comercio' ),
							'type' => 'text',
							'label' => __( 'Número de comercio' ),
							'descripcion' => __( 'PayPal verifies addresses therefore this setting can cause errors (we recommend keeping it disabled).' )
						),
			'numero_terminal' => array(
							'titulo' => __( 'numero_terminal' ),
							'type' => 'text',
							'label' => __( 'Número de terminal' ),
							'default' => 'no',
							'descripcion' => __( 'Log PayPal events, such as IPN requests, inside <code>shop/logs/sermepa.txt</code>' )
						),
			/*'id_moneda' => array(
							'titulo' => __( 'id_moneda' ),
							'type' => 'text',
							'label' => __( 'Identificador de moneda' ),
							'descripcion' => __( 'Enable this to post order data to PayPal via a form instead of using a redirect/querystring.' )
						),*/
			'clave_secreta' => array(
							'titulo' => __( 'clave_secreta' ),
							'label' => __( 'Clave secreta' ),
							'type' => 'text',
							'descripcion' => __( 'Optionally enter the name of the page style you wish to use. These are defined within your PayPal account.' )
						),
			'extended_sha1_algorithm' => array(
							'titulo' => __( 'extended_sha1_algorithm' ),
							'type' => 'checkbox',
							'label' => __( 'Activar algoritmo SHA1 extendido' ),
							'descripcion' => __( 'PayPal allows us to send 1 address. If you are using PayPal for shipping labels you may prefer to send the shipping address rather than billing.' )
						),
			'testing' => array(
									'title' => __( 'Gateway Testing', 'wc_sermepa_payment_gateway' ),
									'type' => 'title',
									'description' => '',
								),
			'testmode' => array(
							'titulo' => __( 'testmode' ),
							'type' => 'checkbox',
							'label' => __( 'Activar sandbox de Sermepa' )
						),
			'debug' => array(
									'titulo' => __( 'Debug Log', 'wc_sermepa_payment_gateway' ),
									'type' => 'checkbox',
									'label' => __( 'Enable logging', 'wc_sermepa_payment_gateway' )
								)		
			);
    }
	/**
	 * Get PayPal Args for passing to PP
	 *
	 * @access public
	 * @param mixed $order
	 * @return array
	 */
	function get_sermepa_args( $order ) {
		global $shop;
		
		$order_id = $order->id;
		if ( 'yes' == $this->debug )
			$this->log->add( 'sermepa', 'Generating payment form for order ' . $order->getid() . '. Notify URL: ' . $this->notificar_url );
			
		$importe = (float) $order->datos['total']['total'];
		if ( $this->id_moneda == 978 ) {
			$importe = $importe * 100;	// For Euros, last two digits are decimals
		}
		// Sermepa Args
		$sermepa_args = array(
			'Ds_Merchant_Amount'             => str_replace(',','.',(string) $importe),		// 12 / num
			'Ds_Merchant_Currency'           => (string) $this->id_moneda,					// 4 / num
			'Ds_Merchant_Order'              => str_pad($order_id, 4, '0', STR_PAD_LEFT),	// 12 / num{4}char{8}
			'Ds_Merchant_MerchantCode'       => $this->numero_comercio,						// FUC code 9 / num
			'Ds_Merchant_Terminal'           => $this->numero_terminal,						// 3 / num
			'Ds_Merchant_TransactionType'    => 0,											// Autorización
			'Ds_Merchant_Titular'            => $this->nombre_propietario,					// Nombre y apellidos del titular 
			'Ds_Merchant_MerchantName'       => $this->nombre_comercio,						// Optional, commerce name
			'Ds_Merchant_MerchantURL'        => $this->notificar_url,						// http://docs.woothemes.com/document/wc_api-the-woocommerce-api-callback/
			'Ds_Merchant_MerchantData'       => self::merchant_data,
			'Ds_Merchant_ProductDescription' => __('Online order', 'wc_sermepa_payment_gateway'),
			'Ds_Merchant_ConsumerLanguage'   => 0,											// Undefined
			'Ds_Merchant_UrlOK'              => $this->get_return_url($order),
			'Ds_Merchant_UrlKO'              => /*$order->get_cancel_order_url()*/'',
			'Ds_Merchant_PayMethods'         => 'T',										// T = credit card, R = bank transfer, D = Domiciliacion				
		);
		
		$sermepa_args['Ds_Merchant_MerchantSignature'] = $this->get_sermepa_digest( $sermepa_args );
		
		return $sermepa_args;
	}
	
	function get_sermepa_digest($sermepa_args) {			
		if ( $this->extended_sha1_algorithm == 'yes' ) {
			$string = "{$sermepa_args['Ds_Merchant_Amount']}{$sermepa_args['Ds_Merchant_Order']}{$sermepa_args['Ds_Merchant_MerchantCode']}{$sermepa_args['Ds_Merchant_Currency']}{$sermepa_args['Ds_Merchant_TransactionType']}{$sermepa_args['Ds_Merchant_MerchantURL']}{$this->secret_key}";
			$digest = sha1( $string );
		} else {
			$string = "{$sermepa_args['Ds_Merchant_Amount']}{$sermepa_args['Ds_Merchant_Order']}{$sermepa_args['Ds_Merchant_MerchantCode']}{$sermepa_args['Ds_Merchant_Currency']}{$this->secret_key}";
			$digest = sha1( $string );
		}
		
		if ( 'yes' == $this->debug )
			$this->log->add('sermepa', sprintf( __( 'Digest calculation for %s is %s.' ), $string, $digest ) );
		
		return $digest;
	}
    /**
	 * Generate the sermepa button link
     *
     * @access public
     * @param mixed $order_id
     * @return string
     */
    function generar_formulario_sermepa( $order ) {
		global $shop;
		//$order = new PedidoShop( $order_id );
		if ( $this->testmode == 'yes' ):
			$sermepa_adr = $this->testurl;
		else :
			$sermepa_adr = $this->liveurl;
		endif;
		$sermepa_args = $this->get_sermepa_args( $order );
		$sermepa_args_array = array();
		foreach ($sermepa_args as $key => $value) {
			$sermepa_args_array[] = '<input type="hidden" name="'.$key.'" value="'.$value.'" />';
		}
		/*$shop->add_inline_js( '
			jQuery("body").block({
					message: "<img src=\"' . esc_url( apply_filters( 'shop_ajax_loader_url', $shop->plugin_url() . '/assets/img/ajax-loader.gif' ) ) . '\" alt=\"Redirecting&hellip;\" style=\"float:left; margin-right: 10px;\" />'.__( 'Thank you for your order. We are now redirecting you to PayPal to make payment.' ).'",
					overlayCSS:
					{
						background: "#fff",
						opacity: 0.6
					},
					css: {
				        padding:        20,
				        textAlign:      "center",
				        color:          "#555",
				        border:         "3px solid #aaa",
				        backgroundColor:"#fff",
				        cursor:         "wait",
				        lineHeight:		"32px"
				    }
				});
			jQuery("#submit_sermepa_payment_form").click();
		' );*/
		
		return '<form action="'. $sermepa_adr .'" method="post" id="sermepa_payment_form" target="_top">
						' . implode('', $sermepa_args_array) . '
						<input type="submit" class="btn btn-primary" id="submit_sermepa_payment_form" name="submit_sermepa_payment_form" value="'.__('Pay via Sermepa').'" /> <a class="btn btn-default" href="javascript:;">'.__('Cancel order &amp; restore cart').'</a>
					</form>
					';
	}
    /**
     * Process the payment and return the result
     *
     * @access public
     * @param int $order_id
     * @return array
     */
	function procesar_pago( $order_id ) {
		
		$order = new PedidoShop();
		$order->setid($order_id);
		$order = $order->BuscarId();
		
		//$order = new WC_Order( $order_id );
		
		return array(
					'result' 	=> 'success',
					'redirect'	=> Helpers::add_query_arg('pedido', $order->id, Helpers::resolveLink(false,'ext_shop','cesta_pago_form',__('Finalizar_pago')).'/')
				);
	}
    /**
     * Output for the order received page.
     *
     * @access public
     * @return void
     */
	function pagina_recibo( $order ) {
		
		echo '<p>'.__( 'Thank you for your order, please click the button below to pay with Sermepa.' ).'</p>';
		echo $this->generar_formulario_sermepa( $order );
	}
	/**
	 * Check PayPal IPN validity
	 **/
	function comprobar_solicitud_ipn_valida() {
		global $shop;
		if ( 'yes' == $this->debug )
			$this->log->add( 'sermepa', 'Checking IPN response is valid...' );
    	// Get recieved values from post data
		$received_values += Helpers::stripslashes_deep( $_POST );
		
		$string = "{$received_values['Ds_Amount']}{$received_values['Ds_Order']}{$received_values['Ds_MerchantCode']}{$received_values['Ds_Currency']}{$received_values['Ds_Response']}{$this->secret_key}";
		$digest = sha1( $string );
		
		// check to see if the response is valid
		if ( strcasecmp ( $digest, $received_values['Ds_Signature'] ) == 0 ) {
			if ( 'yes' == $this->debug )
				$this->log->add( 'sermepa', 'Received valid notification from Sermepa' );
			return true;
		} 
		if ( 'yes' == $this->debug )
			$this->log->add( 'sermepa', "Received invalid notification from Sermepa.\nString: {$string}\nDigest: {$digest}\nDs_Signature: {$received_values['Ds_Signature']}" );
		
		return false;
    }
	/**
	 * Check for PayPal IPN Response
	 *
	 * @access public
	 * @return void
	 */
	function comprobar_respuesta_ipn() {
		
		if (isset($_POST['Ds_MerchantData']) && $_POST['Ds_MerchantData'] == self::merchant_data):
		
			@ob_clean();
			$_POST = Helpers::stripslashes_deep($_POST);
				
			if ($this->comprobar_solicitud_ipn_valida()) :
				header('HTTP/1.1 200 OK');
				$this->solicitud_exitosa( $_POST );
			else :
				die("Sermepa notification Failure");
			endif;
		endif;
	}
	/**
	 * Successful Payment!
	 *
	 * @access public
	 * @param array $posted
	 * @return void
	 */
	function solicitud_exitosa( $posted ) {
		global $shop;
		$posted = Helpers::stripslashes_deep( $posted );
		// Custom holds post ID
	    if ( ! empty( $posted['Ds_Order'] ) && ! empty( $posted['Ds_Response'] ) ) {
		    $order_id = (int) $posted['Ds_Order'];
			$order = new PedidoShop();
			$order->setid($order_id);
			$order = $order->BuscarId();
		     if ( $order->id != $order_id ) {
				if ( 'yes' == $this->debug ) $this->log->add( 'sermepa', "Error: received order id {$order_id} does not match {$order->id}." );
				exit;
			}
			
			if ( $this->id_moneda != $posted['Ds_Currency'] ) {
				if ( 'yes' == $this->debug ) $this->log->add( 'sermepa', 'Error: Currency id does not match sent data.' );
				exit;
			}
			
			if ( $posted['Ds_TransactionType'] != 0 ) {
				if ( 'yes' == $this->debug ) $this->log->add( 'sermepa', 'Error: transaction type does not match.' );
				exit;
			}
			if ( 'yes' == $this->debug )
				$this->log->add( 'sermepa', 'Payment status: ' . $posted['Ds_Response'] );
				
			// We are here so lets check status and do actions
			$response = (int) $posted['Ds_Response'];
			if ( $response >= 0 && $response <= 99 ) {	// Completed
				// Check order not already completed
				if ($order->estado == 'completado') :
					 if ( 'yes' == $this->debug ) $this->log->add( 'sermepa', 'Aborting, Order #' . $order_id . ' is already complete.' );
					 exit;
				endif;
				
				
				// Validate Amount
				$order_amount = (float) $order->datos['total']['total']; 
				$received_amount = (float) $posted['Ds_Amount'];
				if ( $this->id_moneda == 978 ) {
					$received_amount = $received_amount / 100;	// For Euros, sermepa assumes that last two digits are decimals
				}
											
				if ( $order_amount != $received_amount ) {
					
					if ( $this->debug == 'yes' ) 
						$this->log->add( 'sermepa', "Payment error: Order's ammount {$order_amount} do not match received amount {$received_amount}" );
					
					$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_creado'] );
					$order->Modificar();
					
					exit;
				}
				
				$emailer = new Emailer();
				
				$to_array = array($GLOBALS['opciones']['emailer']['email_emailer']=>$GLOBALS['opciones']['emailer']['nombre_emailer']);
				$asunto = sprintf(__('Pedido # %s pagado mediante Sermepa'), $order->id);
				
				$emailer->to = $to_array;
				$emailer->asunto = $asunto;
				
				$emailer->contenido = '<h1>'.$asunto.'</h1>';
				//$emailer->contenido .= print_r($_POST, true);
				//$emailer->contenido .= print_r($metodo_pago->id, true);
				$emailer->contenido .= '<p>' . sprintf(__('El pedido # %s se ha pagado mediante Sermepa con el siguiente resultado: %s'), $order->id, $posted['payment_status']) . '</p>';
				
				if ($emailer->enviar()){
				}
	
				 // Store payment Details
				/*if ( ! empty( $posted['Ds_Date'] ) )
					update_post_meta( $order_id, 'Payment date', $posted['Ds_Date'] );
				if ( ! empty( $posted['Ds_Hour'] ) )
					update_post_meta( $order_id, 'Payment hour', $posted['Ds_Hour'] );	
				if ( ! empty( $posted['Ds_AuthorisationCode'] ) )
					update_post_meta( $order_id, 'Authorisation code', $posted['Ds_AuthorisationCode'] );
				if ( ! empty( $posted['Ds_Card_Country'] ) )
					update_post_meta( $order_id, 'Card country', $posted['Ds_Card_Country'] );
				if ( ! empty( $posted['last_name'] ) )
					update_post_meta( $order_id, 'Consumer language', $posted['Ds_ConsumerLanguage'] );
				if ( ! empty( $posted['Ds_Card_Type'] ) )
					update_post_meta( $order_id, 'Card type', $posted['Ds_Card_Type'] == 'C' ? 'Credit' : 'Debit' );*/
	
				// Payment completed
				//$order->add_order_note( __('Sermepa payment completed', 'wc_sermepa_payment_gateway') );
				$order->pago_completo();
	
				if ( 'yes' == $this->debug )
					$this->log->add( 'sermepa', 'Payment complete.' );
		
			} else if ( $response == 900 ) {
				
				
			} else if ( $response >= 101 && $response <= 202 ) {
				// Order failed
				$message = sprintf( __('Payment error: code: %s.'), $response);
				$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_fallido'] );
				$order->Modificar();
				if ( $this->debug == 'yes' ) 
					$this->log->add( 'sermepa', "{$message}" );		        	
			} else if ( $response == 912 || $response == 9912 ) {
				// Order failed
				$message = sprintf( __('Payment error: bank unavailable.' ) );
				$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_fallido'] );
				$order->Modificar();
				if ( $this->debug == 'yes' ) 
					$this->log->add( 'sermepa', "{$message}" );
			} else {
				// Order failed
				$message = sprintf( __('Payment error: code: %s.', 'wc_sermepa_payment_gateway'), $response );
				$order->setestado( $GLOBALS['opciones']['ext_shop']['venta']['estado_fallido'] );
				$order->Modificar();
				if ( $this->debug == 'yes' ) 
					$this->log->add( 'sermepa', "{$message}" );
			}
			
			exit;
	    }
	}
	/**
	 * get_pedido_sermepa function.
	 *
	 * @access public
	 * @param mixed $posted
	 * @return void
	 */
	function get_pedido_sermepa( $posted ) {
		$custom = Helpers::maybe_unserialize( $posted['custom'] );
    	// Backwards comp for IPN requests
    	if ( is_numeric( $custom ) ) {
	    	$order_id = (int) $custom;
	    	$order_key = $posted['invoice'];
    	} elseif( is_string( $custom ) ) {
	    	$order_id = (int) str_replace( $this->prefijo_factura, '', $custom );
	    	$order_key = $custom;
    	} else {
    		list( $order_id, $order_key ) = $custom;
		}
		$order = new PedidoShop();
		$order->setid($order_id);
		$order = $order->BuscarId();
		if ( ! isset( $order->id ) ) {
			// We have an invalid $order_id, probably because prefijo_factura has changed
			$order_id 	= shop_get_order_id_by_order_key( $order_key );
			$order = new PedidoShop();
			$order->setid($order_id);
			$order = $order->BuscarId();
		}
		// Validate key
		/*if ( $order->order_key !== $order_key ) {
        	if ( $this->numero_terminal=='yes' )
        		$this->log->add( 'sermepa', 'Error: Order Key does not match invoice.' );
        	exit;
        }*/
        return $order;
	}
}