<?php
/**
 * BBVA Standard Payment Gateway
 *
 * Provides a PayPal Standard Payment Gateway.
 *
 * @class 		WC_Paypal
 * @extends		PagoPaypal
 * @version		2.0.0
 * @package		Shop/Classes/Payment
 * @author 		WooThemes
 */
 defined('CONF_URL_APP') or die;
 
class PagoTransferencia extends MetodoPago {
	var $notificar_url;
    /**
     * Constructor for the gateway.
     *
     * @access public
     * @return void
     */
	public function __construct() {
		global $shop;
        $this->id           = 'transferencia';
        $this->icono         = CONF_URL_APP . 'extensions/ext_shop/assets/img/iconos/transferencia.png';
        $this->tiene_campos   = false;
        $this->liveurl      = 'https://w3.grupotransferencia.com/TLPV/tlpv/TLPV_pub_RecepOpModeloServidor';
		$this->testurl      = 'https://w3.grupotransferencia.com/TLPV/tlpv/TLPV_pub_RecepOpModeloServidor';
        $this->titulo_metodo = __( 'Transferencia bancaria' );
        $this->notificar_url   = Helpers::add_query_arg('payment','transferencia',Helpers::resolveLink(false, 'ext_shop', 'cesta_fin', __('Procesando_pago')).'/') . '&';
		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();
		
		if (sizeof($this->settings) == 0){
			$this->settings = array(	'titulo' 			=> 	__('Transferencia bancaria'),
										'descripcion'		=>	__('Realice su pago directamente en nuestra cuenta bancaria. Por favor use su identificador de pedido como referencia de pago. Su pedido no será enviado hasta que los fondos hayan sido recibidos en nuestra cuenta.'),
										'instrucciones'	=> __('Realice su pago directamente en nuestra cuenta bancaria. Por favor use su identificador de pedido como referencia de pago. Su pedido no será enviado hasta que los fondos hayan sido recibidos en nuestra cuenta.'),
										'nombre_cuenta'			=>	'',
										'banco_cuenta'			=>	'',
										'codigo_entidad_cuenta'	=>	'',
										'codigo_oficina_cuenta'	=>	'',
										'digito_control_cuenta'	=>	'',
										'numero_cuenta'			=>	'',
										'prefijo_factura'		=>	'TB_',
										'activo'				=>	'0'
									);
		}
		
		// Define user set variables
		$this->titulo 			= $this->settings['titulo'];
		$this->descripcion 		= $this->settings['descripcion'];
		$this->instrucciones 	= $this->settings['instrucciones'];
		$this->nombre_cuenta  		= $this->settings['nombre_cuenta'];
		$this->banco_cuenta  		= $this->settings['banco_cuenta'];
		$this->codigo_entidad_cuenta	= $this->settings['codigo_entidad_cuenta'];
		$this->codigo_oficina_cuenta		= $this->settings['codigo_oficina_cuenta'];
		$this->digito_control_cuenta			= $this->settings['digito_control_cuenta'];
		$this->numero_cuenta = $this->settings['numero_cuenta'];
		$this->prefijo_factura	= $this->settings['prefijo_factura'];
		$this->activo	= $this->settings['activo'];
		if ( !$this->es_valido_para_uso() ) $this->activo = false;
    }
    /**
     * Check if this gateway is activo and available in the user's country
     *
     * @access public
     * @return bool
     */
    function es_valido_para_uso() {
        //if ( ! in_array( get_shop_currency(), apply_filters( 'shop_transferenciasupported_currencies', array( 'AUD', 'BRL', 'CAD', 'MXN', 'NZD', 'HKD', 'SGD', 'USD', 'EUR', 'JPY', 'TRY', 'NOK', 'CZK', 'DKK', 'HUF', 'ILS', 'MYR', 'PHP', 'PLN', 'SEK', 'CHF', 'TWD', 'THB', 'GBP', 'RMB' ) ) ) ) return false;
        return true;
    }
	/**
	 * Admin Panel Options
	 * - Options for bits like 'titulo' and availability on a country-by-country basis
	 *
	 * @since 1.0.0
	 */
	public function admin_options() {
		?>
		<h3><?php echo __( 'Transferencia bancaria' ); ?></h3>
		<p><?php echo __('Realice su pago directamente en nuestra cuenta bancaria. Por favor use su identificador de pedido como referencia de pago. Su pedido no será enviado hasta que los fondos hayan sido recibidos en nuestra cuenta.'); ?></p>
    	<?php if ( $this->es_valido_para_uso() ) : ?>
			<table class="form-table">
			<?php
    			// Generate the HTML For the settings form.
    			$this->generate_settings_html();
			?>
			</table><!--/.form-table-->
		<?php else : ?>
            <div class="inline error"><p><strong><?php echo __( 'Gateway Disabled' ); ?></strong>: <?php echo __( 'Transferencia bancaria does not support your store currency.' ); ?></p></div>
		<?php
			endif;
	}
    /**
     * Initialise Gateway Settings Form Fields
     *
     * @access public
     * @return void
     */
    function init_form_fields() {
    	$this->form_fields = array(
			'activo' => array(
							'titulo' => __( 'activo' ),
							'type' => 'checkbox',
							'label' => __( 'Activar Transferencia bancaria' ),
							'default' => '1'
						),
			'titulo' => array(
							'titulo' => __( 'titulo' ),
							'label' => __( 'Título' ),
							'type' => 'text',
							'descripcion' => '',
							'default' => __( 'Transferencia bancaria' ),
							'desc_tip'      => true,
							'attributes'	=>	array('multilanguage' => 'true')
						),
			'descripcion' => array(
							'titulo' => __( 'descripcion' ),
							'label' => __( 'Descripción' ),
							'type' => 'textarea',
							'descripcion' => '',
							'default' => __('Realice su pago directamente en nuestra cuenta bancaria. Por favor use su identificador de pedido como referencia de pago. Su pedido no será enviado hasta que los fondos hayan sido recibidos en nuestra cuenta.'),
							'attributes'	=>	array('multilanguage' => 'true')
						),
			'instrucciones' => array(
							'titulo' => __( 'instrucciones' ),
							'label' => __( 'Instrucciones' ),
							'type' 			=> 'textarea',
							'descripcion' => '',
							'default' => __('Realice su pago directamente en nuestra cuenta bancaria. Por favor use su identificador de pedido como referencia de pago. Su pedido no será enviado hasta que los fondos hayan sido recibidos en nuestra cuenta.'),
							'attributes'	=>	array('multilanguage' => 'true')
						),
			'nombre_cuenta' => array(
							'titulo' => __( 'nombre_cuenta' ),
							'label' => __( 'Nombre del titular de la cuenta' ),
							'type' 			=> 'text',
							'descripcion' => '',
							'default' => '',
							'desc_tip'      => true,
							'placeholder'	=> ''
						),
			'banco_cuenta' => array(
							'titulo' => __( 'banco_cuenta' ),
							'label' => __( 'Entidad bancaria de la cuenta' ),
							'type' 			=> 'text',
							'descripcion' => '',
							'default' => '',
							'desc_tip'      => true,
							'placeholder'	=> ''
						),
			'codigo_entidad_cuenta' => array(
							'titulo' => __( 'codigo_entidad_cuenta' ),
							'label' => __( 'Código de la entidad' ),
							'type' => 'text',
							'descripcion' => '',
							'default' => '',
							'desc_tip'      => true,
						),
			'codigo_oficina_cuenta' => array(
							'titulo' => __( 'codigo_oficina_cuenta' ),
							'label' => __( 'Código de la oficina' ),
							'type' => 'text',
							'descripcion' => '',
							'default' => '',
							'desc_tip'      => true,
						),
			'digito_control_cuenta' => array(
							'titulo' => __( 'digito_control_cuenta' ),
							'label' => __( 'Dígito de control' ),
							'type' => 'text',
							'descripcion' => '',
							'default' => '',
							'desc_tip'      => true,
						),
			'numero_cuenta' => array(
							'titulo' => __( 'numero_cuenta' ),
							'label' => __( 'Número de cuenta' ),
							'type' => 'text',
							'descripcion' => '',
							'default' => '',
							'desc_tip'      => true,
						)
			);
    }
    /**
	 * Generate the paypal button link
     *
     * @access public
     * @param mixed $order_id
     * @return string
     */
    function generar_formulario_transferencia( $order ) {
		global $shop;
		//$order = new PedidoShop( $order_id );
		
		$transferenciaadr = $this->liveurl;
		/*$transferenciaargs = $this->get_transferenciaargs( $order );
		$transferenciaargs_array = array();
		foreach ($transferenciaargs as $key => $value) {
			$transferenciaargs_array[] = '<input type="hidden" name="'.$key.'" value="'.$value.'" />';
		}
		$shop->add_inline_js( '
			jQuery("body").block({
					message: "<img src=\"' . esc_url( apply_filters( 'shop_ajax_loader_url', $shop->plugin_url() . '/assets/img/ajax-loader.gif' ) ) . '\" alt=\"Redirecting&hellip;\" style=\"float:left; margin-right: 10px;\" />'.__( 'Thank you for your order. We are now redirecting you to PayPal to make payment.' ).'",
					overlayCSS:
					{
						background: "#fff",
						opacity: 0.6
					},
					css: {
				        padding:        20,
				        textAlign:      "center",
				        color:          "#555",
				        border:         "3px solid #aaa",
				        backgroundColor:"#fff",
				        cursor:         "wait",
				        lineHeight:		"32px"
				    }
				});
			jQuery("#submit_transferenciapayment_form").click();
		' );*/
		return '<form action="'. $transferenciaadr .'" method="post" id="transferenciapayment_form" target="popup" class="popupform">
				' . implode( '', $transferenciaargs_array) . '
				<input type="submit" class="btn btn-large btn-inverse" id="submit_transferenciapayment_form" value="'.__( 'Pago mediante transferencia bancaria' ).'" /> <input type="button" class="btn  btn-large btn-inverse cancel" value="'.__( 'Cancelar pedido &amp; vaciar el carrito' ).'" />
			</form>';
	}
    /**
     * Process the payment and return the result
     *
     * @access public
     * @param int $order_id
     * @return array
     */
	function procesar_pago( $order_id ) {
		
		$order = new PedidoShop();
		$order->setid($order_id);
		$order = $order->BuscarId();
		
		/*if ( ! $this->numero_cuenta ) {
			$transferenciaargs = $this->get_transferenciaargs( $order );
			$transferenciaargs = http_build_query( $transferenciaargs, '', '&' );
			$transferenciaadr = $this->liveurl . '?';
			return array(
				'result' 	=> 'info',
				'redirect'	=> $transferenciaadr . $transferenciaargs
			);
		} else {*/
			return array(
				'result' 	=> 'info',
				'redirect'	=> Helpers::add_query_arg('pedido', $order->id, $this->get_return_url( $order ))
			);
		//}
	}
    /**
     * Output for the order received page.
     *
     * @access public
     * @return void
     */
	function pagina_recibo( $order ) {
		
		echo '<p>'.__( 'Gracias por su pedido, haz clic en el botón de abajo para confirmar tu pedido.' ).'</p>';
		echo $this->generar_formulario_transferencia( $order );
	}
	
}