<?php
defined('CONF_URL_APP') or die;
class ReglaPrecio extends EntidadModel
{
	protected $dbt_const = "DBT_SHOP_REGLAS_PRECIOS";
	protected $dbt = "shop_reglas_precios";
	
	protected $ext_folder = "ext_shop";
	
	protected $all_vars = array("id","titulo","contenido","condiciones","accion_regla","fecha_inicio","fecha_fin","estado","fecha","params","usado","limite_usos");
	
	protected $idx_vars = array("id");
	protected $update_vars = array("titulo","contenido","condiciones","accion_regla","fecha_inicio","fecha_fin","estado","params","usado","limite_usos");
	
	protected $campos_multilang = array("titulo","contenido");
	protected $campos_array = array("params");
	
	public function __construct() {
		parent::__construct();
	}
	public static function getConds(){
		$dir_condiciones = dirname(__FILE__) . '/condiciones_reglas/';
		$dir_list = Helpers::dir_list($dir_condiciones, '','class.php');
		
		$arr_dir = $dir_list[substr($dir_condiciones,0,-1)];
		
		$results = array();
		
		if (is_array($arr_dir) && sizeof($arr_dir) > 0){
			
			foreach ($arr_dir as $file){
				$filename = substr($file, 0, -10);
				
				$class_nombre = ucfirst($filename) . "ReglaPrecioCondicion";
				
				if (!is_file($dir_condiciones . $filename .'.class.php')){
					continue;
				}
				
				if (!class_exists($class_nombre)){
					include($dir_condiciones . $filename .'.class.php');
				}
				
				if (!class_exists($class_nombre)){
					continue;
				}
				
				$tmp_clase = new $class_nombre();
				
				$info_cond = $tmp_clase->getInfo();
				
				if (is_array($info_cond) && sizeof($info_cond) == 3){
					$results[$info_cond['grupo']][$info_cond['id']] = array("value"=>$info_cond["grupo"].'.'.$info_cond['id'],"label"=>$info_cond["nombre"]);
				} else {
					continue;
				}
			}
			
			return $results;
		} else {
			return false;
		}
		
	}
	
	function cumpleCondicion($excluir=array()){
		$val1 = json_decode($this->condiciones);
		
		if (is_array($val1) && sizeof($val1) > 0){
			
			foreach ($val1 as $kval=>$val){
				$val1[$kval] = Helpers::toArray($val);
			}
			
			foreach ($val1 as $val_parsed){
				
				if (!in_array($val_parsed['campo_condiciones'], $excluir)){
				
					$cond_parsed = explode(".",$val_parsed["campo_condiciones"]);
	
					$grupo_cond = $cond_parsed[0];
					$id_cond = $cond_parsed[1];
					
					$dir_condiciones = dirname(__FILE__) . '/condiciones_reglas/';
					
					if (!empty($grupo_cond) && !empty($id_cond)){
						$filename = $grupo_cond.ucfirst($id_cond).'.class.php';
						
						$class_name = ucfirst($grupo_cond).ucfirst($id_cond)."ReglaPrecioCondicion";
						
						if (!is_file($dir_condiciones . $filename)){
							return false;
						}
						
						if (!class_exists($class_name)){
							include($dir_condiciones . $filename);
						}
						
						if (!class_exists($class_name)){
							return false;
						}
						
						$tmp_clase = new $class_name($val_parsed["campo_operacion"],$val_parsed["valor_condiciones"]);
						
						if (!$tmp_clase->get_resultado())
							return false;
						
					}
				}
			}
			
			return true;
			
		}
		
		return false;
	}
	
	function esCupon($cumplir_resto_condiciones=false){
		$cumple_resto = $cumplir_resto_condiciones == true ? $this->cumpleCondicion(array("cliente.cuponDescuento")) : true;
		
		if ($cumple_resto !== true)
			return __('No se cumplen todas las condiciones para usar ese cupón');
		
		$val_parsed = json_decode($this->condiciones);
		
		if (is_array($val_parsed) && sizeof($val_parsed) > 0 && is_object($val_parsed[0]) && $cumple_resto){
			
			foreach ($val_parsed as $cond){
			
				$cond_parsed = explode(".",$cond->campo_condiciones);
				
				$grupo_cond = $cond_parsed[0];
				$id_cond = $cond_parsed[1];
				
				if ($grupo_cond == 'cliente' && $id_cond == 'cuponDescuento'){
					return true;
				}
			
			}
			
			return false;
			
		} else {
			return false;
		}
	}
	
	function esCuponAplicable($cumplir_resto_condiciones=false){
		$es_cupon = $this->esCupon($cumplir_resto_condiciones);
		
		if ($es_cupon !== true)
			return $es_cupon;
			
		if ((int) $this->usado >= (int) $this->limite_usos)
			return __('Ese cupón no se puede usar más veces');
			
		return true;
	}
	
	function aplicarCuponSesion($codigo){
		if (empty($codigo))
			return false;
			
		if(sizeof(Helpers::getSesion()->get('shop.cupones_aplicados')) > 0)
			return __('No se puede aplicar más de 1 Cupón de descuento');	
			
		$es_aplicable = $this->esCuponAplicable(true);
			
		$val_parsed = (array) json_decode($this->condiciones);
		
		if (is_array($val_parsed) && sizeof($val_parsed) > 0 && is_object($val_parsed[0])){
			foreach ($val_parsed as $cond){
				if ($cond->campo_condiciones == 'cliente.cuponDescuento'){
					if ($cond->valor_condiciones == $codigo){
						if (!array_key_exists($cond->valor_condiciones, Helpers::getSesion()->get('shop.cupones_aplicados'))){
							
							if ($es_aplicable !== true)
								return $es_aplicable;
								
							$cupones_sess = Helpers::getSesion()->get('shop.cupones_aplicados');
							$cupones_sess[$cond->valor_condiciones] = Helpers::toArray($this);
							
							Helpers::getSesion()->set('shop.cupones_aplicados', $cupones_sess);
							return true;
						} else {
							return __('El cupón ya estaba aplicado. No se ha realizado ninguna acción.');
						}
					}
				}
			}
		}
		
		return false;
	}
	
	function quitarCuponSesion($codigo){
		if (empty($codigo))
			return false;
			
		if(is_array(Helpers::getSesion()->get('shop.cupones_aplicados')) && sizeof(Helpers::getSesion()->get('shop.cupones_aplicados')) > 0){
			$cupones_sess = Helpers::getSesion()->get('shop.cupones_aplicados');
			foreach ($cupones_sess as $kcup=>$cup){
				if ($cup['id'] == $codigo){
					unset($cupones_sess[$kcup]);
					Helpers::getSesion()->set('shop.cupones_aplicados', $cupones_sess);
					return true;
				}
			}
		} else {
			return false;
		}
	}
	
	public static function checkAll($user_id){
		if (empty($user_id))
			return false;
			
		$logros = new self();
		
		$logros_c = $logros->ListarCount('estado=1','orden','ASC');
		
		if ($logros_c > 0){
			$logros = $logros->Listar('estado=1','orden','ASC');
			
			$c_news = 0;
			foreach ($logros as $logro){
				$cu = $logro->checkUsuario($user_id);
				if ($cu == 'new' && is_string($cu)){
					$c_news++;
				}
			}
			
			c_include('app');
			if ($c_news > 0){
				if ($c_news == 1){
					App::setMsg(__('¡Has conseguido 1 nuevo trofeo!'), 'success');
				} else {
					App::setMsg(sprintf(__('¡Has conseguido %s nuevos trofeos!'), $c_news), 'success');
				}
			}
		}
	}
	
	function checkUsuario($user_id){
		if (empty($user_id) || empty($this->id))
			return false;
			
		c_include('usuario');
		$usr = new Usuario();
		$usr->setid($user_id);
		if ($usr = $usr->BuscarId()){
		} else {
			return false;
		}
		
		$conds = json_decode($this->condiciones);
		if (!is_array($conds) || sizeof($conds) != 3){
			return false;
		}
		
		$opts_campo = explode('.', $conds[0]);
		$campo = '';
		
		switch (count($opts_campo)){
			case 1:
				break;
			case 2:
				$campo = $this->getOpcionesCampo($opts_campo);
				break;
			default:
				break;
		}
		
		if (empty($campo))
			return false;
			
		$resultado_operacion = 0;
		switch ($campo){
			case 'listas_privadas':
			case 'listas':
				$resultado_operacion = 0;
				if ($ext_favoritos = Helpers::checkExtension('ext_favoritos')){
					e_include('ext_favoritos/class/favoritoListaDinamica');
					if ($campo == 'listas_privadas'){
						$total_listas_usr = FavoritoListaDinamica::getCountListaFavoritosUsuario($usr->id,'privada');
					} else {
						$total_listas_usr = FavoritoListaDinamica::getCountListaFavoritosUsuario($usr->id,'publica');
					}
				}
				break;
			case 'comentarios':
				$resultado_operacion = $usr->get_comentarios_count();
				break;
			case 'media':
				$resultado_operacion = $usr->get_comentarios_media_count();
				break;
			case 'seguidores':
				$resultado_operacion = $usr->get_seguidores_count();
				break;
			default:
				$resultado_operacion = $usr->$campo;
				break;
		}
			
		$conseguido = false;
		switch ($conds[1]){
			case '>=':
				if ($resultado_operacion >= $conds[2]){
					$conseguido = true;
				} else {
					$conseguido = false;
				}
				break;
			case '<=':
				if ($resultado_operacion <= $conds[2]){
					$conseguido = true;
				} else {
					$conseguido = false;
				}
				break;
			case '>':
				if ($resultado_operacion > $conds[2]){
					$conseguido = true;
				} else {
					$conseguido = false;
				}
				break;
			case '<':
				if ($resultado_operacion < $conds[2]){
					$conseguido = true;
				} else {
					$conseguido = false;
				}
				break;
			case '==':
			case '=':
				if ($resultado_operacion == $conds[2]){
					$conseguido = true;
				} else {
					$conseguido = false;
				}
				break;
		}
		
		e_include('ext_usuarios/class/usuarioLogroUsuario');
		
		$ch = ReglaPrecioUsuario::checkLogroUsuario($usr->id, $this->id);
		
		if ($conseguido == true){
			if ($ch == false){
				$ulu = new ReglaPrecioUsuario();
				$ulu->setusuario_id($usr->id);
				$ulu->setlogro_id($this->id);
				
				if ($ulu->Guardar()){
					return 'new';
				} else {
					return false;
				}
			} else {
				return true;
			}
		} else {
			if ($ch == true){
				$ulu = new ReglaPrecioUsuario();
				$ulu->setusuario_id($usr->id);
				$ulu->setlogro_id($this->id);
				
				if ($ulu->Borrar()){
					return false;
				} else {
					return false;
				}
			} else {
				return false;
			}
		}
	}
	
	private function getOpcionesCampo($opts=array()){
		if (empty($opts) || !is_array($opts)){
			return false;
		}
		
		switch ($opts[0]){
			case 'usuario':
			default:
				if (isset($opts[1])){
					return $opts[1];
				} else {
					return '';
				}
				break;
		}
	}
}
?>