<?php
/**
 * Modelo del factura, permite manejar los facturas de forma fácil
 *
 * @package		Mideas.Muzzim
 * @subpackage	Model
 * @since		1.0
 */
class FacturaModel {
	/**
     * ID del factura
     *
     * @var	integer
     * @access  public
     */
	var $id;
	/**
     * Título del factura
     *
     * @var	string
     * @access  public
     */
	var $creador_id;
	/**
     * Parámetros del factura
     *
     * @var	array
     * @access  public
     */
	var $parametros;
	/**
     * Parámetros de la vista de factura
     *
     * @var	array
     * @access  public
     */
	var $vista;
	/**
     * Objeto del factura
     *
     * @var	object
     * @access  public
     */
	var $objeto;
	/**
     * Constructor de la clase
     *
     * @param	string	$id ID del factura
     */
	function __construct($id=''){
		//Incluimos los archivos necesarios
		self::includes();
		if (!empty($_GET['id'])){
			$id = $_GET['id'];
		}
		//Si se establece un ID, se intenta localizar el factura
		if (!empty($id)){
			if (!$this->get_factura($id)){
				Helpers::redirect(CONF_URL_APP . SistemaIdioma::getIdiomaSesion());
				exit;
			}
		}
	}
	public function getTitulo(){
		
		return sprintf(__('Factura #%s/%s'), $this->objeto->codigo, $this->objeto->ejercicio);
	}
	/**
     * Busca y recupera los datos del factura
     *
     * @param	string	$id ID del factura
     */
	private function get_factura($id=''){
		global $shop;
		//Si el ID está vacío, paramos
		if (empty($id))
			return false;
		//Creamos una variable local para almacenar el objeto del factura
		$obj = new FacturaShop();
		$obj->setid($id);
		$obj = $obj->BuscarId();

		if ($obj->id != ''){
			$obj->loadLineas();
			$obj->datos_formateados = Helpers::toArray(json_decode($obj->datos_facturacion));
			
			$cliente_usr = new ClienteShop();
			$cliente_usr->setid_usuario(Usuario::getLoged()->id);
			$cliente_usr = $cliente_usr->BuscarIdUsuario();
			
			if (!empty($cliente_usr->id)){
				$id_cliente = $cliente_usr->id;
			} else {
				$id_cliente = '-1';
			}
			
			if ($obj->creador_id != Usuario::getLoged()->id && $obj->datos_formateados["cliente_id"] != $id_cliente){
				App::setMsg(__('¡Estás intentando acceder a una factura que no es tuya!'), 'error');
				return false;
			}
			//Si encontramos el factura, almacenamos las variables
			$this->id = $obj->id;
			$this->creador_id = $obj->creador_id;
			
			if (!empty($_GET['download'])){
				$obj->descargar($_GET['download'], true);
			}
			
			$this->objeto = $obj;
			$shop->current_url = $obj->getLink();
			//Recuperamos los parámetros del factura
			$this->get_params();
			//Configuramos la vista del factura
			$this->configurar_vista();
			
			return true;
		} else {
			//Si no encontramos el factura, paramos
			return false;
		}
	}
	/**
     * Incluye los archivos necesarios
     */
	private function includes(){
		//Clase necesaria
		e_include('ext_shop/class/facturaShop');
	}
	/**
     * Recupera el objeto del factura si existiera
     */
	function getFactura(){
		if (!empty($this->objeto))
			return $this->objeto;
		else
			return false;
	}
	/**
     * Configura la vista del factura
     */
	private function configurar_vista(){
		if ($this->objeto instanceof FacturaShop && !empty($this->objeto->id)){
			global $shop;
			$config_shop = $shop->config;
			//if (empty($_GET['estilo']) && trim($config_shop['clientes']['estilo_factura']) != '')
//
//				$_GET['estilo'] = $config_shop['clientes']['estilo_factura'];			
		}
	}
	/**
     * Recupera los parámetros del factura
     */
	private function get_params(){
		//Si no se ha establecido el objeto del factura o no tiene parámetros, paramos
		if (empty($this->objeto) || !isset($this->objeto->params))
			return false;
		global $shop;
		//Configuración global de la extensión Facturas
		$config_facturas = App::getConfigValue('ext_shop');
		//Modelo de los parámetros del factura
		//$params_art = $config_facturas['factura'];
		$params_art = Helpers::toArray(json_decode($this->objeto->params));
		//Parámetros específicos del factura
		$params_art_tmp = Helpers::toArray(json_decode($this->objeto->params));
		//Chequeamos los parámetros específicos del factura
		foreach ($params_art as $kparam=>$param){
			if (array_key_exists($kparam, $params_art_tmp)){
				$params_art[$kparam] = $params_art_tmp[$kparam];
			}
		}
		//Si se ha establecido en la configuración de la extensión Facturas que se ignoren los parámetros individuales de los facturas, sustituimos los parámetros individuales del factura con los parámetros globales de la extensión Facturas
		if ($config_facturas['factura']['ignorar_individuales']=="1") {
			$params_art = $config_facturas['factura'];
		}
		//Si el enlace actual es una vista tipo factura, sustituimos los parámetros individuales del factura con los parámetros del enlace actual
		if (!empty($GLOBALS['enlace_actual']) && ($GLOBALS['enlace_actual']->vista=='factura')) {
			//Chequeamos los parámetros específicos del factura
			foreach ($params_art as $kparam=>$param){
				if (array_key_exists($kparam, $GLOBALS['params_enlace_actual']['params'])){
					$params_art[$kparam] = $GLOBALS['params_enlace_actual']['params'][$kparam];
				}
				
			}
		}
		//Si no se ha establecido un estilo personalizado de la vista, aplicamos el estilo predeterminado de la vista
		if (empty($_GET['estilo']) && trim($shop->config['facturas']['estilo']) != '')
			$_GET['estilo'] = $shop->config['facturas']['estilo'];
		//Si no se ha establecido un estilo personalizado de la vista, chequeamos si en los parámetros del factura se ha establecido un estilo personalizado
		if (trim($params_art['estilo']) != '' && empty($GLOBALS['params_enlace_actual']['estilo']))
			$_GET['estilo'] = $params_art['estilo'];
			
		//Si se ha establecido la vista de impresión, aplicamos el estilo de impresión global
		if ($_GET['print'] == '1')
			$_GET["estilo"] = $GLOBALS["opciones"]["sitio"]["estilo_imprimir"];
			
		if (empty($_GET['estilo']) && !empty($GLOBALS['params_enlace_actual']['estilo']))
			$_GET['estilo'] = $GLOBALS['params_enlace_actual']['estilo'];
		
		//Almacenamos los parámetros
		$this->parametros = $params_art;
		return true;
	}
	/**
     * Returns a property of the object or the default value if the property is not set.
     *
     * @param   string  $property  The name of the property.
     * @param   mixed   $default   The default value.
     *
     * @return  mixed    The value of the property.
     *
     * @since   11.1
     *
     * @see     getProperties()
     */
    public function get($property, $default = null) {
        if (isset($this->$property)) {
            return $this->$property;
        }
        return $default;
    }
    /**
     * Returns a property of the object or the default value if the property is not set.
     *
     * @param   string  $property  The name of the property.
     * @param   mixed   $default   The default value.
     *
     * @return  mixed    The value of the property.
     *
     * @since   11.1
     *
     * @see     getProperties()
     */
    public function set($property, $default = null) {
        $this->$property = $default;
        return $default;
    }
}