<?php
/**
 * Modelo del pedido, permite manejar los pedidos de forma fácil
 *
 * @package		Mideas.Muzzim
 * @subpackage	Model
 * @since		1.0
 */
class PedidoModel {
	/**
     * ID del pedido
     *
     * @var	integer
     * @access  public
     */
	var $id;
	/**
     * Título del pedido
     *
     * @var	string
     * @access  public
     */
	var $usuario_id;
	/**
     * Parámetros del pedido
     *
     * @var	array
     * @access  public
     */
	var $parametros;
	/**
     * Parámetros de la vista de pedido
     *
     * @var	array
     * @access  public
     */
	var $vista;
	/**
     * Objeto del pedido
     *
     * @var	object
     * @access  public
     */
	var $objeto;
		
	/**
     * Constructor de la clase
     *
     * @param	string	$id ID del pedido
     */
	function __construct($id=''){
		//Incluimos los archivos necesarios
		self::includes();
		
		if (!empty($_GET['id'])){
			$id = $_GET['id'];
		}
		
		//Si se establece un ID, se intenta localizar el pedido
		if (!empty($id)){
			$this->get_pedido($id);
		}
	}
	
	public function getTitulo(){
		return sprintf(__('Detalles del Pedido #%s'), $this->objeto->id);
	}
	
	/**
     * Busca y recupera los datos del pedido
     *
     * @param	string	$id ID del pedido
     */
	private function get_pedido($id=''){
		global $shop;
		
		//Si el ID está vacío, paramos
		if (empty($id))
			return false;
			
		//Creamos una variable local para almacenar el objeto del pedido
		$obj = new PedidoShop();
		$obj->setid($id);
		$obj = $obj->BuscarId();
		if ($obj->id != ''){
			if ($obj->usuario_id != Usuario::isLoged()){
				App::setMsg(__('Estás intentando acceder a un pedido que no es tuyo!'), 'error');
				return false;
			}
			//Si encontramos el pedido, almacenamos las variables
			$this->id = $obj->id;
			$this->usuario_id = $obj->usuario_id;
			$this->objeto = $obj;
			
			$shop->current_url = $obj->getLink();
						
			//Recuperamos los parámetros del pedido
			$this->get_params();
			
			//Configuramos la vista del pedido
			$this->configurar_vista();
			
			return true;
		} else {
			//Si no encontramos el pedido, paramos
			return false;
		}
	}
		
	/**
     * Incluye los archivos necesarios
     */
	private function includes(){
		//Clase necesaria
		e_include('ext_shop/class/pedidoShop');
	}
	
	/**
     * Recupera el objeto del pedido si existiera
     */
	function getPedido(){
		if (!empty($this->objeto))
			return $this->objeto;
		else
			return false;
	}
	
	/**
     * Configura la vista del pedido
     */
	private function configurar_vista(){
		if ($this->objeto instanceof PedidoShop && !empty($this->objeto->id)){
			global $shop;
			
			$config_shop = $shop->config;
			
			if (empty($_GET['estilo']) && trim($config_shop['clientes']['estilo_pedido']) != '')
				$_GET['estilo'] = $config_shop['clientes']['estilo_pedido'];			
		}
	}
	
	/**
     * Recupera los parámetros del pedido
     */
	private function get_params(){
		//Si no se ha establecido el objeto del pedido o no tiene parámetros, paramos
		if (empty($this->objeto) || !isset($this->objeto->params))
			return false;
			
		global $shop;
		
		//Configuración global de la extensión Pedidos
		$config_pedidos = App::getConfigValue('ext_shop');
		
		//Modelo de los parámetros del pedido
		//$params_art = $config_pedidos['pedido'];
		$params_art = Helpers::toArray(json_decode($this->objeto->params));
		
		
		//Parámetros específicos del pedido
		$params_art_tmp = Helpers::toArray(json_decode($this->objeto->params));
		
		//Chequeamos los parámetros específicos del pedido
		foreach ($params_art as $kparam=>$param){
			if (array_key_exists($kparam, $params_art_tmp)){
				$params_art[$kparam] = $params_art_tmp[$kparam];
			}
			
		}
		//Si se ha establecido en la configuración de la extensión Pedidos que se ignoren los parámetros individuales de los pedidos, sustituimos los parámetros individuales del pedido con los parámetros globales de la extensión Pedidos
		if ($config_pedidos['pedido']['ignorar_individuales']=="1") {
			$params_art = $config_pedidos['pedido'];
		}
		
		//Si el enlace actual es una vista tipo pedido, sustituimos los parámetros individuales del pedido con los parámetros del enlace actual
		if (!empty($GLOBALS['enlace_actual']) && ($GLOBALS['enlace_actual']->vista=='pedido')) {
			//Chequeamos los parámetros específicos del pedido
			foreach ($params_art as $kparam=>$param){
				if (array_key_exists($kparam, $GLOBALS['params_enlace_actual']['params'])){
					$params_art[$kparam] = $GLOBALS['params_enlace_actual']['params'][$kparam];
				}
				
			}
		}
		
		//Si no se ha establecido un estilo personalizado de la vista, aplicamos el estilo predeterminado de la vista
		if (empty($_GET['estilo']) && trim($shop->config['pedidos']['estilo']) != '')
			$_GET['estilo'] = $shop->config['pedidos']['estilo'];
		
		//Si no se ha establecido un estilo personalizado de la vista, chequeamos si en los parámetros del pedido se ha establecido un estilo personalizado
		if (trim($params_art['estilo']) != '' && empty($GLOBALS['params_enlace_actual']['estilo']))
			$_GET['estilo'] = $params_art['estilo'];
			
		//Almacenamos los parámetros
		$this->parametros = $params_art;
			
		return true;
	}
	
	/**
     * Returns a property of the object or the default value if the property is not set.
     *
     * @param   string  $property  The name of the property.
     * @param   mixed   $default   The default value.
     *
     * @return  mixed    The value of the property.
     *
     * @since   11.1
     *
     * @see     getProperties()
     */
    public function get($property, $default = null) {
        if (isset($this->$property)) {
            return $this->$property;
        }
        return $default;
    }
    /**
     * Returns a property of the object or the default value if the property is not set.
     *
     * @param   string  $property  The name of the property.
     * @param   mixed   $default   The default value.
     *
     * @return  mixed    The value of the property.
     *
     * @since   11.1
     *
     * @see     getProperties()
     */
    public function set($property, $default = null) {
        $this->$property = $default;
        return $default;
    }
}