<?php
/**
 * Modelo del producto, permite manejar los productos de forma fácil
 *
 * @package		Mideas.Muzzim
 * @subpackage	Model
 * @since		1.0
 */
class ProductoModel extends DetalleModel {
	
	protected $global_main_class = 'shop';
	
	protected $ext_folder = 'ext_shop';
	
	protected $categoria_class = 'CategoriaProducto';
	protected $item_class = 'Producto';
	
	protected $categoria_vista = 'categoria_productos';
	protected $item_vista = 'producto';
	
	function __construct($id=''){
		parent::__construct($id);
		
		if ($this->objeto instanceof Producto){
			//Recuperamos las variaciones del producto
			$this->variaciones = $this->objeto->getVariaciones();
			
			//Recuperamos los campos de información del producto
			$this->getCamposInfo();
			
			//Recuperamos las variaciones que afectan a los atributos del producto
			$this->getVariacionesAtributos();
			
			//Recuperamos el stock del producto
			$this->getStock();
			
			$this->configurar_vista();
		}
	}
	
	public function includes(){
		c_include('enlace');
		e_include('ext_shop/class/producto','ext_shop/class/categoriaProducto');
	}
	private function getStock(){
		if (empty($this->variaciones_atributos)){
			$this->stock = $this->objeto->stock;
		} else {
			$tmp_stock=0;
			foreach ($this->variaciones_atributos as $varia){
				$tmp_stock += $varia->stock;
			}
			$this->stock = $tmp_stock;
		}
	}
	
	private function getCamposInfo(){
		if (empty($this->objeto))
			return false;
			
		if (empty($this->objeto->campos_extra))
			$this->objeto->campos_extra = $this->objeto->getCamposExtra();
			
		$campos_info = array();
		
		if (!empty($this->objeto->campos_extra) && is_array($this->objeto->campos_extra)){
			foreach ($this->objeto->campos_extra as $ce){
				if ($ce->visible == 1){
					$tmp_arr = array("id"=>$ce->id,"name"=>$ce->nombre, "label"=>SistemaIdioma::Imprimir($ce->etiqueta),"value"=>$ce->printValue(false));
					$campos_info[] = $tmp_arr;
				}
			}
		}
		
		$this->campos_info = $campos_info;
		
		return true;
	}
	
	private function getVariacionesAtributos(){
		if (empty($this->objeto))
			return false;
			
		$variaciones_atributos = $this->objeto->getProductoVariaciones();
		
		$this->variaciones_atributos = $variaciones_atributos;
		
		return true;
	}
	
	public function printVariacionesAtributosJS(){
		if (empty($this->variaciones_atributos))
			$this->getVariacionesAtributos();
			
		if (empty($this->variaciones_atributos))
			return false;
			
		$arr = array();
		
		$producto_padre = $this->objeto;
		
		foreach ($this->variaciones_atributos as $var_at){
			$atr_var = Helpers::toArray(json_decode($var_at->atributos));
			$tmp_arr = array();
			
			$tmp_arr["variation_id"] = $var_at->id;
			$tmp_arr["atributos"] = $atr_var;
			$tmp_arr["image_src"] = $var_at->getimagen_destacada(300,300);
			$tmp_arr["image_link"] = $var_at->imagen_destacada;
			$tmp_arr["image_title"] = SistemaIdioma::Imprimir($producto_padre->nombre);
			$tmp_arr["price_html"] = $var_at->getprecio_html(true);
			$tmp_arr["disponibilidad_html"] = $var_at->getstock(true);
			$tmp_arr["sku"] = $var_at->getsku(true);
			
			$arr[] = $tmp_arr;
		}
		
		$out = json_encode($arr);
		
		return $out;
		
	}
	
	/**
     * Recupera el objeto del producto si existiera
     */
	function getProducto(){
		if (!empty($this->objeto))
			return $this->objeto;
		else
			return false;
	}
	
	/**
     * Configura la vista del producto
     */
	private function configurar_vista(){
		if ($this->objeto instanceof Producto && !empty($this->objeto->id)){
			global $shop;
			
			$config_shop = $shop->config;
			
			if ($config_shop['impuestos']['activar_impuestos'] == '1'){
				if ($config_shop['impuestos']['impuestos_incluidos'] == '1'){
					$this->vista->precio_producto = $this->objeto->getprecio(true);
					$this->vista->impuesto_producto = 0;
					
					if ($this->objeto->getprecio_rebajado(true) > 0){
						$this->vista->precio_producto_antes = $this->objeto->getprecio();
					} else {
						$this->vista->precio_producto_antes = 0;
					}
					
				} else {
					$this->vista->precio_producto = $this->objeto->getprecio(true) * 1.18;
					$this->vista->impuesto_producto = $this->objeto->getprecio(true) * 0.18;
					
					if ($this->objeto->getprecio_rebajado(true) > 0){
						$this->vista->precio_producto_antes = $this->objeto->getprecio() * 1.18;
					} else {
						$this->vista->precio_producto_antes = 0;
					}
				}
			} else {
				$this->vista->precio_producto = $this->objeto->getprecio(true);
				$this->vista->impuesto_producto = 0;
				
				if ($this->objeto->getprecio_rebajado(true) > 0){
					$this->vista->precio_producto_antes = $this->objeto->getprecio();
				} else {
					$this->vista->precio_producto_antes = 0;
				}
			}
			$params_obj = Helpers::toArray(json_decode($this->objeto->params));
			
			$this->vista->productos_relacionados = array();
			if (isset($params_obj['productos_relacionados']) && !empty($params_obj['productos_relacionados']) && $this->parametros['mostrar_relacionados']=='1'){
				foreach ($params_obj['productos_relacionados'] as $pr){
					$producto_relacionado = new Producto();
					$producto_relacionado->setid($pr);
					if ($producto_relacionado = $producto_relacionado->BuscarId()){
						if ($producto_relacionado->estado == '1'){
							$this->vista->productos_relacionados[] = $producto_relacionado;
						}
					}
				}
				
				array_slice($this->vista->productos_relacionados, 0, (int) $this->parametros['num_relacionados']);
				reset($this->vista->productos_relacionados);
			}
			
			if (empty($_GET['estilo']) && trim($this->parametros['estilo']) != '')
				$_GET['estilo'] = $this->parametros['estilo'];
				
			$this->vista->categoria_link = Helpers::resolveLink(false,'ext_shop','categoria_productos',__('Tienda'),$GLOBALS['opciones']['ext_shop']['shop']['categoria_defecto'][0]);
		
			$this->vista->categorias_html = '';
			if ($this->parametros['mostrar_categorias'] == '1'){
				$categorias = new CategoriaProducto();
				$categorias = $categorias->Listar('estado=1 AND id<>1', 'nombre', 'ASC');
				
				foreach ($categorias as $kc=>$c){
					$categorias[$kc]->link = $c->getLink();
				}
				
				$this->vista->categorias_html = Jerarquia::getAllChilds($categorias, 1);
			}
			
			$this->vista->productos_masvendidos = array();
			if ($this->parametros['mostrar_masvendidos'] == '1'){
				$this->vista->productos_masvendidos = new Producto();
				
				$this->vista->productos_masvendidos = $this->vista->productos_masvendidos->Listar("estado=1 AND fecha_publicacion < '".strftime("%Y-%m-%d %H:%M:%S", time())."'", 'fecha_publicacion', 'DESC');
				
				if ($this->vista->productos_masvendidos == false)
					$this->vista->productos_masvendidos = array();
			}
			
		}
	}
	
	function getdownload_link($false_return){
		if (empty($false_return))
			$false_return = 'javascript:;';
			
		$params_pro = Helpers::toArray(json_decode($this->objeto->params));
		
		if ($params_pro['virtual'] != '1' || $params_pro['tipo_virtual'] != 'descargable')
			return $false_return;
			
		c_include('usuario');
			
		if ($params_pro['solo_registrados_descargable_virtual'] == '1' && !Usuario::isLoged())
			return $false_return;
			
		//Falta comprobación: Si el producto es de pago y no lo ha comprado el usuario activo, debe devolverle false
		
		$link_download_producto = $this->objeto->getLink();
		
		if ($GLOBALS['opciones']['metadatos']['urls_amigables_metadatos'] == '1'){
			$link_download_producto .= '/?descargar=1';
		} else {
			$link_download_producto .= '&descargar=1';
		}
		
		return $link_download_producto;
	}
}