<?php    
/**
 * Clase Envio
 * 
 * Maneja los envíos
 *
 * @class 		Envio
 * @package		Shop
 * @category	Envío
 * @author		Mideas
 */  
defined('CONF_URL_APP') or die;
class ReglasPreciosModel {
	
	var $activo			= false;
	var $reglas_precios 	= array();
	var $reglas_aplicables		= array();
	var $cupones_aplicables		= array();
	var $descuento_total 	= 0;
    /**
     * init function.
     * 
     * Init the envio - load methods and reorder based on settings.
     *
     * @access public
     */
    function init() {
		
		$config_shop = $GLOBALS['opciones']['ext_shop'];
		
		$this->activo = ( $config_shop['reglas']['activar_reglas'] == '0' ) ? false : true;
		
		if ($this->activo){
			$this->get_reglas_aplicables();
			$this->get_cupones_aplicables();
		}
		
	}
	
	private function get_reglas_aplicables(){
		$todas_reglas = $this->get_reglas_activas();
		
		$reglas_aplicables = array();
		
		if (is_array($todas_reglas) && sizeof($todas_reglas) > 0){
			foreach ($todas_reglas as $reg){
				if ($reg->cumpleCondicion()){
					$reglas_aplicables[$reg->id] = $reg;
				}
			}
		}
		
		$this->reglas_aplicables = $reglas_aplicables;
		
		$this->setSession();
		
		return $this->reglas_aplicables;
		
	}
	
	private function get_cupones_aplicables(){
		$todas_reglas = $this->get_reglas_activas();
		
		$cupones_aplicables = array();
		
		if (is_array($todas_reglas) && sizeof($todas_reglas) > 0){
			foreach ($todas_reglas as $reg){
				if ($reg->esCupon()){
					$cupones_aplicables[$reg->id] = $reg;
				}
			}
		}
		
		$this->cupones_aplicables = $cupones_aplicables;
		
		$this->setSession();
		
		return $this->cupones_aplicables;
		
	}
	
	private function setSession(){
		$arr_reglas = array();
		
		if (is_array($this->reglas_aplicables) && sizeof($this->reglas_aplicables) > 0){
			
			foreach ($this->reglas_aplicables as $reg){
				$cond = (array) json_decode($reg->condiciones);
				
				$conds = array();
				
				if (sizeof($cond) > 0){
					foreach ($cond as $kc=>$c){
						$cond[$kc] = (array) $c;
						
						$exp_tmp = explode(".",$cond[$kc]["campo_condiciones"]);
					
						$grupo_cond = $exp_tmp[0];
						$id_cond = $exp_tmp[1];
						$operacion_condicion = $cond[$kc]["campo_operacion"];
						$valor_condicion = $cond[$kc]["valor_condiciones"];
						
						$conds[$kc] = array("grupo"=>$grupo_cond,"id"=>$id_cond, "operacion"=>$operacion_condicion, "valor"=>$valor_condicion);
						
					}
				}
				
				$acc = (array) json_decode($reg->accion_regla);
				
				if (is_array($acc) && sizeof($acc) == 3){
					$tipo_accion = $acc["tipo_accion"];
					$cantidad_descuento_accion = $acc["cantidad_descuento_accion"];
					$envio_gratis_accion = $acc["envio_gratis_accion"];
				}
				
				$reg_array = array("id"=>$reg->id,"nombre"=>SistemaIdioma::Imprimir($reg->titulo),"descripcion"=>$reg->contenido_formateado,"estado"=>$reg->estado,"fecha_inicio"=>$reg->fecha_inicio,"fecha_fin"=>$reg->fecha_fin, "condiciones"=>$conds, "accion"=>array("tipo"=>$tipo_accion, "cantidad_descuento"=>$cantidad_descuento_accion, "envio_gratis"=>$envio_gratis_accion));
				
				$arr_reglas[$reg->id] = $reg_array;
			}
		}
		
		Helpers::getSesion()->set('shop.reglas', $arr_reglas);
		
	}
	
	private function get_reglas_activas(){
		e_include('ext_shop/class/reglaPrecio');
		
		if (is_array($this->reglas_precios) && sizeof($this->reglas_precios) > 0){
			return $this->reglas_precios;
		}
		
		$todas_reglas = new ReglaPrecio();
		
		$todas_reglas = $todas_reglas->Listar("estado=1","fecha","DESC");
		
		$reglas_parsed = array();
		
		if (is_array($todas_reglas) && sizeof($todas_reglas) > 0){
			foreach ($todas_reglas as $reg){
				if (empty($reg->fecha_inicio) || strtotime($reg->fecha_inicio) < time()){
					if (empty($reg->fecha_fin) || strtotime($reg->fecha_fin) > time()){
						if (empty($reg->limite_usos) || $reg->limite_usos >= $reg->usado){
							$reglas_parsed[$reg->id] = $reg;
						}
					}
				}
			}
		}
		
		$this->reglas_precios = $reglas_parsed;
		
		return $this->reglas_precios;
		
	}
	/**
	 * calcular_envio function.
	 *
	 * Calculate envio for (multiple) paquetes of cart items. 
	 * 
	 * @access public
	 * @param array $paquetes multi-dimentional array of cart items to calc envio for
	 */
	function calcular_envio( $paquetes = array() ) {
		if ( ! $this->activo ) return;
		if ( empty( $paquetes ) ) return;
		
		$this->descuento_total 	= 0;
		$this->envio_impuestos 	= array();
		$this->envio_nombre 	= null;
		$this->paquetes 		= array();
		$_coste_mas_barato = $_metodo_mas_barato = $reglas_aplicables = '';
		
		$_metodo_eleg = Helpers::getSesion()->get('shop._metodo_envio_elegido');
		
		if ( !empty( $_metodo_eleg ) ) 
			$reglas_aplicables = Helpers::getSesion()->get('shop._metodo_envio_elegido');
				
		// Calculate costes for passed paquetes
		$paquete_keys 		= array_keys( $paquetes );
		$paquete_keys_size 	= sizeof( $paquete_keys );
		
		for ( $i=0; $i < $paquete_keys_size; $i++ ) 
			$this->paquetes[$paquete_keys[$i]] = $this->calcular_envio_por_paquete( $paquetes[$paquete_keys[$i]] );
			
		// Get available methods (in this case methods for all paquetes)
		$_metodos_disponibles = $this->get_reglas_precios_disponibles();
		
		if ( sizeof( $_metodos_disponibles ) > 0 ) {
			
			// If not set, set a default
			//if ( empty( $reglas_aplicables ) || ! isset( $_metodos_disponibles[$reglas_aplicables] ) ) {
				
				//$reglas_aplicables = get_option('woocommerce_metodo_envio_defecto');
				
				//if ( empty( $reglas_aplicables ) || ! isset( $_metodos_disponibles[$reglas_aplicables] ) ) {
				
					// Default to cheapest
					foreach ( $_metodos_disponibles as $method_id => $method ) {
						if ( $method->coste < $_coste_mas_barato || ! is_numeric( $_coste_mas_barato ) ) {
							$_coste_mas_barato 	= $method->coste;
							$_metodo_mas_barato 	= $method_id;
						}
					}
					
					$reglas_aplicables = $_metodo_mas_barato;
				//}
			
			//}
			
			if ( $reglas_aplicables ) {
				Helpers::getSesion()->set('shop._metodo_envio_elegido', $reglas_aplicables);
				$this->descuento_total 	= $_metodos_disponibles[$reglas_aplicables]->coste;
				$this->envio_impuestos 	= $_metodos_disponibles[$reglas_aplicables]->impuestos;
				$this->envio_nombre 	= $_metodos_disponibles[$reglas_aplicables]->nombre;
			}
		}
	}
	/**
	 * calcular_envio_por_paquete function.
	 *
	 * Calculates each envio methods coste. Rates are cached based on the paquete to speed up calculations.
	 * 
	 * @access public
	 * @param array $paquete cart items
	 */
	function calcular_envio_por_paquete( $paquete = array() ) {
		if ( ! $this->activo ) return false;
		if ( ! $paquete ) return false;
		
			
		// Calculate envio method tarifas
		$paquete['tarifas'] = array();
		
		foreach ( $this->reglas_precios as $metodo_envio ) {
				
			if ( $metodo_envio->is_available( $paquete ) ) {
			
				// Reset Rates
				$metodo_envio->tarifas = array();
				
				// Calculate Shipping for paquete
				$metodo_envio->calcular_envio( $paquete );
				
				// Place tarifas in paquete array
				if ( ! empty( $metodo_envio->tarifas ) && is_array( $metodo_envio->tarifas ) )
					foreach ( $metodo_envio->tarifas as $tarifa ) 
						$paquete['tarifas'][$tarifa->id] = $tarifa;
			}
			
		}
		
		return $paquete;
	}
	
	/**
	 * get_reglas_precios_disponibles function.
	 *
	 * Gets all available envio methods which have tarifas. 
	 *
	 * @todo Currently we support 1 envio method per order so this function merges tarifas - in the future we should offer
	 * 1 tarifa per paquete and list them accordinly for user selection
	 * 
	 * @access public
	 * @return array
	 */
	function get_reglas_precios_disponibles() {
		if ( ! $this->activo ) return;
		if ( empty( $this->paquetes ) ) return;
		
		// Loop paquetes and merge tarifas to get a total for each envio method
		$metodos_disponibles = array();
		foreach ( $this->paquetes as $paquete ) {
			if ( ! $paquete['tarifas'] ) continue;
			
			foreach ( $paquete['tarifas'] as $id => $tarifa ) {
				
				if ( isset( $metodos_disponibles[$id] ) ) {
					// Merge coste and impuestos - label and ID will be the same
					$metodos_disponibles[$id]->coste += $tarifa->coste;
					
					foreach ( array_keys( $metodos_disponibles[$id]->impuestos + $tarifa->impuestos ) as $key ) {
					    $metodos_disponibles[$id]->impuestos[$key] = ( isset( $tarifa->impuestos[$key] ) ? $tarifa->impuestos[$key] : 0 ) + ( isset( $metodos_disponibles[$id]->impuestos[$key] ) ? $metodos_disponibles[$id]->impuestos[$key] : 0 );
					}
				} else {
					$metodos_disponibles[$id] = new TarifaEnvioShop( $tarifa->id, $tarifa->nombre, $tarifa->coste, $tarifa->impuestos );
				}
								
			}
			
		}
				
		return $metodos_disponibles;
	}
	/**
	 * reset_envio function.
	 * 
	 * Reset the totals for envio as a whole.
	 *
	 * @access public
	 * @return void
	 */
	function reset_envio() {
		Helpers::getSesion()->clear('shop._metodo_envio_elegido');
		$this->descuento_total = 0;
		$this->envio_impuestos = array();
		$this->envio_nombre = null;
		$this->paquetes = array();
	}
		
}